const mdpCarterElementor = {
    carterElementor: function ( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-carter-elementor-box` );
        const cartAutoUpdate = $box.dataset.cartAutoUpdate;
        const widgetId = $box.dataset.widgetId;

        this.setButtonsAnimations( wrapperName, $box );

        this.stickyRightColumn( wrapperName, $box )

        if ( cartAutoUpdate === 'yes' ) {
            this.autoUpdateCart( wrapperName );
        }

        // disable actions in editor
        if ( elementorFrontend.isEditMode() === true ) {
            this.disableActions( wrapperName );
        }

        // update wp referrers
        this.updateReferrers( wrapperName, widgetId );

    },

    updateReferrers: function( wrapperName, widgetId ) {
        const wpReferrers = document.querySelector( `.${wrapperName} [name=_wp_http_referer]` );
        const url = new URL ( document.location );
        url.searchParams.set( 'elementorPageId', elementorFrontend.config.post.id );
        url.searchParams.set( 'elementorWidgetId', widgetId );
        if ( wpReferrers ) {
            wpReferrers.setAttribute( 'value', url );
        }
    },

    disableActions: function ( wrapperName ) {
        const updateCartBtn = document.querySelector( `.${wrapperName} button[name=update_cart]` );
        const productRemoveBtn = document.querySelector( `.${wrapperName} .product-remove a` );
        const quantityInput = document.querySelector( `.${wrapperName} .qty` );
        const couponBtn = document.querySelector( `.${wrapperName} button[name=apply_coupon]` );

        // disable update cart button
        if ( updateCartBtn ) {
            updateCartBtn.setAttribute( 'disabled', 'disabled' );
            updateCartBtn.setAttribute( 'aria-disabled', 'true' );
        }

        // disable product remove button
        if ( productRemoveBtn ) {
            productRemoveBtn.style.pointerEvents = 'none';
        }

        // disable quantity button
        if ( quantityInput ) {
            quantityInput.setAttribute( 'disabled', 'disabled' );
        }

        // disable coupon button
        if ( couponBtn ) {
            couponBtn.setAttribute( 'disabled', 'disabled' );
        }

    },

    setButtonsAnimations: function ( wrapperName, $box ) {
        const updateCartBtn = document.querySelector( `.${wrapperName} button[name=update_cart]` );
        const updateShippingBtn = document.querySelector( `.${wrapperName} button[name=calc_shipping]` );
        const updateCartAnimationClass = `elementor-animation-${$box.dataset.updateCartAnimation}`;
        const updateShippingAnimationClass = `elementor-animation-${$box.dataset.updateShippingAnimation}`;

        if ( updateCartBtn ) {
            updateCartBtn.classList.add(updateCartAnimationClass);
        }

        if ( updateShippingBtn ) {
            updateShippingBtn.classList.add( updateShippingAnimationClass );
        }


    },

    stickyRightColumn: function ( wrapperName, $box ) {

        const $cartContentContainer = document.querySelector( `.${wrapperName} .mdp-carter-elementor-cart-container` );
        if ( !$cartContentContainer ) { return; }

        const $productTable = document.querySelector( `.${wrapperName} .mdp-carter-elementor-cart-product-table` );
        const containerHeight = $cartContentContainer.offsetHeight;
        const layout = $box.dataset.cartLayout;
        const stickyRightColumn = $box.dataset.stickyRightColumn;
        const $totalSection = document.querySelector( `.${wrapperName} .mdp-carter-elementor-cart-product-total-content` );
        const totalSectionWidth = $totalSection.getBoundingClientRect().width;
        const productTableHeight = $productTable ? $productTable.getBoundingClientRect().height : 0;

        const scrollListener = () => {
            const containerOffset = $cartContentContainer.getBoundingClientRect().top;
            const startPosition = 0;
            let isNotEqualHeight;

            $totalSection.getBoundingClientRect().height

            if ( $totalSection.getBoundingClientRect.height !== productTableHeight ) {
                isNotEqualHeight = true;
            }

            if ( isNotEqualHeight ) {
                if ( containerOffset <= startPosition ) {
                    if ( $cartContentContainer.getBoundingClientRect().bottom >= $totalSection.getBoundingClientRect().height ) {
                        $totalSection.classList.add( 'mdp-carter-elementor-total-sticky' );
                        $totalSection.style.width = Math.floor( totalSectionWidth ) + 'px';
                        $totalSection.style.removeProperty( 'margin-top' );

                    } else {
                        $totalSection.classList.remove( 'mdp-carter-elementor-total-sticky' );
                        $totalSection.style.removeProperty( 'width' );
                        $totalSection.style.marginTop = ( containerHeight - $totalSection.getBoundingClientRect().height ) + 'px';
                    }
                } else {
                    $totalSection.classList.remove( 'mdp-carter-elementor-total-sticky' );
                    $totalSection.style.removeProperty( 'width' );
                    $totalSection.style.removeProperty( 'margin-top' );
                }
            }
        };

        if ( stickyRightColumn === 'yes' ) {
            window.addEventListener( 'scroll', scrollListener );
        }

        // remove if one column layout
        if ( layout === 'one-column' ) { window.removeEventListener( 'scroll', scrollListener ) }
    },

    autoUpdateCart: function ( wrapperName ) {
        let timeout;
        jQuery(`.${wrapperName} .woocommerce`).on('input', 'input.qty', function(){

            if ( timeout !== undefined ) {
                clearTimeout( timeout );
            }

            timeout = setTimeout(function() {
                jQuery( `.${wrapperName} button[name=update_cart]` ).trigger( "click" );
            }, 1000 );

        });
    }
 }

/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-carter-elementor' ) ) {
            mdpCarterElementor.carterElementor.call( mdpCarterElementor, `elementor-element-${$scope[0].dataset.id}` );
        }
    } );

} );
