"use strict";

/**
 * Mdp Countdowner main object
 * @type {{countdownerMain: mdpCountdowner.countdownerMain, expireAction: mdpCountdowner.expireAction, addCountdowner: mdpCountdowner.addCountdowner}}
 */

const mdpCountdowner = {
    intervals: [],

    countdownerMain: function ( wrapperName ) {
        const countdownType = document.querySelector( `.${wrapperName}` ).dataset.countdownType;
        const animationType = document.querySelector( `.${wrapperName}` ).dataset.animation;
        const time = document.querySelector( `.${wrapperName}` ).dataset.time;
        const timeStart = document.querySelector( `.${wrapperName}` ).dataset.start;
        const expireAction = document.querySelector( `.${wrapperName}` ).dataset.expireAction;
        const timezone = document.querySelector( `.${wrapperName}` ).dataset.timezone;
        const countdownView = document.querySelector( `.${wrapperName}` ).dataset.countdownView;
        let timeCount = null;

        let units = document.querySelector( `.${wrapperName}` ).dataset.units;
        let unitsArray = units.split( ',' );

        // calculate difference between two dates
        function getDateTimeDiff( todayDate, futureDate, intervals ) {

            let out = {};

            for ( let i = 0; i < intervals.length; i++ ) {
                let diff = futureDate.diff( todayDate, intervals[i] );
                todayDate.add( diff, intervals[i] );
                out[ intervals[i] ] = diff;
            }

            return out;
        }

        function getWpTimeArray() {
            let current_wp_time_array = wpGlobalTime.split( ' ' ).map( element => {
                return parseInt( element, 10 );
            } );

            current_wp_time_array[1] -= 1;

            return current_wp_time_array;
        }

        let todayDiff = '';

        let todayDiffWp = '';

        if ( countdownType === 'evergreen-timer' ) {

            const secondsOffset = document.querySelector( `.${wrapperName}` ).dataset.evergreenSeconds;

            todayDiff = moment().add( secondsOffset , 'seconds' );

            todayDiffWp = moment( getWpTimeArray() ).add( secondsOffset, 'seconds' );

            if ( localStorage.getItem( wrapperName + '_evergreen-time-interval' ) === null || localStorage.getItem( wrapperName + '_evergreen-time-interval' ) !== secondsOffset ) {
                localStorage.setItem( wrapperName + '_evergreen-time-interval', secondsOffset );
                localStorage.setItem( wrapperName + '_evergreen-time-due-date', todayDiff );
                localStorage.setItem( wrapperName + '_evergreen-wp-time-due-date', todayDiffWp );
            }
        }

        // change html inside countdown values used for animations
        function changeHtml( selector, value, animation ) {
            const contElem = document.querySelector( selector );
            const showClassName = 'mdp-countdowner-elementor--wrapper-item-show-' + animation;
            const hideClassName = 'mdp-countdowner-elementor--wrapper-item-hide-' + animation;
            contElem.querySelector( `.${showClassName}` ).className = hideClassName;

            const span = document.createElement( 'span' );
            span.className = showClassName;
            span.innerText = value;
            contElem.appendChild( span );
            contElem.querySelector( `.${hideClassName}` ).addEventListener( 'animationend', () => {
                contElem.querySelector( `.${hideClassName}` ).remove();
            } );
        }

        function circleAnimation( unit, mult, resultDate ) {
            const circle = document.querySelector( `.${wrapperName} .mdp-countdowner-circle-animation-${unit}` );
            let unitResult = resultDate[unit] * mult;
            if ( unit === 'years' && mult === 0 ) {
                unitResult = 0;
            }
            circle.style.strokeDashoffset = getFullCircle() - unitResult;
            if ( resultDate[unit] < 0 ) {
                circle.style.strokeDashoffset = getFullCircle();
            }
        }

        function flipAnimation( type, data ) {
            const digit = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-digit-${type}` );
            let card = digit.querySelector( '.mdp-countdowner-elementor--wrapper-digit-card' );

            if ( data < 0 || isNaN( data ) ) data = 0;

            let next = data - 1;

            if ( next < 0 ) next = data;

            if ( !digit.dataset.digitBefore || !digit.dataset.digitAfter ) {
                digit.dataset.digitBefore = data < 10 ? '0' + data : data + '';
                digit.querySelector( '.mdp-countdowner-elementor--wrapper-digit-face-front' ).textContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-digit-${type}` ).dataset.digitBefore;

                digit.dataset.digitAfter = next < 10 ? '0' + next : next + '';
                digit.querySelector( '.mdp-countdowner-elementor--wrapper-digit-face-back' ).textContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-digit-${type}` ).dataset.digitAfter;

            }

            if( data !== +document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-digit-face-front-${type}` ).textContent ) {

                card.addEventListener( 'transitionend', () => {
                    digit.dataset.digitBefore = data < 10 ? '0' + data : data + '';
                    digit.querySelector( '.mdp-countdowner-elementor--wrapper-digit-face-front' ).textContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-digit-${type}` ).dataset.digitBefore;

                    const cardClone = card.cloneNode( true );
                    cardClone.classList.remove( 'flipped' );

                    card.replaceWith( cardClone );

                    card = cardClone;

                    digit.dataset.digitAfter = next < 10 ? '0' + next : next + '';
                    digit.querySelector( '.mdp-countdowner-elementor--wrapper-digit-face-back' ).textContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-digit-${type}` ).dataset.digitAfter;

                }, { once: true } );

                if ( !card.classList.contains( 'flipped' ) ) {
                    card.classList.add( 'flipped' );
                }
            }

        }

        // setting styles for roll digits animation
        if ( animationType === 'roll-digits' ) {
            const items = document.querySelectorAll( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-val` );
            const parentItems = document.querySelectorAll( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item` );

            items.forEach( item => {
                item.style.height = '1.5em';
                item.style.overflow = 'hidden';
                if ( countdownView === 'inline-block' ) {
                    item.style.width = '2.3em';
                }
            } );

            parentItems.forEach( parentItem => {
                parentItem.style.height = '100%';
                if ( countdownView === 'inline-block' ) {
                    parentItem.style.display = 'flex';
                } else {
                    parentItem.style.display = 'block';
                }
            } );
        }

        function getResultDate() {
            let result = getDateTimeDiff( moment( new Date() ), moment ( new Date( time ) ), unitsArray );

            if ( timezone === 'wordpress_default' ) {

                result = getDateTimeDiff( moment( getWpTimeArray() ), moment( new Date( time ) ), unitsArray );
            }

            if ( countdownType === 'evergreen-timer' ) {

                if ( timezone === 'wordpress_default' ) {

                    result = getDateTimeDiff( moment( getWpTimeArray() ), moment( new Date( localStorage.getItem( wrapperName + '_evergreen-wp-time-due-date' ) )  ), unitsArray );
                }

                result = getDateTimeDiff( moment( new Date() ) , moment( new Date( localStorage.getItem( wrapperName + '_evergreen-time-due-date' ) )  ), unitsArray );
            }

            return result;
        }

        function getFullCircle() {
            const circleObject = document.querySelector( `.${wrapperName} .mdp-countdowner-circle-animation` ).r;

            return 2 * Math.PI * circleObject.baseVal.value;
        }

        // fill initial values for animation
        if ( animationType === 'glowing' || animationType === 'fade' || animationType === 'roll-digits' ) {

            const result = getResultDate();

            const className = 'mdp-countdowner-elementor--wrapper-item-show-' + animationType;

            const seconds = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-seconds-val` );
            const minutes = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-minutes-val` );
            const hours = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-hours-val` );
            const days = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-days-val` );
            const weeks = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-weeks-val` );
            const month = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-month-val` );
            const year = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-year-val` );

            if ( seconds !== null ) {
                seconds.innerHTML = `<span class='${className}'>`+( result.seconds <= 0 || isNaN( result.seconds ) ? '00' : result.seconds < 10 ? '0' + result.seconds : result.seconds + '' )+"</span>";
            }

            if ( minutes !== null ) {
                minutes.innerHTML = `<span class='${className}'>`+( result.minutes <= 0 || isNaN( result.minutes ) ? '00' : result.minutes < 10 ? '0' + result.minutes : result.minutes + '' )+"</span>";
            }

            if ( hours !== null ) {
                hours.innerHTML = `<span class='${className}'>`+( result.hours <= 0 || isNaN( result.hours ) ? '00' : result.hours < 10 ? '0' + result.hours : result.hours + '' )+"</span>";
            }

            if ( days !== null ) {
                days.innerHTML = `<span class='${className}'>`+( result.days <= 0 || isNaN( result.days ) ? '00' : result.days < 10 ? '0' + result.days : result.days + '' )+"</span>";
            }

            if ( weeks !== null ) {
                weeks.innerHTML = `<span class='${className}'>`+( result.weeks <= 0 || isNaN( result.weeks ) ? '00' : result.weeks < 10 ? '0' + result.weeks : result.weeks + '' )+"</span>";
            }

            if ( month !== null ) {
                month.innerHTML = `<span class='${className}'>`+( result.months <= 0 || isNaN( result.months ) ? '00' : result.months < 10 ? '0' + result.months : result.months + '' )+"</span>";
            }

            if ( year !== null ) {
                year.innerHTML = `<span class='${className}'>`+( result.years <= 0 || isNaN( result.years ) ? '00' : result.years < 10 ? '0' + result.years : result.years + '' )+"</span>";
            }

        }

        let mult = {};
        if ( animationType === 'circle' ) {

            const fullCircle = getFullCircle();
            const $circles = document.querySelectorAll( `.${wrapperName} .mdp-countdowner-circle-animation` );
            const $svgs = document.querySelectorAll( `.${wrapperName} .mdp-countdowner-elementor-circle-animation-svg` );
            const radius = document.querySelector( `.${wrapperName} .mdp-countdowner-circle-animation` ).r.baseVal.value

            //setting height and width for svg
            $svgs.forEach( $svg => {
                $svg.style.height = radius * 2 + 'px';
                $svg.style.width = radius * 2 + 'px';
            } );


            $circles.forEach( circle => {
                circle.style.strokeDasharray = fullCircle;
            } );

            if ( countdownType === 'due-date' ) {

                // Prepare data for years counter
                const fullYears = getDateTimeDiff( moment( new Date( timeStart ) ), moment ( new Date( time ) ), unitsArray );
                const leftYears = ( 'wordpress_default' === timezone ) ?
                    getDateTimeDiff( moment( getWpTimeArray() ), moment ( new Date( time ) ), unitsArray ) :
                    getDateTimeDiff( moment( new Date() ), moment ( new Date( time ) ), unitsArray );

                if ( fullYears.years === leftYears.years && fullYears.years > 0 && leftYears.years > 0 ) {
                    mult['years'] = fullCircle / fullYears.years;
                } else {
                    mult['years'] = fullYears.years - leftYears.years > 0 ?
                        fullCircle / fullYears.years : 0;
                }

            } else if( countdownType === 'evergreen-timer' ) {

                const yearInterval = localStorage.getItem( wrapperName + '_evergreen-time-interval' ) / 31536000;
                mult['years'] = yearInterval >= 1 ? fullCircle / Math.floor( yearInterval ) : 0;

            }

        }

        let wpDate = moment( getWpTimeArray() );

        timeCount = setInterval( () => {
             let timerTypeInterval;
             let animationTypeInterval;
             let timeInterval;
             let timezoneInterval;
             let resetTimerOnExpire;
             if ( document.querySelector( `.${wrapperName}` ) !== null ) {
                 timerTypeInterval = document.querySelector( `.${wrapperName}` ).dataset.countdownType;
                 animationTypeInterval = document.querySelector( `.${wrapperName}` ).dataset.animation;
                 timeInterval = document.querySelector( `.${wrapperName}` ).dataset.time;
                 timezoneInterval = document.querySelector( `.${wrapperName}` ).dataset.timezone;
                 resetTimerOnExpire = document.querySelector( `.${wrapperName}` ).dataset.resetTimerOnExpire;
             }

            // clear localstorage on deleting widget
            if (  document.querySelector( `.${wrapperName}` ) === null ) {
                localStorage.removeItem( wrapperName + '_evergreen-time-interval' );
                localStorage.removeItem( wrapperName + '_evergreen-time-due-date' );
                localStorage.removeItem( wrapperName + '_evergreen-wp-time-due-date' );
            }

            let today = moment( new Date() );
            const date = moment( timeInterval, "YYYY-MM-DDTHH:mm:ssZ" );


           if ( timezoneInterval === 'wordpress_default' ) {
                wpDate.add( 1, 's' );
                today = moment( wpDate.format() );
           }

            let resultDate = getDateTimeDiff( today, date, unitsArray );

            if ( timerTypeInterval === 'evergreen-timer' ) {
                let storageDueDate = '';

                if ( timezoneInterval === 'wordpress_default' ) {
                    storageDueDate = localStorage.getItem( wrapperName + '_evergreen-wp-time-due-date' );
                } else {
                    storageDueDate = localStorage.getItem( wrapperName + '_evergreen-time-due-date' );
                }


                resultDate = getDateTimeDiff( today, moment( storageDueDate ), unitsArray );

            }

            if ( Object.values( resultDate ).every( ( element ) => { return element <= 0 } ) ) {
                if ( !resetTimerOnExpire || resetTimerOnExpire !== 'yes' ) {
                    this.expireAction(wrapperName, expireAction, resultDate.seconds);
                    clearInterval( timeCount );
                } else {
                    localStorage.removeItem( wrapperName + '_evergreen-time-due-date' );
                    localStorage.removeItem( wrapperName + '_evergreen-wp-time-due-date' );
                    localStorage.removeItem( wrapperName + '_evergreen-time-interval' );
                }
            }

            if ( 'years' in resultDate ) {
                if ( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'years', resultDate.years );
                }
                if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const yearsTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-year-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( yearsTextContent !== resultDate.years + '' && yearsTextContent !== '0' + resultDate.years && !isNaN( resultDate.years ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-year-val`, resultDate.years <= 0 || isNaN( resultDate.years ) ? '00' : resultDate.years < 10 ? '0' + resultDate.years : resultDate.years + '', animationTypeInterval );
                    }
                } else if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'years', mult.years, resultDate );
                    }
                    const $years = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-year-val` );
                    $years.textContent = resultDate.years <= 0 || isNaN( resultDate.years ) ? '00' : resultDate.years < 10 ? '0' + resultDate.years : resultDate.years + '';
                }
            }

            if ( 'months' in resultDate ) {
                if ( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'months', resultDate.months );
                }
                if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const monthsTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-month-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( monthsTextContent !== resultDate.months + '' && monthsTextContent !== '0' + resultDate.months && !isNaN( resultDate.months ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-month-val`, resultDate.months <= 0 || isNaN( resultDate.months ) ? '00' : resultDate.months < 10 ? '0' + resultDate.months : resultDate.months + '', animationTypeInterval );
                    }
                } else if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'months', getFullCircle() / 12, resultDate );
                    }
                    const $months = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-month-val` );
                    $months.textContent = resultDate.months <= 0 || isNaN( resultDate.months ) ? '00' : resultDate.months < 10 ? '0' + resultDate.months : resultDate.months + '';
                }
            }

            if ( 'weeks' in resultDate ) {
                if ( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'weeks', resultDate.weeks );
                }
                if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const weeksTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-weeks-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( weeksTextContent !== resultDate.weeks + '' && weeksTextContent !== '0' + resultDate.weeks && !isNaN( resultDate.weeks ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-weeks-val`, resultDate.weeks <= 0 || isNaN( resultDate.weeks ) ? '00' : resultDate.weeks < 10 ? '0' + resultDate.weeks : resultDate.weeks + '', animationTypeInterval );
                    }
                } else if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'weeks', getFullCircle() / 4.34, resultDate );
                    }
                    const $weeks = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-weeks-val` );
                    $weeks.textContent = resultDate.weeks <= 0 || isNaN( resultDate.weeks ) ? '00' : resultDate.weeks < 10 ? '0' + resultDate.weeks : resultDate.weeks + '';
                }
            }

            if ( 'days' in resultDate ) {
                if ( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'days', resultDate.days );
                }
                if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const daysTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-days-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( daysTextContent !== resultDate.days + '' && daysTextContent !== '0' + resultDate.days && !isNaN( resultDate.days ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-days-val`, resultDate.days <= 0 || isNaN( resultDate.days ) ? '00' : resultDate.days < 10 ? '0' + resultDate.days : resultDate.days + '', animationTypeInterval );
                    }
                } else if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'days', getFullCircle() / 7, resultDate );
                    }
                    const $days = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-days-val` );
                    $days.textContent = resultDate.days <= 0 || isNaN( resultDate.days ) ? '00' : resultDate.days < 10 ? '0' + resultDate.days : resultDate.days + '';
                }
            }

            if ( 'hours' in resultDate ) {
                if ( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'hours', resultDate.hours );
                }
                if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const hoursTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-hours-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( hoursTextContent !== resultDate.hours + '' && hoursTextContent !== '0' + resultDate.hours && !isNaN( resultDate.hours ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-hours-val`, resultDate.hours <= 0 || isNaN( resultDate.hours ) ? '00' : resultDate.hours < 10 ? '0' + resultDate.hours : resultDate.hours + '', animationTypeInterval );
                    }
                } else if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'hours', getFullCircle() / 24, resultDate );
                    }
                    const $hours = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-hours-val` );
                    $hours.textContent = resultDate.hours <= 0 || isNaN( resultDate.hours ) ? '00' : resultDate.hours < 10 ? '0' + resultDate.hours : resultDate.hours + '';
                }
            }

            if ( 'minutes' in resultDate ) {
                if( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'minutes', resultDate.minutes );
                }
                if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const minutesTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-minutes-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( minutesTextContent !== resultDate.minutes + '' && minutesTextContent !== '0' + resultDate.minutes && !isNaN( resultDate.minutes ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-minutes-val`, resultDate.minutes <= 0 || isNaN( resultDate.minutes ) ? '00' : resultDate.minutes < 10 ? '0' + resultDate.minutes : resultDate.minutes + '', animationTypeInterval );
                    }
                } else if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'minutes', getFullCircle() / 60, resultDate );
                    }
                    const $minutes = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-minutes-val` );
                    $minutes.textContent = resultDate.minutes <= 0 || isNaN( resultDate.minutes ) ? '00' : resultDate.minutes < 10 ? '0' + resultDate.minutes : resultDate.minutes + '';
                }
            }

            if ( 'seconds' in resultDate ) {
                if ( animationTypeInterval === 'flip' ) {
                    flipAnimation( 'seconds', resultDate.seconds );
                }

                if ( animationTypeInterval === 'none' || animationTypeInterval === 'circle' ) {
                    if ( animationTypeInterval === 'circle' ) {
                        circleAnimation( 'seconds', getFullCircle() / 60, resultDate );
                    }
                    const $seconds = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-seconds-val` );
                    $seconds.textContent = resultDate.seconds <= 0 || isNaN( resultDate.seconds ) ? '00' : resultDate.seconds < 10 ? '0' + resultDate.seconds : resultDate.seconds + '';
                } else if ( animationTypeInterval === 'glowing' || animationTypeInterval === 'fade' || animationTypeInterval === 'roll-digits' ) {
                    const secondsTextContent = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-seconds-val .mdp-countdowner-elementor--wrapper-item-show-${animationTypeInterval}` ).textContent;
                    if ( secondsTextContent !== resultDate.seconds + '' && secondsTextContent !== '0' + resultDate.seconds && !isNaN( resultDate.seconds ) ) {
                        changeHtml( `.${wrapperName} .mdp-countdowner-elementor--wrapper-item-seconds-val`, resultDate.seconds <= 0 || isNaN( resultDate.seconds ) ? '00' : resultDate.seconds < 10 ? '0' + resultDate.seconds : resultDate.seconds + '', animationTypeInterval );
                    }
                }

            }

        }, 1000 );

        this.intervals.push( timeCount );

    },

    expireAction: function ( wrapperName, expireType, seconds ) {
        const countdowner = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper` );
        const editorMode = document.querySelector( `.${wrapperName}` ).dataset.editorMode;

        switch ( expireType ) {
            case 'redirection-link':
                const redirectLink = document.querySelector( `.${wrapperName}` ).dataset.redirectLink;
                if ( !editorMode ) {
                    window.location.href = redirectLink;
                }
                break;

            case 'hide':
                countdowner.style.display = 'none';
                if ( editorMode ) {
                    document.querySelector( `.${wrapperName}` ).style.height = '5px';
                }
                break;

            case 'message':
                const $messageBeforeExpire = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-countdowner-before-expire-message` );
                const $messageAfterExpire = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-countdowner-after-expire-message` );

                if( $messageAfterExpire !== null && $messageBeforeExpire !== null ) {
                    $messageAfterExpire.style.display = 'block';

                    if ( seconds > 0 ) {
                        countdowner.style.display = 'block'
                        $messageBeforeExpire.style.display = 'block';
                        $messageAfterExpire.style.display = 'none';
                    }

                    countdowner.style.display = 'none';

                }
                break;

            case 'button':
                const $buttonBeforeExpire = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-before-button` );
                const $buttonAfterExpire = document.querySelector( `.${wrapperName} .mdp-countdowner-elementor--wrapper-after-button` );

                if( $buttonBeforeExpire !== null && $buttonAfterExpire !== null ) {

                    $buttonAfterExpire.style.display = 'inline-block';

                    if ( seconds > 0 ) {
                        countdowner.style.display = 'block';
                        $buttonBeforeExpire.style.display = 'inline-block';
                        $buttonAfterExpire.style.display = 'none';
                    }

                    countdowner.style.display = 'none';

                }
                break;
            case 'none':

                break;
        }

    },

    addCountdowner: function () {

        //clear all intervals
        this.intervals.forEach( interval => {
            clearInterval( interval );
        } );

        this.intervals = [];

        const $countdownerBox = document.querySelectorAll( '.mdp-countdowner-elementor-box' );

        for ( let i = 0; i < $countdownerBox.length; i++ ) {
            $countdownerBox[i].classList.add( 'mdp-countdowner-elementor-box-' + i );
            this.countdownerMain( 'mdp-countdowner-elementor-box-' + i );
        }

    }

};

/**
 * Init for Front-End
 * @param callback
 */
document.addEventListener( 'DOMContentLoaded', mdpCountdowner.addCountdowner.bind( mdpCountdowner ) );