

const mdpGalerieElementor = {

    galerieElementor: function ( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-galerie-elementor-box` );
        const $content = document.querySelector( `.${wrapperName} .mdp-galerie-elementor-content` );
        const allFilter = $box.dataset.allFilter;
        const layout = $box.dataset.layout;
        const $filters = document.querySelectorAll( `.${wrapperName} .mdp-galerie-elementor-filter-item` );
        const imageAnimation = $box.dataset.imageAnimation;
        const widescreenColumns = $box.dataset.columnsWidescreen;
        const desktopColumns = $box.dataset.columns;
        const laptopColumns =  $box.dataset.columnsLaptop;
        const tabletExtraColumns = $box.dataset.columnsTablet_extra;
        const tabletColumns = $box.dataset.columnsTablet;
        const mobileExtraColumns = $box.dataset.columnsMobile_extra;
        const mobileColumns = $box.dataset.columnsMobile;
        const height = window.innerHeight;

        const activeBreakpoints = elementorFrontend.config.responsive.activeBreakpoints;

        const setColumns = () => {
            let columns;

            // set responsive columns
            if ( activeBreakpoints.hasOwnProperty( 'widescreen' ) &&
                window.matchMedia( `(min-width: ${activeBreakpoints.widescreen.value}px)` ).matches ) {
                columns = widescreenColumns;
            }

            if ( widescreenColumns && activeBreakpoints.hasOwnProperty( 'widescreen' ) ) {
                if ( window.matchMedia( `(min-width: ${activeBreakpoints.tablet.value + 1}px)` ).matches
                    && window.matchMedia( `(max-width: ${activeBreakpoints.widescreen.value - 1}px)` ).matches ) {
                    columns = desktopColumns;
                }
            } else {
                if ( window.matchMedia( `(min-width: ${activeBreakpoints.tablet.value + 1}px)` ).matches ) {
                    columns = desktopColumns;
                }
            }

            if ( activeBreakpoints.hasOwnProperty( 'laptop' )
                && window.matchMedia( `(max-width: ${activeBreakpoints.laptop.value}px)` ).matches ) {
                columns = laptopColumns;
            }

            if ( activeBreakpoints.hasOwnProperty( 'tablet_extra' )
                && window.matchMedia( `(max-width: ${activeBreakpoints.tablet_extra.value}px) ` ).matches ) {
                columns = tabletExtraColumns;
            }

            if ( window.matchMedia( `(max-width: ${activeBreakpoints.tablet.value}px)` ).matches  ) {
                columns = tabletColumns;
            }

            if ( activeBreakpoints.hasOwnProperty( 'mobile_extra' )
                && window.matchMedia( ` (max-width: ${activeBreakpoints.mobile_extra.value}px) ` ).matches ) {
                columns = mobileExtraColumns;
            }

            if ( window.matchMedia( `(max-width: ${activeBreakpoints.mobile.value}px)` ).matches ) {
                columns = mobileColumns;
            }

            return columns;
        }

        const setGalerie = () => {
            this.switchGalerie( wrapperName, setColumns(), height );
        }

        const columns = !setColumns() ? 1 : setColumns();

        if ( layout === 'masonry' ) {
            // reset previous layout before creating new
            this.resetMasonryLayout( $content );

            // create layout
            this.displayMasonryLayout( $content, +columns );
        }

        const initialFilter = allFilter === 'yes' ? 'all' : 0;
        $filters[0].classList.add('mdp-galerie-elementor-filter-item-active');
        this.showGalleryImages( wrapperName, initialFilter, imageAnimation );

        setGalerie();

        window.addEventListener( 'resize', setGalerie );

    },

    switchGalerie: function ( wrapperName, columns, height ) {
        const $filters = document.querySelectorAll( `.${wrapperName} .mdp-galerie-elementor-filter-item` );
        const $content = document.querySelector( `.${wrapperName} .mdp-galerie-elementor-content` );
        const $box = document.querySelector( `.${wrapperName} .mdp-galerie-elementor-box` );
        const layout = $box.dataset.layout;
        const lazyLoad = $box.dataset.lazyLoad;
        const allFilter = $box.dataset.allFilter;
        const imageAnimation = $box.dataset.imageAnimation;
        const filterTextAnimation = $box.dataset.filterTextAnimation;

        if ( !columns ) { columns = 1 }

        $content.style.gridTemplateColumns = `repeat( ${columns}, 1fr )`;

        if ( elementorFrontend.isEditMode() && layout === 'masonry' ) {
            // reset previous layout before creating new
            this.resetMasonryLayout( $content );

            // create layout
            this.displayMasonryLayout( $content, +columns );
        }

        // if there is no filters return
        if ( $filters.length === 0 ) { return; }

        if ( elementorFrontend.isEditMode() ) {
            const initialFilter = allFilter === 'yes' ? 'all' : 0;
            $filters[0].classList.add('mdp-galerie-elementor-filter-item-active');
            this.showGalleryImages( wrapperName, initialFilter, imageAnimation );
        }

        // lazy loading
        if ( lazyLoad === 'yes' ) {
            this.lazyLoading( wrapperName );
        }

        function resetActiveFilters() {
            const $activeFilters = document.querySelectorAll( `.${wrapperName} .mdp-galerie-elementor-filter-item-active` );

            $activeFilters.forEach( filter => {
                filter.classList.remove( 'mdp-galerie-elementor-filter-item-active' );
            } );
        }

        for ( let i = 0; i < $filters.length; i++ ) {

            $filters[i].addEventListener( 'mouseenter', () => {
                $filters[i].style.animation = `${filterTextAnimation} both`;
            } );

            $filters[i].addEventListener( 'mouseleave', () => {
                $filters[i].style.animation = '0';
            } );

            $filters[i].addEventListener( 'click', e => {

                const filterIndex = e.target.dataset.index;

                resetActiveFilters();

                e.target.classList.add( 'mdp-galerie-elementor-filter-item-active' );

                this.showGalleryImages( wrapperName, filterIndex, imageAnimation );

                // lazy loading
                if ( lazyLoad === 'yes' ) {
                    this.lazyLoading( wrapperName );
                }

            } );
        }

        // scroll filters if necessary
        this.scrollFilters( wrapperName );

    },

    scrollFilters: function ( wrapperName ) {
        const $filtersBar = document.querySelector( `.${wrapperName} .mdp-galerie-elementor-filter-nav` );
        const $filtersScroll = document.querySelector( `.${wrapperName} .mdp-galerie-elementor-filter-nav-scroll` );
        const filtersBarWidth = $filtersBar.getBoundingClientRect().width;
        const filtersWidth = $filtersBar.scrollWidth;
        let startX = 0;
        let prevTranslate = 0;


        if ( $filtersScroll.classList.contains( 'mdp-galerie-elementor-filter-alignment-right' ) ) {
            $filtersScroll.style.transform = `translateX(${ -(filtersWidth - filtersBarWidth) }px)`;
        }


        // Exit if scrolling is unnecessary
        if ( filtersWidth <= filtersBarWidth ) { return; }

        // Bind mouse events
        $filtersBar.addEventListener( 'mousedown', startDragging, false );
        $filtersBar.addEventListener( 'mouseup', stopDragging, false );
        $filtersBar.addEventListener( 'mouseleave', stopDragging, false );

        // Bind touch events
        $filtersBar.addEventListener( 'touchstart', startDragging, false );
        $filtersBar.addEventListener( 'touchend', stopDragging, false );
        $filtersBar.addEventListener( 'touchcancel', stopDragging, false );

        /**
         * Start dragging
         * @param e
         */
        function startDragging ( e ) {

            if ( 'touchstart' === e.type ) {

                startX = e.touches[0].clientX;
                $filtersBar.addEventListener( 'touchmove', moveTabs, false);

            } else {

                startX = e.clientX;
                $filtersBar.addEventListener( 'mousemove', moveTabs, false );

            }

        }

        /**
         * Stop dragging
         * @param e
         */
        function stopDragging ( e ) {

            // Un-bind move events
            $filtersBar.removeEventListener( 'mousemove', moveTabs, false );
            $filtersBar.removeEventListener( 'touchmove', moveTabs, false );

            prevTranslate = parseInt( $filtersScroll.style.transform.replace( /[()a-zA-Z]/g, "" ) );

        }

        /**
         * Move tabs
         * @param e
         */
        function moveTabs ( e ) {

            const delta = 'touchmove' === e.type ?
                Math.round( startX - e.touches[0].clientX ) :
                startX - e.clientX;

            // Exit if new delta same as old delta
            const prevDelta = parseInt( $filtersScroll.style.transform.replace( /[()a-zA-Z]/g, "" ) );
            if ( delta === Math.abs( prevDelta ) ) { return; }

            if ( isNaN( prevTranslate ) ) { prevTranslate = 0 }

            // Exit if dragging more than container width
            let newDelta = prevTranslate - delta;

            if ( $filtersScroll.classList.contains( 'mdp-galerie-elementor-filter-alignment-center' ) ) {
                if ( filtersWidth - filtersBarWidth <= Math.abs( newDelta ) ) { return; }
            } else {
                if ( newDelta > 0 || filtersWidth - filtersBarWidth <= Math.abs( newDelta ) ) { return; }
            }

            // Apply transforms for tabs wrapper
            document.querySelector( `.${ wrapperName } .mdp-galerie-elementor-filter-nav-scroll` ).style.transform = `translateX(${ newDelta }px)`;

        }


    },

    lazyLoading: function ( wrapperName ) {

        // check if images already in viewport
        lazyLoad();

        function lazyLoad() {
            const $images = document.querySelectorAll( `.${wrapperName} .mdp-galerie-elementor-gallery-image-lazy` );
            $images.forEach( image => {
                if ( image.getBoundingClientRect().top <= ( window.innerHeight || document.documentElement.clientHeight ) ) {
                    image.src = image.dataset.src;
                    image.classList.remove( "mdp-galerie-elementor-gallery-image-lazy" );
                }
            } );

            if ( $images.length === 0 ) { window.removeEventListener( 'scroll', lazyLoad ); }
        }

        window.addEventListener( 'scroll', lazyLoad );

    },

    showGalleryImages: function ( wrapperName, index, imageAnimation ) {
        const $images = document.querySelectorAll( `.${wrapperName} .mdp-galerie-elementor-gallery-image` );

        $images.forEach( image => {
            const imageFilters = image.dataset.filter;

            if ( imageFilters.search( index ) !== -1 ) {
                image.classList.add( 'mdp-galerie-elementor-image-is-active' );

                // add image animation on mouseenter
                image.addEventListener( 'mouseenter', () => {
                    image.style.animation = `${imageAnimation} both`;
                } );

                // remove image animation on mouseleave
                image.addEventListener( 'mouseleave', () => {
                    image.style.animation = '0';
                } );
            } else {
                image.classList.remove( 'mdp-galerie-elementor-image-is-active' );
            }

        } );

    },

    resetMasonryLayout: function ( $container ) {
        const columns = $container.querySelectorAll( '.mdp-galerie-elementor-gallery-masonry-column' );
        const images = $container.querySelectorAll( '.mdp-galerie-elementor-gallery-image' );

        images.forEach( image => {
            $container.appendChild( image );
        } );

        columns.forEach( column => {
            column.remove();
        } );

    },

    displayMasonryLayout: function ( $container, columns ) {
        const images = $container.querySelectorAll( '.mdp-galerie-elementor-gallery-image' );
            for ( let i = 1; i <= columns; i++ ) {
            const column = document.createElement( 'div' );
            column.classList.add( "mdp-galerie-elementor-gallery-masonry-column" );
            column.classList.add( "mdp-galerie-elementor-gallery-masonry-column-" + i );
            $container.appendChild( column );
        }
        let countColumn = 1;

        images.forEach( ( image ) => {
            let col = $container.querySelector( `.mdp-galerie-elementor-gallery-masonry-column-${countColumn}` );
            col.appendChild( image );
            countColumn = countColumn < columns ? countColumn + 1 : 1;
        } );
    }
}


/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-galerie-elementor' ) ) {
            mdpGalerieElementor.galerieElementor.call( mdpGalerieElementor, `elementor-element-${$scope[0].dataset.id}` );
        }
    } );

} );