/**
 * Liner for Elementor
 * The plugin for Elementor which allows you to animate and manage SVG images. Customize each SVG path to create a unique style of your animation.
 * Exclusively on https://1.envato.market/liner-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.0
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com)
 * @support         help@merkulov.design
 **/

/**
 * Initialize liner for element
 * @param uniqueId {string} Unique element hash
 * @constructor
 */
window.Liner = function ( uniqueId ) {

    if ( typeof uniqueId !== 'string' ) { return }

    /** Play Animation */
    const linerPlay = function() {

        liner.play( 1 );
        if ( loop && 'end' === liner.getStatus() ) { liner.reset().play( 1 ) }

    }

    /** Stop Animation */
    const linerStop = function() {

        liner.stop();

    }

    /** Looping Animation */
    const linerLoop = function () {

        if ( 'end' === liner.getStatus() ) {

            liner.reset().play( 1 );

        }

    }

    /** Callback for Liner Init */
    const linerReady = function () {

        let linerAppear = setInterval( () => {

            if ( 'end' !== liner.getStatus() ) {

                $box.classList.remove( 'mdp-liner-init' );
                clearInterval( linerAppear );

            }

        }, 1 );

    }

    const $box = document.getElementById( `mdp-liner-${ uniqueId }` );

    const start = $box.getAttribute( 'data-start' );
    const url = $box.getAttribute( 'data-url' );
    const type = $box.getAttribute( 'data-type' );
    const duration = JSON.parse( $box.getAttribute( 'data-duration' ) );
    const delay = JSON.parse( $box.getAttribute( 'data-delay' ) );
    const dashGap = JSON.parse( $box.getAttribute( 'data-dash' ) );
    const loop = JSON.parse( $box.getAttribute( 'data-loop' ) );
    const reverseStack = JSON.parse( $box.getAttribute( 'data-reverse' ) );
    const anim = $box.getAttribute( 'data-anim' );
    const path = $box.getAttribute( 'data-path' );

    let options = {
        start: 'autostart' === start || 'inViewport' === start ? start : 'manual',
        file: url,
        type: type,
        duration: duration,
        reverseStack: reverseStack,
        dashGap: dashGap,
        animTimingFunction: Vivus[ anim ],
        pathTimingFunction: Vivus[ path ]
    };

    if ( 'delayed' === type ) { options.delay = delay >= duration ? duration - 1 : delay }

    /** Initialize Vivus for single Liner */
    const liner = new Vivus( $box.id, options, void( 0 ) );
    linerReady();

    /** Autostart and Play in Viewport */
    if ( 'autostart' === start || 'inViewport' === start ) {

        /** Looping for Autostart and inViewport */
        if ( loop ) {

            setInterval( linerLoop, 500 );

        }

    }

    /** Play on hover */
    if ( 'hover' === start ) {

        /** Start the animation if you hover over the element and this is not the end of the animation. */
        $box.removeEventListener( 'mouseover', linerPlay );
        $box.addEventListener( 'mouseover', linerPlay );

        /** Stop animation if the cursor is not over the element. */
        $box.removeEventListener( 'mouseout', linerStop );
        $box.addEventListener( 'mouseout', linerStop );

    }

    /** Play on click */
    if ( 'click' === start ) {

        let play = false;

        $box.addEventListener( 'click', ( e ) => {


            if ( 'start' === liner.getStatus() ) {

                /** First click */
                liner.play( 1 );
                play = true;

            } else {

                /** Second and future clicks */
                if ( 'end' === liner.getStatus() ) {

                    /** Click after playback ending */
                    if ( loop ) {

                        liner.reset().play( 1 );
                        play = true;

                    } else {

                        liner.stop();
                        play = false;

                    }

                }

                if ( 'progress' === liner.getStatus() ) {

                    /** Click during playback */
                    if ( play ) {

                        liner.stop();
                        play = false;

                    } else {

                        liner.play( 1 );
                        play = true;

                    }

                }

            }

        } );

    }

}
