const mdpMapperElementor = {

    mapperElementor: function( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-mapper-elementor-box` );
        const $mapWrapper = document.querySelector( `.${wrapperName} .mdp-mapper-elementor-map-wrapper` );
        const mapHeight = $box.dataset.mapHeight ? JSON.parse( $box.dataset.mapHeight ) : '';
        const mapWidth = $box.dataset.mapWidth ? JSON.parse( $box.dataset.mapWidth ) : '';
        const tooltipTrigger = $box.dataset.tooltipTrigger;
        const mapResolution = $box.dataset.resolution;
        const displayType = $box.dataset.displayType;
        const displayMode = $box.dataset.displayMode;
        const regions = $box.dataset.regions;
        const regionsArray = this.isValidJsonString( regions ) ? JSON.parse( regions ) : '';
        const inactiveRegionsColor = $box.dataset.inactiveRegionColor;
        const backgroundColor = $box.dataset.backgroundColor;
        const regionToDisplay = $box.dataset.regionToDisplay;
        const markers = $box.dataset.markers;
        const markersArray = this.isValidJsonString( markers ) ? JSON.parse( markers ) : '';
        const textRegions = $box.dataset.textRegions;
        const textRegionsArray = this.isValidJsonString( textRegions ) ? JSON.parse( textRegions ) : '';
       // const tooltipFont = $box.dataset.tooltipFont;
        const tooltipFontSize = $box.dataset.tooltipFontSize;
        const tooltipTextBold = $box.dataset.tooltipTextBold;
        const tooltipTextItalic = $box.dataset.tooltipTextItalic;
        let regionsLinks = [];
        let markersLinks = [];

        google.load( 'visualization', "1", {
            packages: ["geochart"],
            mapsApiKey: mdpMapperObjectWP.maps_key
        } );

        const drawMap = () => {
            let data = new google.visualization.DataTable();

            // set map options
            const options = {
                displayMode: displayMode,
                datalessRegionColor: inactiveRegionsColor,
                backgroundColor: backgroundColor,
                resolution: mapResolution,
                tooltip: {
                    isHtml: true,
                    trigger: tooltipTrigger,
                    textStyle: {
                        fontSize: tooltipFontSize,
                        bold: tooltipTextBold === 'yes',
                        italic: tooltipTextItalic === 'yes'
                    }
                }
            };

            // set map width and height
            if ( mapHeight ) { options.height = this.checkScreenSize( mapHeight, 'height', $box ); }
            if ( mapWidth ) { options.width = this.checkScreenSize( mapWidth, 'width', $box ); }

            // set region to display
            regionToDisplay ? options.region = regionToDisplay : '';

            if ( displayMode === 'regions' ) {

                if ( displayType === 'custom' ) {
                    this.createCustomRegions( $box, regionsArray, data, options, regionsLinks );
                    options.legend = 'none';
                } else {
                    const colorAxis = $box.dataset.regionColorAxis;
                    const colorAxisArray = this.isValidJsonString( colorAxis ) ? JSON.parse( colorAxis ) : '';
                    const enableLegend = $box.dataset.regionEnableLegend;
                    const legendTextColor = $box.dataset.regionLegendColor;
                    const legendTextFontSize = $box.dataset.regionLegendFontSize;
                    const legendTextBold = $box.dataset.regionLegendBoldText;
                    const legendTextItalic = $box.dataset.regionLegendItalicText;

                    const regionsData = this.createDataVisRegions( $box, regionsArray );

                    data = regionsData.data;

                    regionsLinks = regionsData.regionsLinks;

                    if ( enableLegend !== 'yes' ) {
                        options.legend = 'none';
                    } else {
                        options.legend = {
                            textStyle: {
                                color: !legendTextColor ? '#000' : legendTextColor,
                                fontSize: !legendTextFontSize ? 16 : legendTextFontSize,
                                bold: legendTextBold === 'yes',
                                italic: legendTextItalic === 'yes',
                            }
                        }
                    }

                    options.colorAxis = { colors: this.createColorAxisArray( colorAxisArray ) };
                }

            } else if ( displayMode === 'markers' ) {
                const magnifyingGlassZoom = $box.dataset.magnifyingGlassZoom;

                options.magnifyingGlass = {
                    enable: true,
                    zoomFactor: magnifyingGlassZoom
                }

                if ( displayType === 'custom' ) {
                    this.createCustomMarkers( $box, markersArray, data, options, markersLinks );
                    options.legend = 'none';
                } else {
                    const colorAxis = $box.dataset.markersColorAxis;
                    const colorAxisArray = this.isValidJsonString( colorAxis ) ? JSON.parse( colorAxis ) : '';
                    const markerSizeAxisMin = $box.dataset.markerSizeAxisMinVal;
                    const markerSizeAxisMax = $box.dataset.markerSizeAxisMaxVal;
                    const enableLegend = $box.dataset.markersEnableLegend;
                    const legendTextColor = $box.dataset.markersLegendColor;
                    const legendTextFontSize = $box.dataset.markersLegendFontSize;
                    const legendTextBold = $box.dataset.markersLegendBoldText;
                    const legendTextItalic = $box.dataset.markersLegendItalicText;

                    if ( enableLegend !== 'yes' ) {
                        options.legend = 'none';
                    } else {
                        options.legend = {
                            textStyle: {
                                color: !legendTextColor ? '#000' : legendTextColor,
                                fontSize: !legendTextFontSize ? 16 : legendTextFontSize,
                                bold: legendTextBold === 'yes',
                                italic: legendTextItalic === 'yes',
                            }
                        }
                    }

                    options.sizeAxis = { minValue: +markerSizeAxisMin, maxValue: +markerSizeAxisMax };
                    options.colorAxis = { colors: this.createColorAxisArray( colorAxisArray ) };

                    const markersData = this.createDataVisMarkers( $box, markersArray, data );

                    data = markersData.data;

                    markersLinks = markersData.markersLinks;
                }

            } else if ( displayMode === 'text' ) {

                if ( displayType === 'custom' ) {
                    this.createCustomRegionsText( textRegionsArray, data, options );
                    options.legend = 'none';
                } else {
                   const colorAxis = $box.dataset.textColorAxis;
                   const colorAxisArray = this.isValidJsonString( colorAxis ) ? JSON.parse( colorAxis ) : '';
                   const enableLegend = $box.dataset.textRegionsEnableLegend;
                   const textTextColor = $box.dataset.textLegendColor;
                   const textTextFontSize = $box.dataset.textLegendFontSize;
                   const textTextBold = $box.dataset.textLegendBoldText;
                   const textTextItalic = $box.dataset.textLegendItalicText;

                   data = this.createDataVisRegionsText( textRegionsArray );

                   if ( enableLegend !== 'yes' ) {
                        options.legend = 'none';
                   } else {
                       options.legend = {
                           textStyle: {
                               color: !textTextColor ? '#000' : textTextColor,
                               fontSize: !textTextFontSize ? 16 : textTextFontSize,
                               bold: textTextBold === 'yes',
                               italic: textTextItalic === 'yes',
                           }
                       }
                   }
                   options.colorAxis = { colors: this.createColorAxisArray( colorAxisArray ) };
                }
            }

            const chart = new google.visualization.GeoChart( $mapWrapper );
            google.visualization.events.addListener( chart, 'select', () => {
                const selection = chart.getSelection();
                if ( selection.length === 1 ) {
                    const selectedRow = selection[0].row;
                    const selectedRegion = data.getValue( selectedRow, 0 );
                    switch ( displayMode ) {
                        case 'regions':
                            regionsLinks[selectedRegion].url !== '' ?
                                window.open( regionsLinks[selectedRegion].url, regionsLinks[selectedRegion].target ) :
                                '';
                            break;
                        case 'markers':
                            markersLinks[selectedRegion].url !== '' ?
                                window.open( markersLinks[selectedRegion].url, markersLinks[selectedRegion].target ) :
                                '';
                            break;
                    }
                }
            } );

            chart.draw( data, options );
        }

        google.charts.setOnLoadCallback( drawMap );
        window.addEventListener( 'resize', () => { google.charts.setOnLoadCallback( drawMap ); } );

    },

    checkScreenSize: function ( valueToSet, type, $box ) {
        let option = '';

        if ( window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile.value}px)` ).matches ) {
            option = valueToSet[`${type}Mobile`];
            $box.style[type] = valueToSet[`${type}Mobile`] + 'px';
        } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'mobile_extra' )
            && window.matchMedia( ` (max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile_extra.value}px) ` ).matches
            && valueToSet[`${type}MobileExtra`] ) {
            option = valueToSet[`${type}MobileExtra`];
            $box.style[type] = valueToSet[`${type}MobileExtra`] + 'px';
        } else if ( window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value}px)` ).matches ) {
            option = valueToSet[`${type}Tablet`];
            $box.style[type] = valueToSet[`${type}Tablet`] + 'px';
        } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'tablet_extra' )
            && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet_extra.value}px) ` ).matches
            && valueToSet[`${type}TabletExtra`] ) {
            option = valueToSet[`${type}TabletExtra`];
            $box.style[type] = valueToSet[`${type}TabletExtra`] + 'px';
        } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'laptop' )
            && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.laptop.value}px)` ).matches
            && valueToSet[`${type}Laptop`] ) {
            option = valueToSet[`${type}Laptop`];
            $box.style[type] = valueToSet[`${type}Laptop`] + 'px';
        } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' )
            && window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.widescreen.value}px)` ).matches
            && valueToSet[`${type}Widescreen`] ){
            option = valueToSet[`${type}Widescreen`];
            $box.style[type] = valueToSet[`${type}Widescreen`] + 'px';
        } else if ( valueToSet[`${type}Widescreen`] && elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' ) ) {
            if ( window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)` ).matches
                && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.widescreen.value - 1}px)` ).matches ) {
                option = valueToSet[type];
                $box.style[type] = valueToSet[type] + 'px';
            }
        } else {
            if ( window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)` ).matches ) {
                option = valueToSet[type];
                $box.style[type] = valueToSet[type] + 'px';
            }
        }

        return option;
    },

    createCustomRegions: function ( $box, regionsArray, data, options, regionsLinks ) {
        const activeRegionsColors = [];

        data.addColumn( 'string', 'Country' );
        data.addColumn( 'number', 'Value' );
        data.addColumn( {type: 'string', role: 'tooltip', 'p': {'html': true}} );

        regionsArray.forEach( ( region, index ) => {
            activeRegionsColors.push( region.active_region_color ? region.active_region_color : "#ccc" );
            options.colors = activeRegionsColors;
            data.addRows( [
                    [{
                        v: region.region_code_name,
                        f: region.active_region_name
                    }, index, region.region_tooltip_content]
                ] );

            regionsLinks[region.region_code_name] = {
                url: region.regions_link ? region.regions_link.url : '',
                target: region.regions_link && !region.regions_link.is_external ? '_self' : ''
            };
        } );

    },

    createDataVisRegions: function ( $box, regionsArray ) {
        const regionsLinks = [];
        const dataVisualArray = [];
        const regionValueTitle = $box.dataset.regionValueTitle;
        const regionAdditionalValueTitle = $box.dataset.regionAdditionalValueTitle;

        if ( regionAdditionalValueTitle !== '' ) {
            dataVisualArray[0] = ['Country', regionValueTitle, regionAdditionalValueTitle];
        } else {
            dataVisualArray[0] = ['Country', regionValueTitle];
        }

        // set active regions
        regionsArray.forEach( region => {
            if ( regionAdditionalValueTitle !== '' ) {
                dataVisualArray.push( [
                    region.data_visualization_region_name,
                    region.data_visualization_value,
                    region.data_visualization_additional_value
                ] );
            } else {
                dataVisualArray.push( [
                    region.data_visualization_region_name,
                    region.data_visualization_value
                ] );
            }

            regionsLinks[region.data_visualization_region_name] = {
                url: region.regions_link ? region.regions_link.url : '',
                target: region.regions_link && !region.regions_link.is_external ? '_self' : ''
            };
        } );

        return { data: google.visualization.arrayToDataTable( dataVisualArray ), regionsLinks };

    },

    createCustomRegionsText: function ( textRegionsArray, data, options ) {
        const activeTextRegionsColors = [];

        data.addColumn( 'string', 'Country' );
        data.addColumn( 'number', 'Value' );

        textRegionsArray.forEach( ( textRegion, index ) => {
            activeTextRegionsColors.push( textRegion.active_region_text_color ? textRegion.active_region_text_color : '#000' );
            options.colors = activeTextRegionsColors;
            options.sizeAxis = { minValue: 1, maxValue: 1, minSize: 15, maxSize: 15 };
            data.addRows( [
                [{
                    v: textRegion.active_region_text
                }, index ]
            ] );
        } );
    },

    createDataVisRegionsText: function ( textRegionsArray ) {
        const dataVisualArray = [];

        dataVisualArray[0] = [ 'Country', 'Val' ];

        textRegionsArray.forEach( region => {

            dataVisualArray.push( [
                region.data_visualization_region_name,
                region.data_visualization_value
            ] );

        } );

        return google.visualization.arrayToDataTable( dataVisualArray );

    },

    createCustomMarkers: function ( $box, markersArray, data, options, markersLinks ) {
        const activeMarkersColors = [];

        data.addColumn( 'number', 'Lat' );
        data.addColumn( 'number', 'Lon' );
        data.addColumn( 'string', 'Country' );
        data.addColumn( 'number', 'Value' );
        data.addColumn( {type:'string', role:'tooltip', 'p': {'html': true}} );
        const imageMarkers = {};

        // set active markers
        markersArray.forEach( ( marker, index ) => {
            activeMarkersColors.push( !marker.marker_color ? '#ccc' : marker.marker_color );
            options.colors = activeMarkersColors;
            options.sizeAxis = { minValue: 1, maxValue: 1, minSize: 15, maxSize: 15 };
            data.addRows( [
                [+marker.marker_latitude, +marker.marker_longitude, marker.marker_title, index, marker.marker_tooltip_content]
            ] );
            markersLinks[marker.marker_latitude] = {
                url: marker.markers_link ? marker.markers_link.url : '',
                target: marker.markers_link && !marker.markers_link.is_external ? '_self' : '',
            };

            // set size of default marker
            setTimeout( () => {
                const $thisMap = $box.querySelector( 'svg' );

                const $icons = $thisMap.querySelectorAll( 'circle' );

                $icons[index].setAttribute( 'r',  marker.marker_size.size );

            }, 500 );

            // create custom images
            if ( this.createCustomIcon( marker, marker.custom_icon_type ) ) {
                imageMarkers[index] = this.createCustomIcon( marker, marker.custom_icon_type );
            }
        } );

        // set custom images as a marker
        if ( Object.keys( imageMarkers ).length > 0 ) {
            setTimeout( () => {
                const $thisMap = $box.querySelector( 'svg' );

                const $icons = $thisMap.querySelectorAll( 'circle' );

                for ( const image in imageMarkers ) {
                    const x = $icons[image].getAttribute( 'cx' );
                    const y = $icons[image].getAttribute( 'cy' );

                    imageMarkers[image].setAttribute( 'x', x );
                    imageMarkers[image].setAttribute( 'y', y );

                    $icons[image].replaceWith( imageMarkers[image].cloneNode( true ) );

                }

            }, 500 );
        }
    },

    createDataVisMarkers: function ( $box, markersArray ) {
        const markersLinks = [];
        const dataVisualArray = [];
        const markerValueTitle = $box.dataset.markerValueTitle;
        const markerAdditionalValueTitle = $box.dataset.markerAdditionalValueTitle;

        markerAdditionalValueTitle !== '' ?
            dataVisualArray[0] = [ 'Country', markerValueTitle, markerAdditionalValueTitle ] :
            dataVisualArray[0] = [ 'Country', markerValueTitle ];

        markersArray.forEach( marker => {
            markerAdditionalValueTitle !== '' ?
                dataVisualArray.push( [
                    marker.data_visualization_marker_region_name,
                    marker.data_visualization_value,
                    marker.data_visualization_additional_value
                ] ) :
                dataVisualArray.push( [
                    marker.data_visualization_marker_region_name,
                    marker.data_visualization_value
                ] );

            markersLinks[marker.data_visualization_marker_region_name] = {
                url: marker.markers_link ? marker.markers_link.url : '',
                target: marker.markers_link && !marker.markers_link.is_external ? '_self' : ''
            };
        } );

        return  { data: google.visualization.arrayToDataTable( dataVisualArray ), markersLinks };

    },

    createCustomIcon: function ( marker, markerType ) {

        if ( !marker.marker_image && !marker.marker_icon ) { return; }

        let markerSize = marker.marker_size.size;

        const width = -( markerSize / 2 );
        const height = -( markerSize / 2 );

        const transform = `translate(${width}, ${height})`;

        let imageIcon = '';

        if ( markerType === 'image' || marker.marker_icon.library === 'svg' ) {
            // creates custom image
            imageIcon = document.createElementNS( 'http://www.w3.org/2000/svg', 'image' );
            imageIcon.setAttributeNS(
            'http://www.w3.org/1999/xlink',
          'href',
                markerType === 'image' ? marker.marker_image.url : marker.marker_icon.value.url
            );
        } else if ( markerType === 'icon' ) {
            // create font awesome icon
            imageIcon = document.createElementNS( 'http://www.w3.org/2000/svg', 'foreignObject' );
            const fontAwesomeIcon = document.createElement( 'i' );
            const iconClasses = marker.marker_icon.value.split( ' ' );
            iconClasses.forEach( iconClass => {
                fontAwesomeIcon.classList.add( iconClass );
            } );
            fontAwesomeIcon.style.fontSize = markerSize + 'px';
            fontAwesomeIcon.style.color = marker.marker_color;
            imageIcon.appendChild( fontAwesomeIcon );

            markerSize += markerSize / 10;

        }
        imageIcon.setAttribute( 'height', markerSize );
        imageIcon.setAttribute( 'width', markerSize );
        imageIcon.setAttribute( 'transform', transform );
        imageIcon.setAttribute( 'preserve', 'xMaxYMax meet' );

        return imageIcon;
    },

    createColorAxisArray: function ( colorAxisArray ) {
        const colors = [];

        // set color axis
        colorAxisArray.forEach( color => {
            if ( color.axis_color !== '' ) {
                colors.push( color.axis_color );
            }
        } );

        return colors;
    },

    isValidJsonString: function( str ) {
        try {
            JSON.parse( str );
        } catch ( e ) {
            return false;
        }
        return true;
    },

}

/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-mapper-elementor' ) ) {
            mdpMapperElementor.mapperElementor.call( mdpMapperElementor, `elementor-element-${$scope[0].dataset.id}` );
        }
    } );

} );