/**
 * Reviewer for Elementor
 * Testimonial carousel for Elementor editor
 * Exclusively on https://1.envato.market/reviewer-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

"use strict";

/**
 * Mdp reviewer main object
 * @type {{intervals: mdpReviewer.intervals, reviewer: mdpReviewer.reviewer, carousel: mdpReviewer.carousel, addReviewer: mdpReviewer.addReviewer}}
 */
const mdpReviewer = {

    intervals: [],

    reviewer: function ( wrapperName ) {
        const $slides = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-slide` );
        const slidesSwitchingMode = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesSwitchingMode;
        const autoplay = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.autoplay;
        const pauseOnHover = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.pauseHover;
        const $arrows = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-arrow` );
        const autoplaySpeed = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.autoplaySpeed;
        const transitionSpeed = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.transitionSpeed;
        let slidesToScroll = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToScroll;
        const slidesToShowDesktop = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShow;
        const slidesToShowWidescreen = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShowWidescreen;
        const slidesToShowLaptop = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShowLaptop;
        const slidesToShowTabletExtra = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShowTabletExtra;
        const slidesToShowMobileExtra = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShowMobileExtra;
        const slidesToShowTablet = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShowTablet;
        const slidesToShowMobile = document.querySelector( ` .${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShowMobile;
        const infiniteLoop = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.infiniteLoop;
        const dotClass = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.dotClass;
        const activeDotClass = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.activeDotClass;
        const centerMode = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.centerMode;


        $slides.forEach( slide => {
            const rating = slide.dataset.rating;
            const $ratingStars = slide.querySelectorAll( '.mdp-reviewer-elementor-slide--rating-star' );

            // rating
            if ( $ratingStars.length > 0 ) {
                for ( let i = 0; i < rating; i++ ) {
                    $ratingStars[i].classList.add( 'mdp-reviewer-elementor-slide--rating-star-active' );
                }
            }

        } );

        // check if slides to scroll empty
        if ( slidesToScroll === '' ) {
            slidesToScroll = 1;
        }

        function initializeSlider () {
            if ( window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile.value}px)` ).matches ) {
                this.carousel( wrapperName, {
                    main: 'mdp-reviewer-elementor-carousel',
                    prev: 'mdp-reviewer-elementor-previous-arrow',
                    next: 'mdp-reviewer-elementor-next-arrow',
                    switchingMode: slidesSwitchingMode,
                    autoplay: autoplay,
                    pauseOnHover: pauseOnHover,
                    arrows: $arrows,
                    autoplaySpeed: autoplaySpeed,
                    transitionSpeed: transitionSpeed,
                    slidesToScroll: slidesToScroll,
                    slidesToShow: slidesToShowMobile,
                    infiniteLoop: infiniteLoop,
                    dotsWrapper: 'mdp-reviewer-elementor-dots',
                    dotClass: dotClass,
                    activeDotClass: activeDotClass,
                    centerMode: centerMode
                } );

            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'mobile_extra' )
                && window.matchMedia( ` (max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile_extra.value}px) ` ).matches
                && slidesToShowMobileExtra ) {
                this.carousel( wrapperName, {
                    main: 'mdp-reviewer-elementor-carousel',
                    prev: 'mdp-reviewer-elementor-previous-arrow',
                    next: 'mdp-reviewer-elementor-next-arrow',
                    switchingMode: slidesSwitchingMode,
                    autoplay: autoplay,
                    pauseOnHover: pauseOnHover,
                    arrows: $arrows,
                    autoplaySpeed: autoplaySpeed,
                    transitionSpeed: transitionSpeed,
                    slidesToScroll: slidesToScroll,
                    slidesToShow: slidesToShowMobileExtra,
                    infiniteLoop: infiniteLoop,
                    dotsWrapper: 'mdp-reviewer-elementor-dots',
                    dotClass: dotClass,
                    activeDotClass: activeDotClass,
                    centerMode: centerMode
                } );

            } else if ( window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value}px)` ).matches ) {
                this.carousel( wrapperName, {
                    main: 'mdp-reviewer-elementor-carousel',
                    prev: 'mdp-reviewer-elementor-previous-arrow',
                    next: 'mdp-reviewer-elementor-next-arrow',
                    switchingMode: slidesSwitchingMode,
                    autoplay: autoplay,
                    pauseOnHover: pauseOnHover,
                    arrows: $arrows,
                    autoplaySpeed: autoplaySpeed,
                    transitionSpeed: transitionSpeed,
                    slidesToScroll: slidesToScroll,
                    slidesToShow: slidesToShowTablet,
                    infiniteLoop: infiniteLoop,
                    dotsWrapper: 'mdp-reviewer-elementor-dots',
                    dotClass: dotClass,
                    activeDotClass: activeDotClass,
                    centerMode: centerMode
                } );

            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'tablet_extra' )
                && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet_extra.value}px) ` ).matches
                && slidesToShowTabletExtra ) {
                this.carousel( wrapperName, {
                    main: 'mdp-reviewer-elementor-carousel',
                    prev: 'mdp-reviewer-elementor-previous-arrow',
                    next: 'mdp-reviewer-elementor-next-arrow',
                    switchingMode: slidesSwitchingMode,
                    autoplay: autoplay,
                    pauseOnHover: pauseOnHover,
                    arrows: $arrows,
                    autoplaySpeed: autoplaySpeed,
                    transitionSpeed: transitionSpeed,
                    slidesToScroll: slidesToScroll,
                    slidesToShow: slidesToShowTabletExtra,
                    infiniteLoop: infiniteLoop,
                    dotsWrapper: 'mdp-reviewer-elementor-dots',
                    dotClass: dotClass,
                    activeDotClass: activeDotClass,
                    centerMode: centerMode
                } );
            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'laptop' )
                && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.laptop.value}px)` ).matches
                && slidesToShowLaptop ) {
                this.carousel( wrapperName, {
                    main: 'mdp-reviewer-elementor-carousel',
                    prev: 'mdp-reviewer-elementor-previous-arrow',
                    next: 'mdp-reviewer-elementor-next-arrow',
                    switchingMode: slidesSwitchingMode,
                    autoplay: autoplay,
                    pauseOnHover: pauseOnHover,
                    arrows: $arrows,
                    autoplaySpeed: autoplaySpeed,
                    transitionSpeed: transitionSpeed,
                    slidesToScroll: slidesToScroll,
                    slidesToShow: slidesToShowLaptop,
                    infiniteLoop: infiniteLoop,
                    dotsWrapper: 'mdp-reviewer-elementor-dots',
                    dotClass: dotClass,
                    activeDotClass: activeDotClass,
                    centerMode: centerMode
                } );
            } else if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' )
                && window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.widescreen.value}px)` ).matches
                && slidesToShowWidescreen ){
                this.carousel( wrapperName, {
                    main: 'mdp-reviewer-elementor-carousel',
                    prev: 'mdp-reviewer-elementor-previous-arrow',
                    next: 'mdp-reviewer-elementor-next-arrow',
                    switchingMode: slidesSwitchingMode,
                    autoplay: autoplay,
                    pauseOnHover: pauseOnHover,
                    arrows: $arrows,
                    autoplaySpeed: autoplaySpeed,
                    transitionSpeed: transitionSpeed,
                    slidesToScroll: slidesToScroll,
                    slidesToShow: slidesToShowWidescreen,
                    infiniteLoop: infiniteLoop,
                    dotsWrapper: 'mdp-reviewer-elementor-dots',
                    dotClass: dotClass,
                    activeDotClass: activeDotClass,
                    centerMode: centerMode
                } );

            } else if ( slidesToShowWidescreen && elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' ) ) {
                if ( window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)` ).matches
                    && window.matchMedia( `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.widescreen.value - 1}px)` ).matches ) {
                    this.carousel( wrapperName, {
                        main: 'mdp-reviewer-elementor-carousel',
                        prev: 'mdp-reviewer-elementor-previous-arrow',
                        next: 'mdp-reviewer-elementor-next-arrow',
                        switchingMode: slidesSwitchingMode,
                        autoplay: autoplay,
                        pauseOnHover: pauseOnHover,
                        arrows: $arrows,
                        autoplaySpeed: autoplaySpeed,
                        transitionSpeed: transitionSpeed,
                        slidesToScroll: slidesToScroll,
                        slidesToShow: slidesToShowDesktop,
                        infiniteLoop: infiniteLoop,
                        dotsWrapper: 'mdp-reviewer-elementor-dots',
                        dotClass: dotClass,
                        activeDotClass: activeDotClass,
                        centerMode: centerMode
                    } );
                }
            } else {
                if ( window.matchMedia( `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)` ).matches ) {
                    this.carousel( wrapperName, {
                        main: 'mdp-reviewer-elementor-carousel',
                        prev: 'mdp-reviewer-elementor-previous-arrow',
                        next: 'mdp-reviewer-elementor-next-arrow',
                        switchingMode: slidesSwitchingMode,
                        autoplay: autoplay,
                        pauseOnHover: pauseOnHover,
                        arrows: $arrows,
                        autoplaySpeed: autoplaySpeed,
                        transitionSpeed: transitionSpeed,
                        slidesToScroll: slidesToScroll,
                        slidesToShow: slidesToShowDesktop,
                        infiniteLoop: infiniteLoop,
                        dotsWrapper: 'mdp-reviewer-elementor-dots',
                        dotClass: dotClass,
                        activeDotClass: activeDotClass,
                        centerMode: centerMode
                    } );
                }
            }

        }

        initializeSlider.call( this );

        // check size of the screen for displaying slides to show on different devices
        window.addEventListener( 'resize', () => {
            initializeSlider.call( this );
        } );


    },

    //carousel function
    carousel: function ( wrapperName, settings ) {

        let options = {
            wrapper: document.querySelector( `.${wrapperName} .${settings.main}` ),
            children: document.querySelector( `.${wrapperName} .${settings.main}` ).children,
            prev: document.querySelector( `.${wrapperName} .${settings.prev}` ),
            next: document.querySelector( `.${wrapperName} .${settings.next}` ),
            dotsWrapper: document.querySelector( `.${wrapperName} .${settings.dotsWrapper}` ),
            position: 0,
            offset: 0,
            maxPosition:  document.querySelector( `.${wrapperName} .${settings.main}` ).children.length,
            switchingMode: settings.switchingMode,
            arrows: settings.arrows,
            pauseOnHover: settings.pauseOnHover,
            autoplay: settings.autoplay,
            autoplaySpeed: +settings.autoplaySpeed * 1000,
            transitionSpeed: settings.transitionSpeed,
            slidesToScroll: +settings.slidesToScroll,
            slidesToShow: +settings.slidesToShow,
            infiniteLoop: settings.infiniteLoop,
            dotClass: settings.dotClass,
            activeDotClass: settings.activeDotClass,
            centerMode: settings.centerMode,
            isDragging: false,
            startX: 0,
            startY: 0
        }


        // if slides to show not defined set desktop slides to show
        if ( !options.slidesToShow ) {
            options.slidesToShow = +document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel` ).dataset.slidesToShow;
        }

        if ( options.maxPosition <= 0 ) return;

        // dots functions
        function generateDots( maxDots ) {
            // don`t add dots if dots length equals to max dots count
            if ( options.dotsWrapper.children.length === maxDots ) return;

            // clear dots
            options.dotsWrapper.innerHTML = '';

            for ( let i = 0; i < maxDots; i++ ) {
                options.dotsWrapper.innerHTML += `<div class="mdp-reviewer-elementor-dot">
                                                       <span class="mdp-reviewer-elementor-dot-active-icon">
                                                          <i class="${options.activeDotClass}"></i>
                                                       </span>
                                                      <span class="mdp-reviewer-elementor-dot-icon">
                                                          <i class="${options.dotClass}"></i>
                                                       </span>
                                                   </div>`;
            }
        }

        function switchActiveDots( activeDotsIcons, index ) {
            let activeDots = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-dot-active-icon--active` );

            // clear active dots
            activeDots.forEach( activeDot => {
                activeDot.classList.remove( 'mdp-reviewer-elementor-dot-active-icon--active' );
            } );

            if ( activeDotsIcons ) {
                activeDotsIcons[index].classList.add('mdp-reviewer-elementor-dot-active-icon--active');
            }
        }

        // autoplay function
        function autoplay( switchSlideFunc ) {
            let autoplayPause = false;
            const autoplayInterval = setInterval(() => {
                if ( !autoplayPause ) {
                    switchSlideFunc();
                }

                // pause autoplay on hover
                if ( options.pauseOnHover === 'yes' ) {
                    document.querySelector( `.${wrapperName}` ).addEventListener( 'mouseenter', () => {
                        autoplayPause = true;
                    } );

                    document.querySelector( `.${wrapperName}` ).addEventListener( 'mouseleave', () => {
                        autoplayPause = false;
                    } );
                }

            }, options.autoplaySpeed );

            this.intervals.push( autoplayInterval );

            // when hover on arrow stop autoplay
            options.arrows.forEach( arrow => {
                arrow.addEventListener( 'mouseenter', () => {
                    autoplayPause = true;
                } );

                arrow.addEventListener( 'mouseleave', () => {
                    autoplayPause = false;
                } );
            } );
        }

        // drag slides function
        function dragSlides( dragAction ) {
            options.wrapper.addEventListener( 'mousedown', dragStart, false );
            options.wrapper.addEventListener( 'mouseup', dragEnd, false );
            options.wrapper.addEventListener( 'mouseleave', dragEnd, false );

            options.wrapper.addEventListener( 'touchstart', dragStart, false );
            options.wrapper.addEventListener( 'touchend', dragEnd, false );
            options.wrapper.addEventListener( 'touchcancel', dragEnd, false );

            function dragStart ( e ) {
                options.startX = e.type === 'touchstart' ? e.touches[0].clientX : e.clientX;
                options.startY = e.type === 'touchstart' ? e.touches[0].clientY : e.clientY;
                options.isDragging = true;
                if ( e.type === 'touchstart' ) {
                    options.wrapper.addEventListener('touchmove', dragAction, false);
                } else {
                    options.wrapper.addEventListener('mousemove', dragAction, false);
                }
            }
            function dragEnd( e ) {
                options.isDragging = false;
                options.wrapper.removeEventListener( 'mousemove', dragAction, false );
                options.wrapper.removeEventListener( 'touchmove', dragAction, false );
            }

        }


        if ( options.switchingMode !== 'fade' ) {

            let spacingValue = parseInt( window.getComputedStyle( options.children[0], null ).getPropertyValue('padding-top' ), 10 );

            // switching slides
            if ( options.prev !== null ) {
                options.prev.addEventListener( 'click', () => {
                    prevSlide();
                } );
            }


            if ( options.next !== null ) {
                options.next.addEventListener( 'click', () => {
                    nextSlide()
                } );
            }

            if ( options.switchingMode === 'to-top' ) options.wrapper.style.flexDirection = 'column';

            if ( options.switchingMode === 'to-bottom' ) options.wrapper.style.flexDirection = 'column-reverse';

            if ( options.switchingMode === 'to-right' ) options.wrapper.style.flexDirection = 'row-reverse';

            if ( options.switchingMode === 'to-bottom' || options.switchingMode === 'to-top' ) {

                // set equal height for all slides
                const slides = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-slide-wrapper .mdp-reviewer-elementor-slide` );
                let slideHeight = 0;

                for ( let slide of slides ) {

                    // Save height of the longest tab
                    if ( slideHeight < slide.getBoundingClientRect().height ) {

                        slideHeight = slide.getBoundingClientRect().height

                    }

                }

                slides.forEach( slide => {
                    slide.style.maxHeight = slideHeight + 'px';
                    slide.style.height = slideHeight + 'px';
                } );

                // set height for slider for to-bottom and to-top switching mode
                if ( options.centerMode === 'yes' ) {
                    spacingValue = 35;
                    document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel-wrapper` ).style.maxHeight = options.children[0].offsetHeight * options.slidesToShow + (  spacingValue * 3 ) + 'px';
                    document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel-wrapper` ).style.height = options.children[0].offsetHeight * options.slidesToShow + ( spacingValue * 3 ) + 'px';
                } else {
                    document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel-wrapper` ).style.maxHeight = options.children[0].offsetHeight * options.slidesToShow  + 'px';
                    document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-carousel-wrapper` ).style.height = options.children[0].offsetHeight * options.slidesToShow  + 'px';
                }

            }


            function slidesSwitchingModes( position, offset ) {
                switch ( options.switchingMode ) {
                    case 'to-left':
                        options.wrapper.style.transform = `translate3d(${offset}px, 0, 0)`;
                        break;
                    case 'to-right':
                        options.wrapper.style.transform = `translate3d(${offset * -1}px, 0, 0)`;
                        break;
                    case 'to-bottom':
                        options.wrapper.style.transform = `translate3d(0, ${offset * -1}px, 0)`;
                        break;
                    case 'to-top':
                        options.wrapper.style.transform = `translate3d(0, ${offset}px, 0)`;
                        break
                }
            }


            // setting slides to show
            for ( let i = 0; i < options.maxPosition; i++ ) {
                if ( options.switchingMode !== 'to-bottom' && options.switchingMode !== 'to-top' ) {
                    options.children[i].style.flexBasis = 100 / options.slidesToShow + '%';
                    options.children[i].style.width = 100 / options.slidesToShow + '%';
                }
            }

            if ( options.dotsWrapper !== null ) generateDots( options.centerMode === 'yes' ? options.maxPosition : Math.ceil( options.maxPosition - ( options.slidesToShow - 1 ) ) );

            const activeDotsIcons = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-dot-active-icon` );
            const activeDots = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-dot-active-icon--active` );
            const dots = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-dot` );


            if ( options.dotsWrapper && activeDots.length <= 0 && dots.length > 0 ) {
                activeDotsIcons[0].classList.add( 'mdp-reviewer-elementor-dot-active-icon--active' );
            }

            // switch slide on click on dots
            if ( dots ) {
                dots.forEach( ( dot, index ) => {
                    dot.addEventListener( 'click', () => {
                        const slideDimension = options.switchingMode === 'to-bottom' || options.switchingMode === 'to-top' ? options.children[0].getBoundingClientRect().height : options.children[0].getBoundingClientRect().width;
                        options.position = index;
                        if ( options.centerMode === 'yes' ) {
                            options.offset = options.switchingMode === 'to-bottom' || options.switchingMode === 'to-top' ? ( slideDimension * ( options.position - Math.floor( options.slidesToShow / 2 ) ) * -1 ) + ( spacingValue * 1.5 ) : slideDimension * ( options.position - Math.floor( options.slidesToShow / 2 ) ) * -1;
                        } else {
                            options.offset = slideDimension * options.position * -1;
                        }
                        slidesSwitchingModes( options.position, options.offset );
                        if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, index );
                    } );
                } );

                if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, options.position );
            }

            setTimeout( () => {
                // center mode initial position settings
                if ( options.centerMode === 'yes' ) {
                    if ( options.switchingMode === 'to-right' || options.switchingMode === 'to-left' ) {
                        options.wrapper.style.width = '85%';
                        options.wrapper.style.margin = '0 auto';
                        options.offset = options.children[0].offsetWidth * Math.floor( options.slidesToShow / 2 );
                        options.wrapper.style.transform = `translate3d(${options.switchingMode === 'to-left' ? options.offset : -options.offset}px, 0, 0)`;
                    } else if ( options.switchingMode === 'to-top' || options.switchingMode === 'to-bottom' ) {
                        options.offset = ( options.children[0].getBoundingClientRect().height * ( options.position - Math.floor( options.slidesToShow / 2 ) ) * -1 ) + ( spacingValue * 1.5 );
                        options.wrapper.style.transform = `translate3d(0, ${options.switchingMode === 'to-top' ? options.offset : -options.offset}px, 0)`;
                    }
                }
            }, 100 );

            // carousel switching slides methods

            // previous slide
            function prevSlide() {
                const slideDimension = options.switchingMode === 'to-bottom' || options.switchingMode === 'to-top' ? options.children[0].getBoundingClientRect().height : options.children[0].getBoundingClientRect().width;
                options.position -= options.slidesToScroll;
                options.offset += slideDimension * options.slidesToScroll;

                if ( options.centerMode === 'yes' ) {
                    if ( options.position < 0 ) {
                        const endPosition = options.slidesToShow % 2 === 0 ? ( slideDimension * ( options.maxPosition - Math.ceil( options.slidesToShow / 2 ) - 1 ) ) * -1 : ( slideDimension * ( options.maxPosition - Math.ceil( options.slidesToShow / 2 ) ) ) * -1;
                        options.position = options.infiniteLoop === 'yes' ? options.maxPosition - 1 : 0;
                        options.offset = options.infiniteLoop === 'yes' ? options.switchingMode === 'to-top' || options.switchingMode === 'to-bottom' ? endPosition + ( spacingValue * 1.5 ) : endPosition : slideDimension;
                    }
                } else {
                    if ( options.position < 0 ) {
                        const endPosition = ( slideDimension * ( options.maxPosition - options.slidesToShow ) ) * -1;
                        options.position = options.infiniteLoop === 'yes' ? options.maxPosition - ( options.slidesToShow - 1 ) - 1 : 0;
                        options.offset = options.infiniteLoop === 'yes' ? endPosition : slideDimension;
                    }
                }

                if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, options.position );

                slidesSwitchingModes( options.position, options.offset );
            }

            // next slide
            function nextSlide() {

                const slideDimension = options.switchingMode === 'to-bottom' || options.switchingMode === 'to-top' ? options.children[0].getBoundingClientRect().height : options.children[0].getBoundingClientRect().width;

                options.position += options.slidesToScroll;
                options.offset -= slideDimension * options.slidesToScroll;


                if ( options.centerMode === 'yes' ) {
                    if ( options.position >= options.maxPosition ) {
                        options.position = options.infiniteLoop === 'yes' ? 0 : options.maxPosition - 1;
                        options.offset = options.infiniteLoop === 'yes' ? options.switchingMode === 'to-top' || options.switchingMode === 'to-bottom' ? ( slideDimension * Math.floor( options.slidesToShow / 2 ) ) + ( spacingValue * 1.5 ) : slideDimension * Math.floor( options.slidesToShow / 2 ) : ( slideDimension * ( options.maxPosition - options.slidesToShow + 1 ) ) * -1;
                    }
                } else {
                    if ( options.position >= options.maxPosition - ( options.slidesToShow - 1 ) ) {
                        options.position = options.infiniteLoop === 'yes' ? 0 : options.maxPosition - ( options.slidesToShow );
                        options.offset = options.infiniteLoop === 'yes' ? 0 : ( slideDimension * ( options.maxPosition - options.slidesToShow ) ) * -1;
                    }
                }

                slidesSwitchingModes( options.position, options.offset );
                if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, options.position );

            }

            // drag action
            function dragAction( e ) {
                if ( options.switchingMode === 'to-top' || options.switchingMode === 'to-bottom' ) {
                    e.preventDefault();
                }

                if ( !options.isDragging ) return;

                const xDiff = 'touchmove' === e.type ?
                    Math.round( options.startX - e.touches[0].clientX ) :
                    options.startX - e.clientX;

                const yDiff = 'touchmove' === e.type ?
                    Math.round( options.startY - e.touches[0].clientY ) :
                    options.startY - e.clientY;

                if ( Math.abs( xDiff ) > Math.abs( yDiff ) ) {
                    // exit if not horizontal swipe
                    if ( options.switchingMode === 'to-top' || options.switchingMode === 'to-bottom' ) { return; }

                    if ( options.switchingMode === 'to-left' ) {
                        if ( xDiff > 0 ) {
                            nextSlide();
                        } else if ( xDiff < 0 ) {
                            prevSlide();
                        }
                    } else {
                        if ( xDiff < 0 ) {
                            nextSlide();
                        } else if ( xDiff > 0 ) {
                            prevSlide();
                        }
                    }
                } else {
                    // exit if not vertical swipe
                    if ( options.switchingMode === 'to-left' || options.switchingMode === 'to-right' ) { return; }

                    if ( options.switchingMode === 'to-top' ) {
                        if ( yDiff > 0 ) {
                            nextSlide();
                        } else if ( yDiff < 0 ) {
                            prevSlide();
                        }
                    } else {
                        if ( yDiff < 0 ) {
                            nextSlide();
                        } else if ( yDiff > 0 ) {
                            prevSlide();
                        }
                    }
                }



                options.isDragging = false;
            }

            dragSlides( dragAction );

            // slides transition
            if ( options.transitionSpeed ) options.wrapper.style.transitionDuration = options.transitionSpeed + 's';

            // carousel autoplay
            if ( options.autoplay === 'yes' ) {
                autoplay.call( this, nextSlide );
            }
        } else {

            // set initial values
            options.wrapper.style.display = 'block';

           for ( let i = 0; i < options.children.length; i++ ) {
               options.children[i].style.flexBasis = '100%';
               options.children[i].style.display = 'none';
               options.children[i].style.animation = 'fade_in_show';
               options.children[i].style.animationDuration = options.transitionSpeed + 's';
           }

           options.children[0].classList.add( 'mdp-reviewer-elementor-slide--active' );

            if ( options.dotsWrapper !== null ) generateDots( options.maxPosition );

            const activeDotsIcons = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-dot-active-icon` );
            const dots = document.querySelectorAll( `.${wrapperName} .mdp-reviewer-elementor-dot` );

            // switching dots on click
            if ( dots ) {
                dots.forEach( ( dot, index ) => {
                    dot.addEventListener( 'click', () => {
                        const $currentElement = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-slide--active` );
                        $currentElement.classList.remove( 'mdp-reviewer-elementor-slide--active' );
                        options.children[index].classList.add( 'mdp-reviewer-elementor-slide--active' );
                        if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, index );
                    } );
                } );
                if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, 0 );
            }

            // switching slides
            function nextSlideFade() {
                const $currentElement = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-slide--active` );
                const nextElement = $currentElement.nextElementSibling;

                if ( !nextElement ) {
                    $currentElement.classList.remove( 'mdp-reviewer-elementor-slide--active' );
                    options.children[options.infiniteLoop === 'yes' ? 0 : options.maxPosition - 1].classList.add( 'mdp-reviewer-elementor-slide--active' );
                    if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, options.infiniteLoop === 'yes' ? 0 : options.maxPosition - 1 )
                } else {
                    $currentElement.classList.remove( 'mdp-reviewer-elementor-slide--active' );
                    if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons,+$currentElement.dataset.index + 1 );
                    nextElement.classList.add( 'mdp-reviewer-elementor-slide--active' );
                }
            }

            function prevSlideFade() {
                const $currentElement = document.querySelector( `.${wrapperName} .mdp-reviewer-elementor-slide--active` );
                const previousElement = $currentElement.previousElementSibling;

                if ( !previousElement ) {
                    $currentElement.classList.remove( 'mdp-reviewer-elementor-slide--active' );
                    options.children[options.infiniteLoop === 'yes' ? options.children.length - 1 : 0].classList.add( 'mdp-reviewer-elementor-slide--active' );
                    if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, options.infiniteLoop === 'yes' ? options.children.length - 1 : 0 )
                } else {
                    $currentElement.classList.remove( 'mdp-reviewer-elementor-slide--active' );
                    if ( options.dotsWrapper !== null ) switchActiveDots( activeDotsIcons, +$currentElement.dataset.index - 1 );
                    previousElement.classList.add( 'mdp-reviewer-elementor-slide--active' );
                }
            }

           // next slide
            options.next.addEventListener( 'click', () => {
               nextSlideFade();
            } );

           // prev slide
           options.prev.addEventListener( 'click', () => {
                prevSlideFade();
           } );

           // autoplay
            if ( options.autoplay === 'yes' ) {
                autoplay.call( this, nextSlideFade );
            }

          // drag action
            function dragActionFade( e ) {
                if ( !options.isDragging ) return;

                const xDiffFade = 'touchmove' === e.type ?
                    Math.round( options.startX - e.touches[0].clientX ) :
                    options.startX - e.clientX;

                const yDiffFade = 'touchmove' === e.type ?
                    Math.round( options.startY - e.touches[0].clientY ) :
                    options.startY - e.clientY;

                if ( Math.abs( xDiffFade ) > Math.abs( yDiffFade ) ) {
                    if ( xDiffFade > 0 ) {
                        nextSlideFade();
                    } else if ( xDiffFade < 0 ) {
                        prevSlideFade();
                    }
                }
                options.isDragging = false;
            }

            dragSlides( dragActionFade );

        }
    },

    addReviewer: function () {

        this.intervals.forEach( interval => {
            clearInterval( interval );
        } );

        this.intervals = [];

        const reviewerBox = document.querySelectorAll( '.mdp-reviewer-elementor-box' );

        for ( let i = 0; i < reviewerBox.length; i++ ) {
            reviewerBox[i].classList.add( 'mdp-reviewer-elementor-box-' + i );
            this.reviewer( 'mdp-reviewer-elementor-box-' + i );
        }

    }

}

/**
 * Init for Front-End
 * @param callback
 */
document.addEventListener( 'DOMContentLoaded', mdpReviewer.addReviewer.bind( mdpReviewer ) );