"use strict"

const mdpScrollingEffect = {

    addScrollingEffect: function ( $element, settings, wrapperName ) {

        if ( !$element ) { return; }

        if ( !document.querySelector( `.${wrapperName}` ) ) return;


        $element.style.transform = this.setTransformStyles( $element, settings );

        $element.style.opacity = this.setTransparencyScrollEffect( $element, settings );

        $element.style.filter = this.setBlurScrollEffect( $element, settings );

        $element.style.transition = `all ${settings.mdp_selection_scrolling_effect_transition_duration.size}s`;

            const addStyles = () => {

                if ( !document.querySelector( `.${wrapperName}` ) ) {
                    window.removeEventListener( 'scroll', addStyles );

                } else {

                    $element.style.transform = this.setTransformStyles($element, settings);

                    $element.style.opacity = this.setTransparencyScrollEffect($element, settings);

                    $element.style.filter = this.setBlurScrollEffect($element, settings);
                }
            }

            window.addEventListener( 'scroll', addStyles );
    },

    setScrollingEffect: function ( $element ) {
        const isEditor = elementorFrontend.isEditMode();
        const elementsEditorData = isEditor ? elementorFrontend.config.elements.data : [];
        let settings = {};
        if ( isEditor ) {
            if ( $element.dataset.modelCid ) {
                const elementsDataset = elementsEditorData[$element.dataset.modelCid].attributes;
                for ( let key in elementsDataset ) {
                    if ( key.startsWith( 'mdp_selection_scrolling_effect' ) ) {
                        settings[key] = elementsDataset[key];
                    }
                }
            } else if ( $element.dataset.settings ) {
                settings = JSON.parse( $element.dataset.settings );
            }
        } else if ( !isEditor && $element.dataset.settings ) {
            settings = JSON.parse( $element.dataset.settings );
        }

        if ( settings.mdp_selection_scrolling_effect_enable === 'yes' ) {

            $element.classList.add( `mdp-selection-scrolling-effect-${$element.dataset.id}` );

            this.checkScreenSize(
                settings.mdp_selection_scrolling_effect_enable_on,
                this.addScrollingEffect.bind( mdpScrollingEffect, $element, settings, `mdp-selection-scrolling-effect-${$element.dataset.id}` )
            );
        } else {
            $element.classList.remove( `mdp-selection-scrolling-effect-${$element.dataset.id}` );
            $element.style.removeProperty( 'transform' );
            $element.style.removeProperty( 'opacity' );
            $element.style.removeProperty( 'transition' );
            $element.style.removeProperty( 'filter' );
        }
    },

    setTransformStyles( $element, settings ) {

        let transformStyles = '';

        // vertical scroll effect
        if ( settings.mdp_selection_scrolling_effect_vertical_enable === 'yes' ) {
            transformStyles += this.transformScrollEffect(
                $element,
                settings.mdp_selection_scrolling_effect_vertical_speed.size,
                settings.mdp_selection_scrolling_effect_vertical_direction,
                settings.mdp_selection_scrolling_effect_vertical_viewport,
                'vertical',
                settings.mdp_selection_scrolling_effect_relative_to
            );
        }

        // horizontal scroll effect
        if ( settings.mdp_selection_scrolling_effect_horizontal_enable === 'yes' ) {
            transformStyles += this.transformScrollEffect(
                $element,
                settings.mdp_selection_scrolling_effect_horizontal_speed.size,
                settings.mdp_selection_scrolling_effect_horizontal_direction,
                settings.mdp_selection_scrolling_effect_horizontal_viewport,
                'horizontal',
                settings.mdp_selection_scrolling_effect_relative_to
            );
        }

        // rotate scroll effect
        if ( settings.mdp_selection_scrolling_effect_rotate_enable === 'yes' ) {
            $element.style.transformOrigin = `${settings.mdp_selection_scrolling_effect_x_anchor} 
                    ${settings.mdp_selection_scrolling_effect_y_anchor}`;
            transformStyles += this.transformScrollEffect(
                $element,
                settings.mdp_selection_scrolling_effect_rotate_speed.size,
                settings.mdp_selection_scrolling_effect_rotate_direction,
                settings.mdp_selection_scrolling_effect_rotate_viewport,
                'rotate',
                settings.mdp_selection_scrolling_effect_relative_to
            );
        }

        // scale scroll effect
        if( settings.mdp_selection_scrolling_effect_scale_enable === 'yes' ) {
            $element.style.transformOrigin = `${settings.mdp_selection_scrolling_effect_x_anchor} 
                    ${settings.mdp_selection_scrolling_effect_y_anchor}`;
            transformStyles += this.transformScrollEffect(
                $element,
                settings.mdp_selection_scrolling_effect_scale_speed.size,
                settings.mdp_selection_scrolling_effect_scale_direction,
                settings.mdp_selection_scrolling_effect_scale_viewport.sizes,
                'scale',
                settings.mdp_selection_scrolling_effect_relative_to
            );
        }

        return transformStyles;
    },

    getPassedPercents: function ( $element ) {
        const elementOffset = $element.getBoundingClientRect(),
            offsetStart = 0,
            offsetEnd = 0,
            windowStartOffset = window.innerHeight * offsetStart / 100,
            windowEndOffset = window.innerHeight * offsetEnd / 100,
            y1 = elementOffset.top - window.innerHeight,
            y2 = elementOffset.top + windowStartOffset + elementOffset.height,
            startPosition = 0 - y1 + windowStartOffset,
            endPosition = y2 - y1 + windowEndOffset,
            percent = Math.max( 0, Math.min( startPosition / endPosition, 1 ) );
        return parseFloat( ( percent * 100 ).toFixed( 2 ) );
    },

    getPassedPageScrollPercents: function () {
        const offsetStart = 0,
            offsetEnd = 0,
            initialPageHeight = document.documentElement.scrollHeight - document.documentElement.clientHeight,
            heightOffset = initialPageHeight * offsetStart / 100,
            pageRange = initialPageHeight + heightOffset + initialPageHeight * offsetEnd / 100,
            scrollPos = document.documentElement.scrollTop + document.body.scrollTop + heightOffset;
        return scrollPos / pageRange * 100;
    },

    updateTransform: function( direction, passedScrollPercents, speed, viewport ) {

        if ( viewport.sizes.start > passedScrollPercents ) {
            passedScrollPercents = viewport.sizes.start;
        }

        if ( viewport.sizes.end < passedScrollPercents ) {
            passedScrollPercents = viewport.sizes.end;
        }

        if ( direction === 'down' || direction === 'right' ) {
            passedScrollPercents = 100 - passedScrollPercents;
        }

        return -( passedScrollPercents - 50 ) * speed;
    },

    transformScrollEffect: function ( $element, speed, direction, viewport, type, relativeTo ) {

        let transform = '';

        const passedScrollPercents = relativeTo === 'entire-page' ?
            this.getPassedPageScrollPercents() :
            this.getPassedPercents( $element.parentNode );

        if ( type === 'vertical' ) {
            transform = `translateY(${this.updateTransform( 
                direction, 
                passedScrollPercents, 
                speed, 
                viewport 
            )}px) `;
        } else if ( type === 'horizontal' ) {
            transform = `translateX(${this.updateTransform( 
                direction, 
                passedScrollPercents, 
                speed, 
                viewport 
            )}px) `;
        } else if ( type === 'rotate' ) {
            transform = `rotateZ(${this.updateTransform(
                direction,
                passedScrollPercents,
                speed,
                viewport
            )}deg) `;
        } else if ( type === 'scale' ) {
            transform = `scale(${this.updateScaleEffect(
                $element,
                speed,
                direction,
                viewport,
                relativeTo
            )}) `;
        }

        return transform;
    },


    getPassedPercentsMovePoint: function ( passedScrollPercents, viewport, direction ) {
        let movePoint;

        if ( passedScrollPercents < viewport.start ) {
            if ( direction === 'effect-in' ) {
                movePoint = 0;
            } else if ( direction === 'effect-out' ) {
                movePoint = 100;
            } else {
                movePoint = ( passedScrollPercents / viewport.start * 100 ).toFixed( 2 );

                if ( direction === 'effect-out-in' ) {
                    movePoint = 100 - movePoint;
                }
            }
        } else if ( passedScrollPercents < viewport.end ) {
            if ( direction === 'effect-out-in' ) {
                movePoint = 0;
            } else if ( direction === 'effect-in-out' ) {
                movePoint = 100;
            } else {
                movePoint = ( ( passedScrollPercents - viewport.start ) /
                    ( viewport.end - viewport.start ) * 100 ).toFixed( 2 );

                if ( direction === 'effect-out' ) {
                    movePoint = 100 - movePoint;
                }
            }
        } else if ( direction === 'effect-out' ) {
            movePoint = 0;
        } else if ( direction === 'effect-in' ) {
            movePoint = 100;
        } else {
            movePoint = ( ( 100 - passedScrollPercents ) / ( 100 - viewport.end ) * 100 ).toFixed( 2 );

            if ( direction === 'effect-out-in' ) {
                movePoint = 100 - movePoint;
            }
        }

        return movePoint;

    },

    updateScaleEffect: function ( $element, speed, direction, viewport, relativeTo ) {
        const step = this.getPassedPercentsMovePoint(
            relativeTo === 'viewport' ?
                this.getPassedPercents( $element.parentNode ) :
                this.getPassedPageScrollPercents(),
            viewport,
            direction
        );

        return 1 + speed * step / 1000;

    },

    updateOpacity: function ( $element, levelOpt, direction, viewport, relativeTo ) {
        const passedScrollPercents = relativeTo === 'viewport' ?
            this.getPassedPercents( $element.parentNode ) :
            this.getPassedPageScrollPercents()
        const step = this.getPassedPercentsMovePoint(
            passedScrollPercents,
            viewport,
            direction
        );
        const level = levelOpt / 10;
        return 1 - ( level ) + ( level * step / 100 );
    },

    setTransparencyScrollEffect: function ( $element, settings ) {

        if ( settings.mdp_selection_scrolling_effect_transparency_enable === 'yes' ) {
            return this.updateOpacity(
                $element,
                settings.mdp_selection_scrolling_effect_transparency_speed.size,
                settings.mdp_selection_scrolling_effect_transparency_direction,
                settings.mdp_selection_scrolling_effect_transparency_viewport.sizes,
                settings.mdp_selection_scrolling_effect_relative_to,
            );
        } else {
            return '';
        }

    },

    updateBlur: function ( $element, level, viewport, direction, relativeTo ) {

        let passedScrollPercents = relativeTo === 'viewport' ?
            this.getPassedPercents( $element.parentNode ) :
            this.getPassedPageScrollPercents();

        if ( viewport.start > passedScrollPercents ) {
            passedScrollPercents = viewport.start;
        }

        if ( viewport.end < passedScrollPercents ) {
            passedScrollPercents = viewport.end;
        }

        const step = this.getPassedPercentsMovePoint( passedScrollPercents, viewport, direction );
        return level - level * step / 100;

    },

    setBlurScrollEffect: function ( $element, settings ) {

        if ( settings.mdp_selection_scrolling_effect_blur_enable === 'yes' ) {
            return `blur(${this.updateBlur( 
                $element,
                settings.mdp_selection_scrolling_effect_blur_speed.size,
                settings.mdp_selection_scrolling_effect_blur_viewport.sizes,
                settings.mdp_selection_scrolling_effect_blur_direction,
                settings.mdp_selection_scrolling_effect_relative_to
            )}px)`;
        } else {
            return '';
        }
    },

    checkScreenSize: function ( enableOn, callback ) {

        if ( !enableOn ) return;

        if ( enableOn.includes( 'mobile' ) && window.matchMedia(
        `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile.value}px)` ).matches ) {
            callback();
        }

        if ( enableOn.includes( 'tablet' ) && window.matchMedia(
            `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.mobile.value + 1}px)`
            ).matches
            && window.matchMedia(
                `(max-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value}px)`
            ).matches ) {
            callback();
        }

        if ( enableOn.includes( 'desktop' ) && window.matchMedia(
            `(min-width: ${elementorFrontend.config.responsive.activeBreakpoints.tablet.value + 1}px)`
        ).matches ) {
            callback();
        }
    }

}


jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        mdpScrollingEffect.setScrollingEffect.call( mdpScrollingEffect, $scope[0], null );

    } );

} );