const mdpSearcherElementor = {
    searcherElementor: function ( wrapperName ) {
        this.asyncSearch( wrapperName );
    },

    asyncSearch: function ( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-searcher-elementor-box` );
        const $searchField = document.querySelector( `.${wrapperName} .mdp-searcher-elementor-search-field` );
        const $resultBox = document.querySelector( `.${wrapperName} .mdp-searcher-elementor-search-result-box` );
        const $searchForm = document.querySelector( `.${wrapperName} .mdp-searcher-elementor-search-form` );
        const $searchButton = document.querySelector( `.${wrapperName} .mdp-searcher-elementor-search-button` );
        const formHeight = $searchForm.getBoundingClientRect().height;

        const postTypes = $box.dataset.postTypes;
        const resultsQty = $box.dataset.searchResultsQty === '' ? 3 : $box.dataset.searchResultsQty;
        const formId = $box.dataset.formId;
        const postId = $box.dataset.postId;
        const requestDelay = $box.dataset.requestDelay;
        const emptyText = $box.dataset.emptyText;
        const searchType = $box.dataset.searchType;

        if ( searchType !== 'async' ) { return; }

        $searchField.addEventListener( 'input', event => {
            if ( event.target.value.length >= 3 ) {
                setTimeout( () => {
                    $resultBox.style.display = 'block';
                    if ( event.target.value === '' ) {
                        $resultBox.style.display = 'none';
                    }
                    $resultBox.style.top = formHeight + 'px';
                    this.getSearchResults( event.target.value, postTypes, resultsQty, formId, postId ).then( res => {
                        $resultBox.innerHTML = '';
                        const result = JSON.parse( res );
                        const resultPosts = result.data;
                        console.log( resultPosts );
                        if ( resultPosts.is_empty ) {
                            $resultBox.innerHTML = emptyText;
                            $searchButton.disabled = true;
                        } else {
                            resultPosts.forEach( post => {
                                $resultBox.innerHTML += post;
                            } );
                            $searchButton.disabled = false;
                        }
                    } );
                }, requestDelay );
            } else {
                $resultBox.innerHTML = '';
                $resultBox.style.display = 'none';
            }
        } );

    },

    getSearchResults: function ( searchValue, postTypes, resultsQuantity, formId, postId ) {
        const xhr = new XMLHttpRequest();

        const url = new URL( mdpSearcherObjectElementorWP.endpoint );

        url.searchParams.set( 'action', 'mdp_async_search' );
        url.searchParams.set( 'mdp_search_value', searchValue );
        url.searchParams.set( 'mdp_post_types', postTypes );
        url.searchParams.set( 'mdp_results_quantity', resultsQuantity );
        url.searchParams.set( 'mdp_form_id', formId );
        url.searchParams.set( 'mdp_post_id', postId );
        url.searchParams.set( 'mdp_searcher_nonce', mdpSearcherObjectElementorWP.nonce );

        return new Promise( ( resolve, reject ) => {
            xhr.open( 'GET', url, true );
            xhr.onload = () => resolve( xhr.responseText );
            xhr.onerror = () => reject( xhr.statusText );
            xhr.send();
        } );

    }


};

/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-searcher-elementor' ) ) {
            mdpSearcherElementor.searcherElementor.call( mdpSearcherElementor, `elementor-element-${$scope[0].dataset.id}` );
        }
    } );

} );