const mdpUpsellerElementor = {

    upsellProducts: function ( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-upseller-elementor-upsells-box` );

        if ( !$box ) { return; }

        const columnsSettings = this.setColumnsSettings( $box );

        this.setProductsDisplaying( $box.querySelector( 'ul.products' ) , columnsSettings );

    },

    relatedProducts: function ( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-upseller-elementor-related-products-box` );

        if ( !$box ) { return; }

        const columnsSettings = this.setColumnsSettings( $box );

        this.setProductsDisplaying( $box.querySelector( 'ul.products' ), columnsSettings );

    },

    setColumnsSettings: function ( $box ) {
        return  {
            widescreenColumns: $box.dataset.columnsWidescreen,
            desktopColumns: $box.dataset.columns,
            laptopColumns: $box.dataset.columnsLaptop,
            tabletColumns: $box.dataset.columnsTablet,
            tabletExtraColumns: $box.dataset.columnsTablet_extra,
            mobileExtraColumns: $box.dataset.columnsMobile_extra,
            mobileColumns: $box.dataset.columnsMobile
        };
    },

    setProductsDisplaying: function ( wrapper, columnsSettings ) {

        if ( !wrapper ) { return; }

        const activeBreakpoints = elementorFrontend.config.responsive.activeBreakpoints;

        setUpseller = () => {
            // set responsive columns
            if ( activeBreakpoints.hasOwnProperty( 'widescreen' ) &&
                window.matchMedia( `(min-width: ${activeBreakpoints.widescreen.value}px)` ).matches &&
                columnsSettings.widescreenColumns ) {
                this.setColumns( wrapper, columnsSettings.widescreenColumns );
            }

            if ( columnsSettings.widescreenColumns && activeBreakpoints.hasOwnProperty( 'widescreen' ) ) {
                if ( window.matchMedia( `(min-width: ${activeBreakpoints.tablet.value + 1}px)` ).matches
                    && window.matchMedia( `(max-width: ${activeBreakpoints.widescreen.value - 1}px)` ).matches ) {
                    this.setColumns( wrapper, columnsSettings.desktopColumns );
                }
            } else {
                if ( window.matchMedia( `(min-width: ${activeBreakpoints.tablet.value + 1}px)` ).matches ) {
                    this.setColumns( wrapper, columnsSettings.desktopColumns );
                }
            }

            if ( activeBreakpoints.hasOwnProperty( 'laptop' )
                && window.matchMedia( `(max-width: ${activeBreakpoints.laptop.value}px)` ).matches
                && columnsSettings.laptopColumns ) {
                this.setColumns( wrapper, columnsSettings.laptopColumns );
            }

            if ( activeBreakpoints.hasOwnProperty( 'tablet_extra' )
                && window.matchMedia( `(max-width: ${activeBreakpoints.tablet_extra.value}px) ` ).matches
                && columnsSettings.tabletExtraColumns ) {
                this.setColumns( wrapper, columnsSettings.tabletExtraColumns );
            }

            if ( window.matchMedia( `(max-width: ${activeBreakpoints.tablet.value}px)` ).matches &&
                columnsSettings.tabletColumns ) {
                this.setColumns( wrapper, columnsSettings.tabletColumns );
            }

            if ( activeBreakpoints.hasOwnProperty( 'mobile_extra' )
                && window.matchMedia( ` (max-width: ${activeBreakpoints.mobile_extra.value}px) ` ).matches
                && columnsSettings.mobileExtraColumns ) {
                this.setColumns( wrapper, columnsSettings.mobileExtraColumns );
            }

            if ( window.matchMedia( `(max-width: ${activeBreakpoints.mobile.value}px)` ).matches &&
                columnsSettings.mobileColumns ) {
                this.setColumns( wrapper, columnsSettings.mobileColumns );
            }
        }

        setUpseller();

        window.addEventListener( 'resize', setUpseller );

    },

    setColumns: function ( $wrapper, columnsQuantity ) {
        $wrapper.style.gridTemplateColumns = `repeat( ${columnsQuantity}, 1fr )`;
    }

};



/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-upseller-upsells-elementor' ) ) {
            mdpUpsellerElementor.upsellProducts.call( mdpUpsellerElementor, `elementor-element-${$scope[0].dataset.id}` );
        } else if ( $scope[0].classList.contains( 'elementor-widget-mdp-upseller-related-products-elementor' ) ) {
            mdpUpsellerElementor.relatedProducts.call( mdpUpsellerElementor, `elementor-element-${$scope[0].dataset.id}` );
        }

    } );

} );