<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Conformer
 * Customizable 'Contact Form 7' form for Elementor editor
 * Exclusively on https://1.envato.market/conformer-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Conformer - Custom Elementor Widget.
 **/
class conformer_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-cfsevenform-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-cfsevenform-elementor',
            UnityPlugin::get_url() . 'css/cfsevenform-elementor' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-conformer-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Contact Form 7', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-conformer-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Contact Form 7' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-cfsevenform-elementor', 'mdp-cfsevenform-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-conformer' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_general();

        /** Content -> Form Content Section. */
        $this->section_content_form();

        /** Content -> Errors Content Section. */
        $this->section_content_errors();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Title. */
        $this->section_style_title();

        /** Style -> Section Style Description. */
        $this->section_style_description();

        /** Style -> Section Style Label. */
        $this->section_style_label();

        /** Style -> Section Style Input. */
        $this->section_style_input();

        /** Style -> Section Style Textarea. */
        $this->section_style_textarea();

        /** Style -> Section Style Radio Button. */
        $this->section_style_radio_button();

        /** Style -> Section Style Checkbox. */
        $this->section_style_checkbox();

        /** Style -> Section Style Select */
        $this->section_style_select();

        /** Style -> Section Style Button */
        $this->section_style_button();

        /** Style -> Section Style Errors */
        $this->section_style_errors();

        /** Style -> Section Style Response Message */
        $this->section_style_response_message();

    }

    /**
     * Get Contact Form 7 forms.
     *
     * @return array
     * @since 1.0.0
     * @access private
     *
     */
    private function get_cf7_forms() {
        if ( !class_exists( 'WPCF7_ContactForm' ) ) {
            return [];
        }

        $cf7_forms = \WPCF7_ContactForm::find( [ 'orderby' => 'title' ] );

        if ( empty( $cf7_forms ) ) { return []; }

        $result = [];

        foreach ( $cf7_forms as $form ) {
            $key = $form->id();
            $result[ $key ] = $form->title();
        }

        return $result;

    }

    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        if ( ! class_exists( 'WPCF7_ContactForm' ) ) {
            $this->add_control(
                'settings_link',
                [
                    'type' => Controls_Manager::RAW_HTML,
                    'raw' => wp_kses_post('<strong>"Contact Form 7" plugin not installed. Please install "Contact Form 7" plugin!</strong>'),
                    'separator' => 'after',
                    'content_classes' => 'elementor-panel-alert elementor-panel-alert-info',
                ]
            );
        }

        $options = $this->get_cf7_forms();

        $this->add_control(
            'select_form',
            [
                'label'     => esc_html__( 'Select form', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'default'   => !empty( array_keys( $options ) ) ? array_keys( $options )[0] : '',
                'options'   => $options,
            ]
        );

        $this->add_control(
            'show_title',
            [
                'label' => esc_html__( 'Show title', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'form_title',
            [
                'label' => esc_html__( 'Title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Default title', 'selection' ),
                'placeholder' => esc_html__( 'Type your title here', 'selection' ),
                'condition' => [
                    'show_title' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'form_title_alignment',
            [
                'label' => esc_html__( 'Title alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-form-title' => 'text-align: {{VALUE}}'
                ],
                'condition' => [
                    'show_title' => 'yes'
                ],
                'default' => 'left',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'show_description',
            [
                'label' => esc_html__( 'Show description', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'form_description',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'type' => Controls_Manager::WYSIWYG,
                'placeholder' => esc_html__( 'Type your description here', 'selection' ),
                'condition' => [ 'show_description' => 'yes' ]
            ]
        );

        $this->add_control(
            'form_description_alignment',
            [
                'label' => esc_html__( 'Description alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-form-description' => 'text-align: {{VALUE}}'
                ],
                'condition' => [
                    'show_description' => 'yes'
                ],
                'default' => 'left',
                'toggle' => true,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Form Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_form()
    {

        $this->start_controls_section('section_content_form', [
            'label' => esc_html__('Form', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'form_alignment',
            [
                'label' => esc_html__( 'Form alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .wpcf7-form' => 'text-align: {{VALUE}}'
                ],
                'default' => 'left',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'show_labels',
            [
                'label' => esc_html__( 'Show labels', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Errors Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_errors()
    {

        $this->start_controls_section( 'section_content_errors', [
            'label' => esc_html__('Errors', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'validation_errors',
            [
                'label' => esc_html__( 'Show validation errors', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->end_controls_section();
    }


    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'hover_html_class' => array_key_exists( 'hover_html_class', $opts ) ?
                $opts['hover_html_class'] : '',
            'focus_html_class' => array_key_exists( 'focus_html_class', $opts ) ?
                $opts['focus_html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'include_outline' => array_key_exists( 'include_outline', $opts ) ?
                $opts['include_outline'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'FOCUS',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ?
                $opts['additional_color_hover_class'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        $hover_html_class = $style_opts['hover_html_class']  !== '' ?
            $style_opts['hover_html_class'] : "{{WRAPPER}} .".$style_opts['html_class'].":hover";

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => $hover_html_class,
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => $hover_html_class,
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $hover_html_class => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => $hover_html_class,
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab( $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'selection' )] );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'selection' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $focus_html_class = $style_opts['focus_html_class']  !== '' ?
                $style_opts['focus_html_class'] : "{{WRAPPER}} ." . $style_opts['html_class'] . ":focus";

            if ( $style_opts['include_outline'] ) {
                $this->add_control(
                    $section_id . '_outline_style',
                    [
                        'label' => esc_html__( 'Outline', 'selection' ),
                        'type' => Controls_Manager::SELECT,
                        'default' => '',
                        'options' => [
                            'none' => esc_html__( 'None', 'selection' ),
                            '' => esc_html__( 'Default', 'selection' ),
                            'auto' => esc_html__( 'Auto', 'selection' ),
                            'solid'  => esc_html__( 'Solid', 'selection' ),
                            'dotted' => esc_html__( 'Dotted', 'selection' ),
                            'double' => esc_html__( 'Double', 'selection' ),
                            'dashed' => esc_html__( 'Dashed', 'selection' ),
                            'groove' => esc_html__( 'Groove', 'selection' ),
                            'ridge' => esc_html__( 'Ridge', 'selection' ),
                            'inset' => esc_html__( 'Inset', 'selection' ),
                            'outset' => esc_html__( 'Outset', 'selection' ),
                        ],
                        'selectors' => [
                            $focus_html_class => 'outline-style: {{VALUE}};',
                        ],
                    ]
                );

                $this->add_control(
                    $section_id . '_outline_color',
                    [
                        'label' => esc_html__( 'Outline color', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'condition' => [
                            $section_id . '_outline_style!' => 'none'
                        ],
                        'selectors' => [
                            $focus_html_class => 'outline-color: {{VALUE}} !important;',
                        ],
                    ]
                );

                $this->add_control(
                    $section_id . '_custom_outline_width',
                    [
                        'label' => esc_html__( 'Custom outline width', 'selection' ),
                        'type' => Controls_Manager::SWITCHER,
                        'return_value' => 'yes',
                        'default' => 'no',
                        'condition' => [
                            $section_id . '_outline_style!' => 'none'
                        ],
                    ]
                );

                $this->add_control(
                    $section_id . '_outline_width',
                    [
                        'label' => esc_html__( 'Outline width', 'selection' ),
                        'type' => Controls_Manager::SELECT,
                        'default' => '',
                        'options' => [
                            '' => esc_html__( 'Default', 'selection' ),
                            'thin' => esc_html__( 'Thin', 'selection' ),
                            'medium'  => esc_html__( 'Medium', 'selection' ),
                            'thick' => esc_html__( 'Thick', 'selection' ),
                        ],
                        'condition' => [
                            $section_id . '_custom_outline_width!' => 'yes',
                            $section_id . '_outline_style!' => 'none'
                        ],
                        'selectors' => [
                            $focus_html_class => 'outline-width: {{VALUE}};',
                        ],
                    ]
                );

                $this->add_responsive_control(
                    $section_id . '_outline_custom_width',
                    [
                        'label' => esc_html__( 'Outline width', 'selection' ),
                        'type' => Controls_Manager::SLIDER,
                        'size_units' => [ 'px', 'em' ],
                        'range' => [
                            'px' => [
                                'min' => 0,
                                'max' => 500,
                                'step' => 1,
                            ],
                            'em' => [
                                'min' => 0.1,
                                'max' => 100,
                                'step' => 0.1
                            ]
                        ],
                        'condition' => [
                            $section_id . '_custom_outline_width' => 'yes',
                            $section_id . '_outline_style!' => 'none'
                        ],
                        'selectors' => [
                            $focus_html_class => 'outline-width: {{SIZE}}{{UNIT}};',
                        ],
                    ]
                );


            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_active',
                    'label' => esc_html__( 'Border Type', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_title() {

        $this->start_controls_section( 'section_style_title', [
            'label' => esc_html__( 'Title', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_title',
            'mdp-cfsevenform-elementor-form-title'
        );

        $this->generate_typography_tabs_controls( 'section_style_title', [
            'html_class' => 'mdp-cfsevenform-elementor-form-title',
            'color_class' => 'mdp-cfsevenform-elementor-form-title',
            'color_hover_class' => 'mdp-cfsevenform-elementor-form-title:hover',
            'typography_class' => 'mdp-cfsevenform-elementor-form-title'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Description.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_description() {

        $this->start_controls_section( 'section_style_description', [
            'label' => esc_html__( 'Description', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_description',
            'mdp-cfsevenform-elementor-form-description'
        );

        $this->generate_typography_tabs_controls( 'section_style_description', [
            'html_class' => 'mdp-cfsevenform-elementor-form-description',
            'color_class' => 'mdp-cfsevenform-elementor-form-description',
            'color_hover_class' => 'mdp-cfsevenform-elementor-form-description:hover',
            'typography_class' => 'mdp-cfsevenform-elementor-form-description'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Field Label.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_label()
    {

        $this->start_controls_section('section_style_label', [
            'label' => esc_html__('Label', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls(
            'section_style_label',
            'mdp-cfsevenform-elementor-box label'
        );


        $this->generate_typography_tabs_controls('section_style_label', [
            'html_class' => 'mdp-cfsevenform-elementor-box label',
            'color_class' => 'mdp-cfsevenform-elementor-box label',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box label:hover',
            'typography_class' => 'mdp-cfsevenform-elementor-box label',
        ]);


        $this->end_controls_section();
    }


    /**
     * Add widget controls: Style -> Section Style Input.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_input() {

        $this->start_controls_section( 'section_style_input', [
            'label' => esc_html__( 'Input', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_input',
            'mdp-cfsevenform-elementor-box input'
        );

        $this->add_control(
            'input_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-text, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'input_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-text, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_input', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]',
            'hover_html_class' => '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-text:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]:hover',
            'active_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz:focus',
            'include_active_tab' => true,
            'additional_color' => true,
            'include_outline' => true,
            'additional_color_name' => 'Placeholder color',
            'color_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]',
            'typography_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]:hover',
            'color_active_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]:focus',
            'additional_color_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]::placeholder',
            'additional_color_active_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text:focus::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date:focus::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number:focus::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz:focus::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]:focus::placeholder',
            'additional_color_hover_class' => 'mdp-cfsevenform-elementor-box .wpcf7-text:hover::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-date:hover::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-number:hover::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz:hover::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]:hover::placeholder',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-text::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-quiz::placeholder, {{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="password"]::placeholder'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Textarea.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_textarea() {

        $this->start_controls_section( 'section_style_textarea', [
            'label' => esc_html__( 'Textarea', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_textarea',
            'mdp-cfsevenform-elementor-box textarea'
        );

        $this->add_control(
            'textarea_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box textarea' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'textarea_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box textarea' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_textarea', [
            'html_class' => 'mdp-cfsevenform-elementor-box textarea',
            'typography_class' => 'mdp-cfsevenform-elementor-box textarea',
            'active_class' => 'mdp-cfsevenform-elementor-box textarea:focus',
            'include_active_tab' => true,
            'additional_color' => true,
            'include_outline' => true,
            'additional_color_name' => 'Placeholder color',
            'color_class' => 'mdp-cfsevenform-elementor-box textarea',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box textarea:hover',
            'color_active_class' => 'mdp-cfsevenform-elementor-box textarea:focus',
            'additional_color_class' => 'mdp-cfsevenform-elementor-box textarea::placeholder',
            'additional_color_hover_class' => 'mdp-cfsevenform-elementor-box textarea:hover::placeholder',
            'additional_color_active_class' => 'mdp-cfsevenform-elementor-box textarea:focus::placeholder',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-cfsevenform-elementor-box textarea::placeholder'
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Radio Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_radio_button() {

        $this->start_controls_section( 'section_style_radio_button', [
            'label' => esc_html__( 'Radio button', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_radio_button',
            'mdp-cfsevenform-elementor-box .wpcf7-radio input[type="radio"]'
        );

        $this->add_control(
            'radio_button_size',
            [
                'label' => esc_html__( 'Scale', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0.1,
                        'max' => 10,
                        'step' => 0.1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box input[type="radio"]' => 'transform: scale({{SIZE}})',
                ],
            ]
        );

        $this->add_control(
            'radio_button_width_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_radio_button', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-radio input[type="radio"]',
            'include_color' => false,
            'include_typography' => false,
            'active_tab_name' => 'CHECKED',
            'include_active_tab' => true,
            'include_outline' => true,
            'active_class' => 'mdp-cfsevenform-elementor-box .wpcf7-radio input[type="radio"]:checked'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Checkbox.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_checkbox() {

        $this->start_controls_section( 'section_style_checkbox', [
            'label' => esc_html__( 'Checkbox', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_checkbox',
            'mdp-cfsevenform-elementor-box .wpcf7-checkbox input[type="checkbox"], {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-acceptance input[type="checkbox"]'
        );

        $this->add_control(
            'checkbox_size',
            [
                'label' => esc_html__( 'Scale', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0.1,
                        'max' => 10,
                        'step' => 0.1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-checkbox input[type="checkbox"], {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-acceptance input[type="checkbox"]' => 'transform: scale({{SIZE}})',
                ],
            ]
        );

        $this->add_control(
            'checkbox_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_checkbox', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-checkbox input[type="checkbox"], {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-acceptance input[type="checkbox"]',
            'hover_html_class' => '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-checkbox input[type="checkbox"]:hover, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-acceptance input[type="checkbox"]:hover',
            'focus_html_class' => '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-checkbox input[type="checkbox"]:focus, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-acceptance input[type="checkbox"]:focus',
            'include_color' => false,
            'include_typography' => false,
            'active_tab_name' => 'CHECKED',
            'include_active_tab' => true,
            'include_outline' => true,
            'active_class' => 'mdp-cfsevenform-elementor-box .wpcf7-checkbox input[type="checkbox"]:checked, {{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-acceptance input[type="checkbox"]:checked'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Select.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_select() {

        $this->start_controls_section( 'section_style_select', [
            'label' => esc_html__( 'Select', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_select',
            'mdp-cfsevenform-elementor-box .wpcf7-select'
        );

        $this->add_control(
            'select_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box select' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'select_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box select' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_select', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-select',
            'typography_class' => 'mdp-cfsevenform-elementor-box .wpcf7-select',
            'color_class' => 'mdp-cfsevenform-elementor-box .wpcf7-select',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box .wpcf7-select:hover',
            'additional_transition_selector' => ''
        ] );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_button() {

        $this->start_controls_section( 'section_style_button', [
            'label' => esc_html__( 'Button', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_button',
            'mdp-cfsevenform-elementor-box .wpcf7-submit'
        );

        $this->add_control(
            'button_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-submit' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'button_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-cfsevenform-elementor-box .wpcf7-submit' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_button', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-submit',
            'typography_class' => 'mdp-cfsevenform-elementor-box .wpcf7-submit',
            'color_class' => 'mdp-cfsevenform-elementor-box .wpcf7-submit',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box .wpcf7-submit:hover'
        ] );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Error Message.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_errors() {

        $this->start_controls_section( 'section_style_errors', [
            'label' => esc_html__( 'Error message', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_errors', 'wpcf7-not-valid-tip' );

        $this->generate_typography_tabs_controls( 'section_style_errors', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-not-valid-tip',
            'typography_class' => 'mdp-cfsevenform-elementor-box .wpcf7-not-valid-tip',
            'color_class' => 'mdp-cfsevenform-elementor-box .wpcf7-not-valid-tip',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box .wpcf7-not-valid-tip',
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Response Message.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_response_message() {

        $this->start_controls_section( 'section_style_response_message', [
            'label' => esc_html__( 'Response message', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_response_message',
            'mdp-cfsevenform-elementor-box .wpcf7-response-output'
        );

        $this->generate_typography_tabs_controls( 'section_style_response_message', [
            'html_class' => 'mdp-cfsevenform-elementor-box .wpcf7-response-output',
            'typography_class' => 'mdp-cfsevenform-elementor-box .wpcf7-response-output',
            'color_class' => 'mdp-cfsevenform-elementor-box .wpcf7-response-output',
            'color_hover_class' => 'mdp-cfsevenform-elementor-box .wpcf7-response-output:hover',
        ] );


        $this->end_controls_section();

    }

    /**
     * Get form title method.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function get_app_title( $settings ) {
        if ( $settings['show_title'] === 'yes' ) {
            return sprintf(
                '<div class="mdp-cfsevenform-elementor-form-title">%s</div>',
                esc_html( $settings['form_title'] )
            );
        }

        return  '';
    }

    /**
     * Get form description method.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function get_app_description( $settings ) {
        if ( $settings['show_description'] === 'yes' ) {
            return sprintf(
                '<div class="mdp-cfsevenform-elementor-form-description">%s</div>',
                wp_kses_post( $settings['form_description'] )
            );
        }

        return '';
    }

    /**
     * Get form method.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function get_app_form( $settings ){
        $form = esc_html__( 'Please select form!', 'selection' );
        if ( !empty( $settings['select_form'] ) ) {
            $text_shortcode = sprintf( '[contact-form-7 id="%s" ]', esc_html__( $settings['select_form'] ) );
            $form = do_shortcode( $text_shortcode );
        }
        return $form;
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();

        $validation_errors = $settings['validation_errors'] !== 'yes' ?
            'mdp-cfsevenform-elementor-hide-validation' : '';

        $show_labels = $settings['show_labels'] !== 'yes' ?
            'mdp-cfsevenform-elementor-hide-labels' : '';

        if ( class_exists( 'WPCF7_ContactForm' ) ) {
            echo sprintf(
                '<!-- Start Cfsevenform for Elementor WordPress Plugin -->
                    <div class="mdp-cfsevenform-elementor-box %s %s"> %s %s %s</div>
                    <!-- End Cfsevenform for Elementor WordPress Plugin -->',
                esc_attr( $validation_errors ),
                esc_attr( $show_labels ),
                $this->get_app_title( $settings ),
                $this->get_app_description( $settings ),
                $this->get_app_form( $settings )
            );
        }else{
            echo sprintf(
                '<!-- Start Cfsevenform for Elementor WordPress Plugin -->
                    <div class="mdp-cfsevenform-elementor-box %s %s">%s</div>
                    <!-- End Cfsevenform for Elementor WordPress Plugin -->',
                esc_attr( $validation_errors ),
                esc_attr( $show_labels ),
                esc_html__('Please install and activate "Contact Form 7" plugin!', 'selection' )
            );
        }

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/tag/cfsevenform';
    }

}
