<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Upseller for Elementor
 * Customizable upsells and related products for Elementor editor
 * Exclusively on https://1.envato.market/upseller-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Upseller - Custom Elementor Widget.
 **/
class related_products_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-selection-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [], UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-upseller-elementor',
            UnityPlugin::get_url() . 'css/upseller-elementor' . UnityPlugin::get_suffix() . '.css',
            [], UnityPlugin::get_version()
        );
        wp_register_script(
            'mdp-upseller-elementor',
            UnityPlugin::get_url() . 'js/upseller-elementor' . UnityPlugin::get_suffix() . '.js',
            [ 'jquery' ],
            UnityPlugin::get_version(),
            true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-upseller-related-products-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Related products', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-related-products-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Upseller' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-upseller-elementor', 'mdp-selection-elementor-admin' ];

    }

    /**
     * Get script dependencies.
     * Retrieve the list of script dependencies the element requires.
     *
     * @access public
     *
     * @return array Element scripts dependencies.
     **/
    public function get_script_depends() {

        return [ 'mdp-upseller-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_related_products_general();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Products. */
        $this->section_style_products();

        /** Style -> Section Style Title. */
        $this->section_style_title();

        /** Style -> Section Style Product Box. */
        $this->section_style_product_box();

        /** Style -> Section Style Product Title. */
        $this->section_style_product_title();

        /** Style -> Section Style Product Image. */
        $this->section_style_product_image();

        /** Style -> Section Style Product Rating. */
        $this->section_style_product_rating();

        /** Style -> Section Style Products. */
        $this->section_style_product_price();

        /** Style -> Section Style Products. */
        $this->section_style_product_sale_price();

        /** Style -> Section Style Products. */
        $this->section_style_product_sale_flash();

        /** Style -> Section Style Button. */
        $this->section_style_product_button();

        /** Style -> Section Style Button. */
        $this->section_style_product_view_cart();

    }

    /**
     * Add widget controls: Content -> Example Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_related_products_general() {

        $this->start_controls_section( 'section_content_related_products_general', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        if ( Caster::get_instance()->is_active_woocommerce() ) {
            $products_list = [];

            $products = wc_get_products( ['return' => 'ids', 'limit' => -1] );
            foreach ( $products as $product ) {
                $products_list[$product] = get_the_title( $product );
            }

            if ( empty( wc_get_product() ) ) {

                $this->add_control(
                    'related_products',
                    [
                        'label' => esc_html__( 'Products', 'selection' ),
                        'type' => Controls_Manager::SELECT2,
                        'multiple' => false,
                        'options' => $products_list,
                        'default' => !empty( $products_list ) ? array_keys( $products_list )[0] : ''
                    ]
                );

            }
        }

        $this->add_control(
            'related_order_by',
            [
                'label' => esc_html__( 'Order by', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'title',
                'options' => [
                    'date'  => esc_html__( 'Date', 'selection' ),
                    'title'  => esc_html__( 'Title', 'selection' ),
                    'popularity'  => esc_html__( 'Popularity', 'selection' ),
                    'price'  => esc_html__( 'Price', 'selection' ),
                    'rating'  => esc_html__( 'Rating', 'selection' ),
                    'rand'  => esc_html__( 'Random', 'selection' ),
                    'menu_order' => esc_html__( 'Menu order', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'related_order',
            [
                'label' => esc_html__( 'Order', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'desc',
                'options' => [
                    'desc'  => esc_html__( 'DESC', 'selection' ),
                    'asc' => esc_html__( 'ASC', 'selection' ),
                ],
            ]
        );

        $this->add_responsive_control(
            'related_columns',
            [
                'label' => esc_html__( 'Columns', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 12,
                'step' => 1,
                'default' => 2,
            ]
        );

        $this->add_control(
            'related_products_per_page',
            [
                'label' => esc_html__( 'Products per page', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 12,
                'step' => 1,
                'default' => 2,
            ]
        );

        $this->add_control(
            'related_alignment',
            [
                'label' => esc_html__( 'Product content alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => ' eicon-text-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'related_title_alignment',
            [
                'label' => esc_html__( 'Title alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => ' eicon-text-align-right',
                    ],
                ],
                'condition' => [
                    'related_show_title' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-upseller-elementor-related-products-box section.related h2:first-child' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'related_show_title',
            [
                'label' => esc_html__( 'Show Title', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );


        $this->add_control(
            'related_show_sale_flash',
            [
                'label' => esc_html__( 'Show sale flash', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'related_show_product_price',
            [
                'label' => esc_html__( 'Show product price', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'related_show_add_to_cart',
            [
                'label' => esc_html__( 'Show add to cart button', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Method for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {


        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }


    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param array $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ? $opts['html_class'] : '',
            'additional_html_class' => array_key_exists( 'additional_html_class', $opts ) ?
                $opts['additional_html_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ? $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ? $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ? $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ? $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ? $opts['color_hover_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'include_css_filters' => array_key_exists( 'include_css_filters', $opts ) ?
                $opts['include_css_filters'] : false,
            'typography_class' => array_key_exists( 'typography_class', $opts ) ? $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] :
                Color::COLOR_3,
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $style_opts['include_color'] ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'],
                ]
            );

        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_normal',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'],
                ]
            );
        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }


        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover",
                ]
            );
        }


        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_hover' . $section_id,
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'].':hover',
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'].$style_opts['additional_html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover",
            ]
        );

        $this->end_controls_tab();


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Products.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_products() {

        $this->start_controls_section( 'section_style_products', [
            'label' => esc_html__( 'Products', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'columns_gap',
            [
                'label' => esc_html__( 'Columns gap', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} ul.products' => 'column-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'rows_gap',
            [
                'label' => esc_html__( 'Rows gap', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} ul.products' => 'row-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_title() {

        $this->start_controls_section( 'section_style_title', [
            'label' => esc_html__( 'Title', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'related_show_title' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_title',
            'mdp-upseller-elementor-related-products-box section.related h2:first-child'
        );

        $this->generate_typography_tabs_controls( 'section_style_title', [
            'html_class' => 'mdp-upseller-elementor-related-products-box section.related h2:first-child',
            'color_hover_class' => 'mdp-upseller-elementor-related-products-box section.related h2:first-child:hover',
            'color_class' => 'mdp-upseller-elementor-related-products-box section.related h2:first-child',
            'typography_class' => 'mdp-upseller-elementor-related-products-box section.related h2:first-child'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_title() {

        $this->start_controls_section( 'section_style_product_title', [
            'label' => esc_html__( 'Product title', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_margin_padding_controls(
            'section_style_product_title',
            'woocommerce-loop-product__title'
        );

        $this->generate_typography_tabs_controls( 'section_style_product_title', [
            'html_class' => 'woocommerce-loop-product__title',
            'color_hover_class' => 'woocommerce-loop-product__title:hover',
            'color_class' => 'woocommerce-loop-product__title',
            'typography_class' => 'woocommerce ul.products li.product h2.woocommerce-loop-product__title'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_image() {

        $this->start_controls_section( 'section_style_product_image', [
            'label' => esc_html__( 'Image', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_product_image',
            'mdp-upseller-elementor-related-products-box li.product img'
        );

        $this->add_responsive_control(
            'image_width',
            [
                'label' => esc_html__( 'Image width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-upseller-elementor-related-products-box .woocommerce-placeholder' => 'width: {{SIZE}}{{UNIT}}'
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_image', [
            'html_class' => 'mdp-upseller-elementor-related-products-box li.product img',
            'color_hover_class' => 'mdp-upseller-elementor-related-products-box li.product img:hover',
            'include_color' => false,
            'include_css_filters' => true,
            'include_typography' => false,
            'color_class' => 'mdp-upseller-elementor-related-products-box li.product img'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Rating.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_rating() {

        $this->start_controls_section( 'section_style_product_rating', [
            'label' => esc_html__( 'Rating', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_product_rating_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .star-rating" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->add_control(
            'section_style_product_rating_star_size',
            [
                'label' => esc_html__( 'Rating star size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', 'em' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'em' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .star-rating' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->start_controls_tabs( 'section_style_product_rating_style_tabs' );

        $this->start_controls_tab( 'section_style_product_rating_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        $this->add_control(
            'section_style_product_rating_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .star-rating" => 'color: {{VALUE}} !important;',
                ],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( 'section_style_product_rating_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        $this->add_control(
            'section_style_product_rating_hover_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .star-rating:hover" => 'color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->add_control(
            'section_style_product_rating_hover_transition',
            [
                'label' => esc_html__( 'Hover transition duration', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    "{{WRAPPER}} .star-rating:hover" => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->end_controls_tabs();



        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Product Price.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_price() {

        $this->start_controls_section( 'section_style_product_price', [
            'label' => esc_html__( 'Price', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_product_price',
            'product:not(.sale) .price .woocommerce-Price-amount, {{WRAPPER}} .price ins'
        );

        $this->generate_typography_tabs_controls( 'section_style_product_price', [
            'html_class' => 'product:not(.sale) .price .woocommerce-Price-amount',
            'additional_html_class' => ', {{WRAPPER}} .price ins',
            'color_hover_class' => 'price .woocommerce-Price-amount:hover bdi',
            'color_class' => 'price .woocommerce-Price-amount bdi',
            'typography_class' => 'price .woocommerce-Price-amount',
            'additional_transition_selector' => '{{WRAPPER}} .price .woocommerce-Price-amount bdi'
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Product Sale Price.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_sale_price() {

        $this->start_controls_section( 'section_style_product_sale_price', [
            'label' => esc_html__( 'Sale price', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_product_sale_price', 'price del' );

        $this->generate_typography_tabs_controls( 'section_style_product_sale_price', [
            'html_class' => 'sale .price del',
            'color_hover_class' => 'sale del .woocommerce-Price-amount:hover bdi',
            'color_class' => 'sale del .woocommerce-Price-amount bdi',
            'typography_class' => 'sale .price del .woocommerce-Price-amount.amount bdi',
            'additional_transition_selector' => '{{WRAPPER}} .price del .woocommerce-Price-amount.amount bdi'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Sale Flash.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_sale_flash() {

        $this->start_controls_section( 'section_style_product_sale_flash', [
            'label' => esc_html__( 'Sale flash', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'related_show_sale_flash' => 'yes'
            ]
        ] );

        $this->add_responsive_control(
            'section_style_product_sale_flash_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .onsale" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_sale_flash_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .onsale" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}; line-height: initial !important; min-height: initial; min-width: initial;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_sale_flash', [
            'html_class' => 'onsale',
            'color_hover_class' => 'onsale:hover',
            'typography_class' => 'onsale',
            'color_class' => 'onsale'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_box() {

        $this->start_controls_section( 'section_style_product_box', [
            'label' => esc_html__( 'Product box', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_product_box_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .product" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_box', [
            'html_class' => 'product',
            'include_color' => false,
            'include_typography' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Add To Cart Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_button() {

        $this->start_controls_section( 'section_style_product_button', [
            'label' => esc_html__( 'Button', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_product_button',
            'product .add_to_cart_button'
        );

        $this->generate_typography_tabs_controls( 'section_style_product_button', [
            'html_class' => 'product .add_to_cart_button',
            'color_hover_class' => 'product .add_to_cart_button:hover',
            'color_class' => 'product .add_to_cart_button',
            'typography_class' => 'product .add_to_cart_button'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style View Cart Link.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_view_cart() {

        $this->start_controls_section( 'section_style_product_view_cart', [
            'label' => esc_html__( 'View cart link', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_product_view_cart',
            'added_to_cart.wc-forward'
        );

        $this->generate_typography_tabs_controls( 'section_style_product_view_cart', [
            'html_class' => 'added_to_cart.wc-forward',
            'color_hover_class' => 'added_to_cart.wc-forward',
            'color_class' => 'added_to_cart.wc-forward',
            'typography_class' => 'added_to_cart.wc-forward'
        ] );

        $this->end_controls_section();

    }

    /**
     * Method for generating columns data attribute.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function get_columns_data_attrs( $settings ) {
        $breakpoints = Plugin::$instance->breakpoints->get_active_breakpoints();
        $attrs = [];

        $attrs[] = 'data-columns="' . esc_attr( $settings['related_columns'] ) .'"';

        foreach ( $breakpoints as $key => $value ) {
            if ( isset( $settings['related_columns_' . $key] ) ) {
                $attrs[] = 'data-columns-' . $key . '=' . '"' . esc_attr( $settings['related_columns_' . $key] ) . '"';
            }
        }

        return implode( ' ', $attrs );
    }

    /**
     * Method that returns related products arguments.
     *
     * @param $settings
     * @param $product
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_related_args( $settings, $product ) {
        $args = [
            'posts_per_page' => !empty( $settings['related_products_per_page'] ) ?
                esc_sql( $settings['related_products_per_page'] ) : 4,
            'columns' => 4,
            'orderby' => !empty( $settings['related_order_by'] ) ? esc_sql( $settings['related_order_by'] ) : 'rand',
            'order' => !empty( $settings['related_order'] ) ? esc_sql( $settings['related_order'] ) : 'desc'

        ];


        $args['related_products'] = array_filter(
            array_map( 'wc_get_product',
                wc_get_related_products( $product->get_id(), $args['posts_per_page'], $product->get_upsell_ids() )
            ),
            'wc_products_array_filter_visible'
        );

        $args['related_products'] = wc_products_array_orderby(
            $args['related_products'],
            $args['orderby'],
            $args['order']
        );

        return $args;
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        if ( !Caster::get_instance()->is_active_woocommerce() ) {
            esc_html_e( 'Install WooCommerce to display related products!' );
            return;
        }

        $settings = $this->get_settings_for_display();
        $columns_attrs = $this->get_columns_data_attrs( $settings );
        $hide_title_class = $settings['related_show_title'] !== 'yes' ? 'mdp-upseller-elementor-related-hide-title' : '';
        $hide_sale_flash_class = $settings['related_show_sale_flash'] !== 'yes' ?
            'mdp-upseller-elementor-related-hide-sale-flash' : '';
        $alignment_class = sprintf( 'mdp-upseller-elementor-related-align-%s', $settings['related_alignment'] );

        global $product;
        $product = wc_get_product();

        if ( empty( $product ) ) {
            $product = wc_get_product( esc_sql( $settings['related_products'] ) );
            if ( empty( $product ) || $product->get_status() === 'trash' ) {
                return;
            }
        }

        $args = $this->get_related_args( $settings, $product );

        echo sprintf(
            '<!-- Start Upseller for Elementor WordPress Plugin -->
                       <div class="mdp-upseller-elementor-related-products-box woocommerce %s %s %s" %s>',
            esc_attr( $hide_title_class ),
            esc_attr( $hide_sale_flash_class ),
            esc_attr( $alignment_class ),
            $columns_attrs
        );
        Caster::get_instance()->remove_product_actions( $settings, [
            'show_cart_key' => 'related_show_add_to_cart',
            'show_price_key' => 'related_show_product_price'
        ] );

        wc_get_template( 'single-product/related.php', $args );

        Caster::get_instance()->restore_product_actions( $settings, [
            'show_cart_key' => 'related_show_add_to_cart',
            'show_price_key' => 'related_show_product_price'
        ] );
        echo '</div><!-- End Upseller for Elementor WordPress Plugin -->';

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/upseller';

    }

}
