<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Sliper
 * Powerful responsive slider plugin for Elementor
 * Exclusively on https://1.envato.market/sliper-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Widget_Base;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;
use Merkulove\Selection\Unity\MobileDetect;


/**
 * Sliper - Custom Elementor Widget.
 **/
class sliper_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        /** CSS */
        wp_register_style( 'flickity', UnityPlugin::get_url() . 'css/flickity' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-sliper', UnityPlugin::get_url() . 'css/sliper' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

        /** JavaScript. */
        wp_register_script( 'flickity-pkgd', UnityPlugin::get_url() . 'js/flickity.pkgd' . UnityPlugin::get_suffix() . '.js', ['jquery'], UnityPlugin::get_version(), true);
        wp_register_script( 'flickity-hash', UnityPlugin::get_url() . 'js/flickity-hash' . UnityPlugin::get_suffix() . '.js', ['jquery', 'flickity-pkgd'], UnityPlugin::get_version(), true);
        wp_register_script( 'mdp-sliper', UnityPlugin::get_url() . 'js/sliper' . UnityPlugin::get_suffix() . '.js', ['jquery'], UnityPlugin::get_version(), true);
    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-sliper-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Slider', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-sliper-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Slider', 'Slider', 'Image', 'Gallery', 'Carousel', 'Grid' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'flickity', 'mdp-sliper', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'flickity-pkgd', 'flickity-hash', 'mdp-sliper' ];

    }

	/**
	 * Group controllers for setting styles.
	 *
	 * @param $name_Section - The name of the styles section.
	 * @param string $css_class - Specifies one or more class names for an element.
	 * @param array $condition - Interconnected controllers.
	 * @param array $display - Determines which controllers to display. {
	 *      @type $key 'margin'                 => (boolean) - Use margin.
	 *      @type $key 'padding'                => (boolean) - Use padding.
	 *      @type $key 'color'                  => (boolean) - Use color.
	 *      @type $key 'typography'             => (boolean) - Use typography.
	 *      @type $key 'text-shadow'            => (boolean) - Use text shadow.
	 *      @type $key 'alignment'              => (boolean) - Use alignment.
	 *      @type $key 'alignment-justify'      => (boolean) - Use alignment justify.
	 *      @type $key 'alignment-horizontal'   => (boolean) - Use alignment horizontal.
	 *      @type $key 'alignment-vertical'     => (boolean) - Use alignment vertical.
	 *      @type $key 'background'             => (boolean) - Use background.
	 *      @type $key 'border'                 => (boolean) - Use border.
	 *      @type $key 'border-radius'          => (boolean) - Use border radius.
	 *      @type $key 'box-shadow'             => (boolean) - Use box shadow.
	 *      @type $key 'css-filter'             => (boolean) - Use css filter.
	 *
	 * }
     * @param string $separator_radius
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return void
	 */
	public function get_settings_style_group( $name_Section, $css_class, $condition = [], $display = [], $separator_radius = 'after' ) {

		/** Margin. */
		if ( isset($display['margin']) && $display['margin'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_margin',
				[
					'label'      => esc_html__( 'Margin', 'selection' ),
					'type'       => Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%', 'em' ],
					'selectors'  => [
						'{{WRAPPER}} ' . $css_class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}} !important;',
					],
					'default' => [
						'top' => 0,
						'right' => 0,
						'bottom' => 0,
						'left' => 0,
						'unit' => 'px',
					],
					'toggle'     => true,
					'condition'  => $condition,
				]
			);
		}

		/** Padding. */
		if ( isset($display['padding']) && $display['padding'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_padding',
				[
					'label'      => esc_html__( 'Padding', 'selection' ),
					'type'       => Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%', 'em' ],
					'selectors'  => [
						'{{WRAPPER}} ' . $css_class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
					],
					'default' => [
						'top' => 0,
						'right' => 0,
						'bottom' => 0,
						'left' => 0,
						'unit' => 'px',
					],
					'toggle'     => true,
					'separator'  => 'after',
					'condition'  => $condition,
				]
			);
		}

		/** Text Color. */
		if ( isset($display['color']) && $display['color'] === true ) {
			$this->add_control(
				'sliper_' . $name_Section . '_text_color',
				[
					'label'     => esc_html__( 'Color', 'selection' ),
					'type'      => Controls_Manager::COLOR,
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'color: {{VALUE}}',
					],
					'condition' => $condition,
				]
			);
		}

		/** Typography */
		if ( isset($display['typography']) && $display['typography'] === true ) {
			$this->add_group_control(
				Group_Control_Typography::get_type(),
				[
					'name'      => 'sliper_' . $name_Section . '_typography',
					'label'     => esc_html__( 'Typography', 'selection' ),
					'scheme'    => Typography::TYPOGRAPHY_1,
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** Text Shadow. */
		if ( isset($display['text-shadow']) && $display['text-shadow'] === true ) {
			$this->add_group_control(
				Group_Control_Text_Shadow::get_type(),
				[
					'name'      => 'sliper_' . $name_Section . '_text_shadow',
					'label'     => esc_html__( 'Text Shadow', 'selection' ),
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** Alignment horizontal. */
		if ( isset($display['alignment-horizontal']) && $display['alignment-horizontal'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_horizontal_align',
				[
					'label'     => esc_html__( 'Horizontal Position', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'flex-start'    => [
							'title' => esc_html__( 'Left', 'selection' ),
							'icon'  => 'eicon-h-align-left',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'eicon-h-align-center',
						],
						'flex-end'   => [
							'title' => esc_html__( 'Right', 'selection' ),
							'icon'  => 'eicon-h-align-right',
						],
					],
					'default'   => '',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'justify-content: {{header_align}};',
						'{{WRAPPER}} .mdp-sliper-box-content' => 'width: 65%;',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Alignment vertical. */
		if ( isset($display['alignment-vertical']) && $display['alignment-vertical'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_vertical_align',
				[
					'label'     => esc_html__( 'Vertical Position', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'flex-start'    => [
							'title' => esc_html__( 'Top', 'selection' ),
							'icon'  => 'eicon-v-align-top',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'eicon-v-align-middle',
						],
						'flex-end'   => [
							'title' => esc_html__( 'Bottom', 'selection' ),
							'icon'  => 'eicon-v-align-bottom',
						],
					],
					'default'   => 'center',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'align-items: {{value}};',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Alignment. */
		if ( isset($display['alignment']) && $display['alignment'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_align',
				[
					'label'     => esc_html__( 'Alignment', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'left'    => [
							'title' => esc_html__( 'Left', 'selection' ),
							'icon'  => 'fa fa-align-left',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'fa fa-align-center',
						],
						'right'   => [
							'title' => esc_html__( 'Right', 'selection' ),
							'icon'  => 'fa fa-align-right',
						],
					],
					'default'   => 'center',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'text-align: {{header_align}};',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Alignment justify. */
		if ( isset($display['alignment-justify']) && $display['alignment-justify'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_align_justify',
				[
					'label'     => esc_html__( 'Alignment', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'left'    => [
							'title' => esc_html__( 'Left', 'selection' ),
							'icon'  => 'fa fa-align-left',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'fa fa-align-center',
						],
						'right'   => [
							'title' => esc_html__( 'Right', 'selection' ),
							'icon'  => 'fa fa-align-right',
						],
						'justify' => [
							'title' => esc_html__( 'Justify', 'selection' ),
							'icon'  => 'fa fa-align-justify',
						],
					],
					'default'   => 'center',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'text-align: {{header_align}};',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Background. */
		if ( isset($display['background']) && $display['background'] === true ) {
			$this->add_group_control(
				Group_Control_Background::get_type(),
				[
					'name'      => 'sliper_' . $name_Section . '_background',
					'label'     => esc_html__( 'Background', 'selection' ),
					'types'     => [ 'classic', 'gradient' ],
					'separator' => 'before',
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** Border. */
		if ( isset($display['border']) && $display['border'] === true ) {
			$this->add_group_control(
				Group_Control_Border::get_type(),
				[
					'name'      => 'sliper_' . $name_Section . '_border',
					'label'     => esc_html__( 'Border', 'selection' ),
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'separator' => 'before',
					'condition' => $condition,
				]
			);
		}

		/** Border Radius. */
		if ( isset($display['border-radius']) && $display['border-radius'] === true ) {
			$this->add_responsive_control(
				'sliper_' . $name_Section . '_border_radius',
				[
					'label'      => esc_html__( 'Border Radius', 'selection' ),
					'type'       => Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%', 'em' ],
					'selectors'  => [
						'{{WRAPPER}} ' . $css_class => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
					],
					'separator'  => $separator_radius,
					'toggle'     => true,
					'condition'  => $condition,
				]
			);
		}

		/** Box Shadow. */
		if ( isset($display['box-shadow']) && $display['box-shadow'] === true ) {
			$this->add_group_control(
				Group_Control_Box_Shadow::get_type(),
				[
					'name'      => 'sliper_' . $name_Section . '_box_shadow',
					'label'     => esc_html__( 'Box Shadow', 'selection' ),
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** CSS filter. */
		if ( isset($display['css-filter']) && $display['css-filter'] === true ) {
			$this->add_group_control(
				Group_Control_Css_Filter::get_type(),
				[
					'name'     => 'sliper_' . $name_Section . '_css_filter',
					'label'    => esc_html__( 'CSS filter', 'selection' ),
					'selector' => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

	}

	/**
	 * We write the existing categories of posts into an array.
	 */
	public function get_post_category() {

		$wp_cat = get_categories('&hide_empty=0');

		/** If no category is selected.  */
		$options['select_category'] = 'Select category';

		/** Add available post categories. */
		foreach( $wp_cat as $cat_item ){
			$options[$cat_item->slug] =  $cat_item->name;
		}

		return $options;
	}

	/**
	 * We write the existing product categories into an array.
	 */
	public function get_products_category() {

		$wp_cat = get_categories( array(
			'taxonomy'     => 'product_cat',
			'orderby'      => 'name',
			'pad_counts'   => false,
			'hierarchical' => 1,
			'hide_empty'   => false
		) );

		/** If no category is selected.  */
		$options['select_category'] = 'Select category';

		/** Add available product categories. */
		foreach( $wp_cat as $cat_item ){
			$options[$cat_item->slug] =  $cat_item->name;
		}

		return $options;
	}

	/**
	 *  TAB: Content.
	 */

	/**
	 * General TAB.
	 */
	public function get_general_section_content() {

		/** Start section */
		$this->start_controls_section(
			'general_content',
			[
				'label' => esc_html__( 'General', 'selection' ),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		/** Source */
		$this->add_control(
			'general_source',
			[
				'label' => esc_html__( 'Source', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'custom',
				'options' => $this->get_general_sources(),
			]
		);

		/** Post category. */
		$this->add_control(
			'general_posts_category',
			[
				'label' => esc_html__( 'Category', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'select_category',
				'options' => $this->get_post_category(),
				'condition' => [ 'general_source' => 'posts' ]
			]
		);

		/** Products category. */
		$this->add_control(
			'general_products_category',
			[
				'label' => esc_html__( 'Category', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'select_category',
				'options' => $this->get_products_category(),
				'condition' => [ 'general_source' => 'woocommerce' ]
			]
		);

		/** Custom posts. */
		$this->add_control(
			'general_item_custom_posts',
			[
				'label' => esc_html__( 'Custom posts', 'selection' ),
				'type' => Controls_Manager::SELECT2,
				'label_block' => true,
				'multiple' => true,
				'options' => $this->get_custom_post_array(),
				'condition' => [ 'general_source' => 'custom-posts' ],
			]
		);

		/** Type. */
		$this->add_control(
			'general_type',
			[
				'label' => esc_html__( 'Type', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'overlay',
				'options' => [
					'card'  => esc_html__( 'Card', 'selection' ),
					'overlay' => esc_html__( 'Overlay', 'selection' ),
				],
			]
		);

		/** Link type. */
		$this->add_control(
			'general_link_type',
			[
				'label' => esc_html__( 'Link type', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'none',
				'options' => [
					'none'  => esc_html__( 'None', 'selection' ),
					'box' => esc_html__( 'Box', 'selection' ),
					'title' => esc_html__( 'Title', 'selection' ),
					'button' => esc_html__( 'Button', 'selection' ),
				]
			]
		);

        /** Link target */
		$this->add_control(
			'general_link_target',
			[
				'label' => esc_html__( 'Link target', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => '_blank',
				'options' => [
					'_blank'  => esc_html__( 'Blank', 'selection' ),
					'_self' => esc_html__( 'Self', 'selection' ),
					'_parent' => esc_html__( 'Parent', 'selection' ),
					'_top' => esc_html__( 'Top', 'selection' ),
				],
				'condition' => [
                    'general_source!' => 'custom'
                ],
			]
		);

		/** Button Text. */
		$this->add_control(
			'general_button_text',
			[
				'label' => esc_html__( 'Button Text', 'selection' ),
				'type' => Controls_Manager::TEXT,
				'placeholder' => esc_html__( 'Button Text', 'selection' ),
				'default' => esc_html__( 'More', 'selection' ),
				'label_block' => true,
				'condition' => [
					'general_source' => ['posts', 'woocommerce', 'custom-posts'],
					'general_link_type' => 'button'
				]
			]
		);

		/** Initial Slide. */
		$this->add_control(
			'slides_initial_slide',
			[
				'label'           => esc_html__( 'Initial Slide', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 0,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 1,
				],
				'separator' => 'before'
			]
		);

		/** Slides align.  */
		$this->add_control(
			'general_slides_align',
			[
				'label' => esc_html__( 'Slides align', 'selection' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'selection' ),
						'icon' => 'fa fa-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'selection' ),
						'icon' => 'fa fa-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'selection' ),
						'icon' => 'fa fa-align-right',
					],
				],
				'default' => 'center',
				'toggle' => true,
				'conditions' => [
					'terms' => [
						[
							'name' => 'slide_style_width[size]',
							'operator' => '<=',
							'value' => 90
						]
					]
				]
			]
		);

		/** Display slides. */
		$this->add_control(
			'general_display_slides',
			[
				'label'           => esc_html__( 'Scroll by', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 1,
				],
			]
		);

		/** Slides shift. */
		$this->add_control(
			'general_slides_shift',
			[
				'label'           => esc_html__( 'Slides shift', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper'  => 'padding-left: {{size}}{{unit}}',
				],
			]
		);

		/** Transition. */
		$this->add_control(
			'general_transition',
			[
				'label' => esc_html__( 'Slides transition', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'none',
				'options' => [
					'none'        =>  esc_html__('None', 'selection' ),
					'rise'        =>  esc_html__('Rice', 'selection' ),
					'slide-up'    =>  esc_html__('Slide Up', 'selection' ),
					'slide-right'  =>  esc_html__('Slide Right', 'selection' ),
					'slide-left'  =>  esc_html__('Slide Left', 'selection' ),
					'slide-down'  =>  esc_html__('Slide Down', 'selection' ),
					'bounce'      =>  esc_html__('Bounce', 'selection' ),
					'flip-x'      =>  esc_html__('Flip X', 'selection' ),
					'flip-y'      =>  esc_html__('Flip Y', 'selection' ),
					'jello'       =>  esc_html__('Jello', 'selection' ),
					'zoom'        =>  esc_html__('Zoom', 'selection' ),
					'fade'        =>  esc_html__('Fade', 'selection' ),
				],
			]
		);

		$this->add_control(
			'general_transition_duration',
			[
				'label'           => esc_html__( 'Transition duration (seconds)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 0,
						'max'  => 5,
						'step' => .1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => .5,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-carousel-cell'  => 'animation-duration: {{size}}s',
				],
				'condition' => [ 'general_transition!' => 'none' ],
			]
		);

		/** Transform origin. */
		$this->add_control(
			'general_transform_origin',
			[
				'label' => esc_html__( 'Transform origin', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'center center',
				'options' => [
					'center top'  =>  esc_html__('Center top', 'selection' ),
					'center center' =>  esc_html__('Center center', 'selection' ),
					'center bottom'  =>  esc_html__('Center bottom', 'selection' ),
					'left top'  =>  esc_html__('Left top', 'selection' ),
					'left center'  =>  esc_html__('Left center', 'selection' ),
					'left bottom'  =>  esc_html__('Left bottom', 'selection' ),
					'right top'  =>  esc_html__('Right top', 'selection' ),
					'right center'  =>  esc_html__('Right center', 'selection' ),
					'right bottom'  =>  esc_html__('Right bottom', 'selection' ),
				],
				'selectors'  => [
					'{{WRAPPER}} .is-selected'  => 'transform-origin: {{value}}',
				],
			]
		);

		/** RTL */
		$this->add_control(
			'general_rtl',
			[
				'label' => esc_html__( 'RTL', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'description' => esc_html__( 'Enables right-to-left layout', 'selection' ),
				'separator' => 'after'
			]
		);

		/** Disable on Desktop. */
		$this->add_control(
			'general_disable_desktop',
			[
				'label' => esc_html__( 'Disable on Desktop', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes'
			]
		);

		/** Disable on Tablet. */
		$this->add_control(
			'general_disable_tablet',
			[
				'label' => esc_html__( 'Disable on Tablet', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes'
			]
		);

		/** Disable on Mobile. */
		$this->add_control(
			'general_disable_mobile',
			[
				'label' => esc_html__( 'Disable on Mobile', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes'
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
     * Get general sources
	 * @return array
	 */
    private function get_general_sources(): array {

        $sources = array(
	        'custom'  => esc_html__( 'Custom', 'selection' ),
	        'posts' => esc_html__( 'Posts', 'selection' ),
	        'custom-posts' => esc_html__( 'Custom posts', 'selection' ),
        );

        if ( class_exists( 'woocommerce' ) ) {
	        $sources[ 'woocommerce' ] = esc_html__( 'WooCommerce', 'selection' );
        }

        return $sources;

    }

	/**
	 * @return array
	 */
	public function get_post_types() {

		$wp_pt = get_post_types( array(
			'public' => 1,
		) );
		$options = array();

		/** Add available post categories. */
		$exceptions = ['post', 'page', 'e-landing-page', 'elementor_library', 'product'];
		foreach( $wp_pt as $post_type ){
			if( !in_array( $post_type, $exceptions, true ) ) {
				$options[] = $post_type;
			}
		}

		return $options;
	}

	/**
	 * @return array
	 */
	public function get_post_taxonomies() {

		$taxonomies = get_taxonomies( array(
			'public'   => true,
			'_builtin' => false
		) );

		/** Add default option */
		$options = array();
		$exceptions = ['product_cat', 'product_tag', 'product_shipping_class' ];
		foreach( $taxonomies as $taxonomy ) {
			if( !in_array( $taxonomy, $exceptions, true ) ){
				$options[] = $taxonomy;
			}
		}

		return $options;

	}

	/**
	 *  Returns a list of custom posts.
	 *
	 */
	public function get_custom_post_array() {

		$post_types         = $this->get_post_types();
		$taxonomies         = $this->get_post_taxonomies();
		$post_list = [];

		$posts = [];
		for( $i = 0, $iMax = count( $taxonomies ); $i < $iMax; $i++){
			$args = array(
				'post_type'   => $post_types,
			);

			/** We get an array with post data. */
			$posts[] = get_posts( $args );
		}

		$all_posts = [];
		array_walk_recursive($posts, function ($item, $key) use (&$all_posts) {
			$all_posts[] = $item;
		});

		/** We count the number of post item in the post type. */
		$quantity_posts = count( $all_posts );

		if( $quantity_posts > 0 ) {
			for( $i = 0; $i < $quantity_posts; $i ++ ) {
				$post_list[ $all_posts[ $i ]->ID ] = $all_posts[ $i ]->post_title;
			}
		}

		return $post_list;

	}

	/**
	 * Navigation TAB.
	 */
	public function get_navigation_section_content() {

		/** Start section */
		$this->start_controls_section(
			'navigation_content',
			[
				'label' => esc_html__( 'Navigation', 'selection' ),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		/** Auto play. */
		$this->add_control(
			'navigation_auto_play',
			[
				'label' => esc_html__( 'Auto play', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
			]
		);

		/** Slide display duration. */
		$this->add_control(
			'navigation_display_duration',
			[
				'label'           => esc_html__( 'Slide display duration', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 2000,
						'step' => 1,
                        'max' => 10000,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 3000,
				],
				'condition' => [ 'navigation_auto_play' => 'yes' ],
			]
		);

		/** Pause on hover. */
		$this->add_control(
			'navigation_pause_hover',
			[
				'label' => esc_html__( 'Pause on hover', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes'
			]
		);

		/** Infinite scrolling. */
		$this->add_control(
			'navigation_infinite_scrolling',
			[
				'label' => esc_html__( 'Infinite scrolling', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		/** Navigation arrows. */
		$this->add_control(
			'navigation_navigation_arrows',
			[
				'label' => esc_html__( 'Navigation arrows', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		/** Navigation dots. */
		$this->add_control(
			'navigation_navigation_dots',
			[
				'label' => esc_html__( 'Navigation dots', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);

		/** Keyboard navigation. */
		$this->add_control(
			'navigation_keyboard_navigation',
			[
				'label' => esc_html__( 'Keyboard navigation', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes'
			]
		);

        /** Hash navigation */
		$this->add_control(
			'navigation_hash',
			[
				'label' => esc_html__( 'Hash navigation', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'default' => '',
			]
		);

		/** Draggable. */
		$this->add_control(
			'navigation_draggable',
			[
				'label' => esc_html__( 'Draggable', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'description' => esc_html__( 'Enables dragging and flicking.', 'selection' ),
			]
		);

		/** Drag threshold. */
		$this->add_control(
			'navigation_drag_threshold',
			[
				'label'           => esc_html__( 'Drag threshold', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 3,
				],
				'condition' => [ 'navigation_draggable' => 'yes' ],
				'description' => esc_html__( 'The number of pixels a mouse or touch has to move before dragging begins.', 'selection' ),
			]
		);

		/** Freely scrolled. */
		$this->add_control(
			'navigation_freely_scrolled',
			[
				'label' => esc_html__( 'Freely scrolled', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes',
				'condition' => [ 'navigation_draggable' => 'yes' ],
			]
		);

		/** Attraction. */
		$this->add_control(
			'navigation_attraction',
			[
				'label'           => esc_html__( 'Attraction', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 0.01,
						'step' => 0.01,
                        'max' => 1
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0.01,
				],
				'description' => esc_html__( 'Attracts the position of the slider to the selected cell', 'selection' )
			]
		);

		/** Friction. */
		$this->add_control(
			'navigation_friction',
			[
				'label'           => esc_html__( 'Friction', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 0.01,
						'step' => 0.01,
                        'max' => 1
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0.15,
				],
				'description' => esc_html__( 'Slows the movement of slider', 'selection' )
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Layout TAB.
	 */
	public function get_layout_section_content() {

		/** Start section */
		$this->start_controls_section(
			'layout_content',
			[
				'label' => esc_html__( 'Layout', 'selection' ),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$layout = new Repeater();

		/** Item. */
		$layout->add_control(
			'layout_item',
			[
				'label'   => esc_html__( 'Item', 'selection' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'title',
				'options' => [
					'image'       => esc_html__( 'Image', 'selection' ),
					'title'       => esc_html__( 'Title', 'selection' ),
					'description' => esc_html__( 'Description', 'selection' ),
					'price'       => esc_html__( 'Price', 'selection' ),
					'category'    => esc_html__( 'Category', 'selection' ),
					'button'      => esc_html__( 'Button', 'selection' ),
				],
			]
		);

		/** Priority. */
		$this->add_control(
			'layout_priority',
			[
				'label'       => esc_html__( 'Priority', 'selection' ),
				'type'        => Controls_Manager::REPEATER,
				'fields'      => $layout->get_controls(),
				'default'     => [
					[ 'layout_item' => 'image' ],
					[ 'layout_item' => 'title' ],
					[ 'layout_item' => 'description' ],
				],
				'title_field' => 'Layout item  - {{{layout_item}}}'
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Slides TAB.
	 */
	public function get_slides_section_content() {

		/** Start section */
		$this->start_controls_section(
			'slides_content',
			[
				'label' => esc_html__( 'Slides', 'selection' ),
				'tab' => Controls_Manager::TAB_CONTENT,
				'condition' => [ 'general_source' => 'custom' ]
			]
		);

		$slides = new Repeater();

		$slides->start_controls_tabs( 'tabs_slides' );

		/** Content tab. */
		$slides->start_controls_tab(
			'tab_slides_content',
			[
				'label' => esc_html__('CONTENT', 'selection')
			]
		);

		/** Background */
		$slides->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'slides_content_background',
				'label' => esc_html__( 'Background', 'selection' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-image',
			]
		);

		/** Title. */
		$slides->add_control(
			'slides_content_title',
			[
				'label' => esc_html__( 'Title', 'selection' ),
				'type' => Controls_Manager::TEXT,
				'placeholder' => esc_html__( 'Title', 'selection' ),
				'label_block' => true
			]
		);

		/** Description. */
		$slides->add_control(
			'slides_content_description',
			[
				'label' => esc_html__( 'Description', 'selection' ),
				'type' => Controls_Manager::WYSIWYG,
				'placeholder' => esc_html__( 'Description', 'selection' ),
			]
		);

		/** Button Text. */
		$slides->add_control(
			'slides_content_button_text',
			[
				'label' => esc_html__( 'Button Text', 'selection' ),
				'type' => Controls_Manager::TEXT,
				'placeholder' => esc_html__( 'Button Text', 'selection' ),
				'label_block' => true
			]
		);

		/** Link. */
		$slides->add_control(
			'slides_content_link',
			[
				'label' => esc_html__( 'Link', 'selection' ),
				'type' => Controls_Manager::URL,
				'placeholder' => esc_html__( 'https://codecanyon.net/user/merkulove', 'selection' ),
				'show_external' => true,
				'default' => [
					'url' => '',
					'is_external' => true,
					'nofollow' => true,
				],
			]
		);

		/** Slide id. */
		$slides->add_control(
			'slides_content_slide_id',
			[
				'label' => esc_html__( 'Slide id', 'selection' ),
				'type' => Controls_Manager::TEXT,
				'placeholder' => esc_html__( 'Slide id', 'selection' ),
				'label_block' => true
			]
		);

		$slides->end_controls_tab();

		/** Style tab. */
		$slides->start_controls_tab(
			'tab_slides_style',
			[
				'label' => esc_html__('STYLE', 'selection')
			]
		);

		/** Custom. */
		$slides->add_control(
			'slides_style_custom',
			[
				'label' => esc_html__( 'Custom', 'selection' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'selection' ),
				'label_off' => esc_html__( 'No', 'selection' ),
				'return_value' => 'yes'
			]
		);

		/** Horizontal Position. */
		$slides->add_control(
			'slides_style_horizontal_position',
			[
				'label' => esc_html__( 'Horizontal Position', 'selection' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'flex-start' => [
						'title' => esc_html__( 'Left', 'selection' ),
						'icon' => 'eicon-h-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'selection' ),
						'icon' => 'eicon-h-align-center',
					],
					'flex-end' => [
						'title' => esc_html__( 'Right', 'selection' ),
						'icon' => 'eicon-h-align-right',
					],
				],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box' => 'justify-content: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content' => 'width: 65%;',
				],
				'default' => 'center',
				'toggle' => true,
				'condition' => [ 'slides_style_custom' => 'yes']
			]
		);

		/** Vertical Position. */
		$slides->add_control(
			'slides_style_vertical_position',
			[
				'label' => esc_html__( 'Vertical Position', 'selection' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'flex-start' => [
						'title' => esc_html__( 'Left', 'selection' ),
						'icon' => 'eicon-v-align-top',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'selection' ),
						'icon' => 'eicon-v-align-middle',
					],
					'flex-end' => [
						'title' => esc_html__( 'Right', 'selection' ),
						'icon' => 'eicon-v-align-bottom',
					],
				],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box' => 'align-items: {{value}}',
				],
				'default' => 'center',
				'toggle' => true,
				'condition' => [ 'slides_style_custom' => 'yes' ]
			]
		);

		/** Text Align. */
		$slides->add_control(
			'slides_style_text_align',
			[
				'label' => esc_html__( 'Text Align', 'selection' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'selection' ),
						'icon' => 'fa fa-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'selection' ),
						'icon' => 'fa fa-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'selection' ),
						'icon' => 'fa fa-align-right',
					],
				],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-title, '.
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-box-description, '.
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mpd-sliper-button-link, '.
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mpd-sliper-button, '.
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-title, '.
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box-description, '.
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mpd-sliper-button-link, '.
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mpd-sliper-button' => 'text-align: {{value}}',
				],
				'default' => 'center',
				'toggle' => true,
				'condition' => [ 'slides_style_custom' => 'yes' ]
			]
		);

		/** Content Color. */
		$slides->add_control(
			'slides_style_color',
			[
				'label' => esc_html__( 'Content Color', 'selection' ),
				'type' => Controls_Manager::COLOR,
				'scheme' => [
					'type' => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-box-description' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box-description' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mpd-sliper-button-link' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mpd-sliper-button-link' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mpd-sliper-button' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mpd-sliper-button' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-title' => 'color: {{value}}',
					'{{WRAPPER}} {{CURRENT_ITEM}}  .mdp-sliper-title' => 'color: {{value}}',
				],
				'condition' => [ 'slides_style_custom' => 'yes' ]
			]
		);

		/** Text Shadow. */
		$slides->add_group_control(
			Group_Control_Text_Shadow::get_type(),
			[
				'name' => 'slides_style_text_shadow',
				'label' => esc_html__( 'Text Shadow', 'selection' ),
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-description',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box-description',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mpd-sliper-button-link',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mpd-sliper-button-link',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mpd-sliper-button',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mpd-sliper-button',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-title',
					'{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-title',
				],
				'condition' => [ 'slides_style_custom' => 'yes' ]
			]
		);

		/** Overlay background. */
		$slides->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'slides_style_overlay_background',
				'label' => esc_html__( 'Overlay background', 'selection' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-sliper-box .mdp-sliper-box-content',
				'condition' => [ 'slides_style_custom' => 'yes' ]
			]
		);

		$slides->end_controls_tab();

		$slides->end_controls_tabs();

		$this->add_control(
			'slides_items',
			[
				'label'       => esc_html__( 'Slides', 'selection' ),
				'type'        => Controls_Manager::REPEATER,
				'fields'      => $slides->get_controls(),
				'default'     => [
					[
						'slides_content_title' => esc_html__( 'Slide 1', 'selection' ),
						'slides_content_description' => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'selection' ),
						'slides_content_button_text' => esc_html__( 'More', 'selection' ),
					],
					[
						'slides_content_title' => esc_html__( 'Slide 2', 'selection' ),
						'slides_content_description' => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'selection' ),
						'slides_content_button_text' => esc_html__( 'More', 'selection' ),
					],
					[
						'slides_content_title' => esc_html__( 'Slide 3', 'selection' ),
						'slides_content_description' => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'selection' ),
						'slides_content_button_text' => esc_html__( 'More', 'selection' ),
					]
				],
				'title_field' => 'Slider - {{{ slides_content_title }}}'
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 *  TAB: Style.
	 */

	/**
	 * Slider styles TAB.
	 */
	public function get_slider_section_styles() {

		/** Start section */
		$this->start_controls_section( 'section_slider_style',
			[
				'label' => esc_html__( 'Slider', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

		/** Width. */
		$this->add_responsive_control(
			'slider_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper' => 'width: {{size}}{{unit}}; overflow: hidden;',
				],
			]
		);

		/** Height. */
		$this->add_responsive_control(
			'slider_style_height',
			[
				'label'           => esc_html__( 'Height', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%', 'vh' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
                        'max' => 2000
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 500,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper .flickity-viewport' => 'height: {{size}}{{unit}}!important',
				],
				'separator' => 'after'
			]
		);

		$this->get_settings_style_group(
			'slider_style',
			'{{WRAPPER}} .mdp-sliper',
			[],
			[
				'padding' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Overlay styles TAB.
	 */
	public function get_overlay_section_styles() {

		/** Start section */
		$this->start_controls_section( 'overlay_style',
			[
				'label' => esc_html__( 'Overlay', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

            /** Width. */
            $this->add_responsive_control(
                'overlay_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%' ],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'selectors'  => [
	                    '{{WRAPPER}} .mdp-sliper-box-content' =>
		                    'width: calc({{size}}{{unit}} - {{sliper_overlay_css_filter_margin.right}}{{sliper_overlay_css_filter_margin.unit}} - {{sliper_overlay_css_filter_margin.left}}{{sliper_overlay_css_filter_margin.unit}});',
                    ],
                ]
            );

            $this->get_settings_style_group(
                'overlay_box_style',
                '.mdp-sliper-box',
                [],
                [
                    'alignment-horizontal' => true,
                    'alignment-vertical' => true,
                ]
            );

            $this->get_settings_style_group(
                'overlay_box_align_style',
                '.mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-title, '.
                '.mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-box-description, '.
                '.mdp-sliper-box .mdp-sliper-box-content .mpd-sliper-button-link, '.
                '.mdp-sliper-box .mdp-sliper-box-content .mpd-sliper-button',
                [],
                [
                    'alignment' => true,
                ]
            );

            $this->get_settings_style_group(
                'overlay_css_filter',
                '.mdp-sliper-box-content',
                [],
                [
                    'margin' => true,
                ]
            );

            $this->get_settings_style_group(
                'overlay_css_filter',
                '.mdp-sliper-box-overlay',
                [],
                [
	                'background' => true,
	                'border' => true,
	                'border-radius' => true,
                    'css-filter' => true,
                ]
            );

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Slide styles TAB.
	 */
	public function get_slide_section_styles() {

		/** Start section */
		$this->start_controls_section( 'slide_style',
			[
				'label' => esc_html__( 'Slide', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

		/** Width. */
		$this->add_responsive_control(
			'slide_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%' ],
				'range'           => [
					'%' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-carousel-cell' => 'width: {{size}}{{unit}}',
				],
			]
		);

		$this->get_settings_style_group(
			'slide_carousel_cell_style_settings',
			'.mdp-carousel-cell',
			[],
			[
				'padding' => true,
			]
		);

		$this->get_settings_style_group(
			'slide_style_settings',
			'.mdp-carousel-cell',
			[],
			[
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'css-filter' => true,
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Title styles TAB.
	 */
	public function get_title_section_styles() {

		/** Start section */
		$this->start_controls_section( 'title_style',
			[
				'label' => esc_html__( 'Title', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

		/** HTML Tag. */
		$this->add_control(
			'title_style_tag',
			[
				'label' => esc_html__( 'HTML Tag', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'options' => [
					'h1' => 'H1',
					'h2' => 'H2',
					'h3' => 'H3',
					'h4' => 'H4',
					'h5' => 'H5',
					'h6' => 'H6',
					'div' => 'div',
					'span' => 'span',
					'p' => 'p',
				],
				'default' => 'h3',
			]
		);

		/** Width. */
		$this->add_responsive_control(
			'title_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-title-link > .mdp-sliper-title, {{WRAPPER}} .mdp-sliper-title' =>
                        'width: calc({{size}}{{unit}} - {{sliper_title_style_margin.right}}{{sliper_title_style_margin.unit}} - {{sliper_title_style_margin.left}}{{sliper_title_style_margin.unit}});',
				],
			]
		);

		$this->get_settings_style_group(
			'title_style',
			'.mdp-sliper-box .mdp-sliper-box-content .mdp-sliper-title, {{WRAPPER}} .mdp-sliper-title',
			[],
			[
				'margin' => true,
				'padding' => true,
				'color' => true,
				'typography' => true,
				'text-shadow' => true,
				'alignment' => true,
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Description styles TAB.
	 */
	public function get_description_section_styles() {

		/** Start section */
		$this->start_controls_section( 'description_style',
			[
				'label' => esc_html__( 'Description', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

		/** Width. */
		$this->add_responsive_control(
			'description_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-box-description' =>
						'width: calc({{size}}{{unit}} - {{sliper_description_style_margin.right}}{{sliper_description_style_margin.unit}} - {{sliper_description_style_margin.left}}{{sliper_description_style_margin.unit}});',
				],
			]
		);

		$this->get_settings_style_group(
			'description_style',
			'.mdp-sliper-box-description',
			[],
			[
				'margin' => true,
				'padding' => true,
				'color' => true,
				'typography' => true,
				'text-shadow' => true,
				'alignment-justify' => true,
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Category styles TAB.
	 */
	public function get_category_section_styles() {

		/** Start section */
		$this->start_controls_section( 'category_style',
			[
				'label' => esc_html__( 'Category', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => [ 'general_source' => [ 'posts', 'woocommerce' ] ]
			] );

		/** Width. */
		$this->add_control(
			'category_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-box-category' =>
						'width: calc({{size}}{{unit}} - {{sliper_category_style_margin.right}}{{sliper_category_style_margin.unit}} - {{sliper_category_style_margin.left}}{{sliper_category_style_margin.unit}});',
				],
			]
		);

		$this->get_settings_style_group(
			'category_style',
			'.mdp-sliper-box-category',
			[],
			[
				'margin' => true,
				'padding' => true,
				'color' => true,
				'typography' => true,
				'text-shadow' => true,
				'alignment-justify' => true,
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Price styles TAB.
	 */
	public function get_price_section_styles() {

		/** Start section */
		$this->start_controls_section( 'price_style',
			[
				'label' => esc_html__( 'Price', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => [ 'general_source' => 'woocommerce' ]
			] );

		/** Width. */
		$this->add_control(
			'price_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-box-price' =>
						'width: calc({{size}}{{unit}} - {{sliper_price_style_margin.right}}{{sliper_price_style_margin.unit}} - {{sliper_price_style_margin.left}}{{sliper_price_style_margin.unit}});',
				],
			]
		);

		$this->get_settings_style_group(
			'price_ins_style',
			'.mdp-sliper-box-price ins, {{WRAPPER}} .mdp-sliper-box-price ins span.woocommerce-Price-amount.amount, {{WRAPPER}} .mdp-sliper-box-price span.woocommerce-Price-amount.amount',
			[],
			[
				'margin' => true,
				'padding' => true,
				'color' => true,
				'typography' => true,
				'text-shadow' => true,
				'alignment-justify' => true,
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		$this->add_control(
			'price_header_style',
			[
				'label' => esc_html__( 'Discount', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->get_settings_style_group(
			'price_style',
			'.mdp-sliper-box-price del, {{WRAPPER}} .mdp-sliper-box-price del span.woocommerce-Price-amount.amount',
			[],
			[
				'margin' => true,
				'padding' => true,
				'color' => true,
				'typography' => true,
				'text-shadow' => true,
				'alignment-justify' => true,
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Background styles TAB.
	 */
	public function get_image_section_styles() {

		/** Start section */
		$this->start_controls_section( 'image_style',
			[
				'label' => esc_html__( 'Background', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

		/** Background settings */
		$this->add_control(
			'image_header_style',
			[
				'label' => esc_html__( 'Background settings', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [ 'general_source' => ['posts', 'woocommerce', 'custom-posts'] ],
			]
		);

		/** Position */
		$this->add_control(
			'image_style_position',
			[
				'label' => esc_html__( 'Position', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'center center',
				'options' => [
					'center center'  => esc_html__( 'Center Center', 'selection' ),
					'center left'  => esc_html__( 'Center Left', 'selection' ),
					'center right'  => esc_html__( 'Center Right', 'selection' ),
					'top center'  => esc_html__( 'Top Center', 'selection' ),
					'top left'  => esc_html__( 'Top Left', 'selection' ),
					'top right'  => esc_html__( 'Top Right', 'selection' ),
					'bottom center'  => esc_html__( 'Bottom Center', 'selection' ),
					'bottom left'  => esc_html__( 'Bottom Left', 'selection' ),
					'bottom right'  => esc_html__( 'Bottom Right', 'selection' ),
					'custom'  => esc_html__( 'Custom', 'selection' ),
				],
				'condition' => [ 'general_source' => ['posts', 'woocommerce', 'custom-posts'] ],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'background-position: {{VALUE}}',
				],
			]
		);

		/** X Position. */
		$this->add_control(
			'image_style_x_position',
			[
				'label'           => esc_html__( 'X Position', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'background-position-x: {{size}}{{unit}};',
				],
				'condition' => [ 'image_style_position' => 'custom' ],
			]
		);

		/** Y Position. */
		$this->add_control(
			'image_style_y_position',
			[
				'label'           => esc_html__( 'Y Position', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'background-position-y: {{size}}{{unit}};',
				],
				'condition' => [ 'image_style_position' => 'custom' ],
			]
		);

		/** Repeat */
		$this->add_control(
			'image_style_repeat',
			[
				'label' => esc_html__( 'Repeat', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'no-repeat',
				'options' => [
					'no-repeat'  => esc_html__( 'No-repeat', 'selection' ),
					'repeat'  => esc_html__( 'Repeat', 'selection' ),
					'repeat-x'  => esc_html__( 'Repeat-x', 'selection' ),
					'repeat-y'  => esc_html__( 'Repeat-y', 'selection' ),
				],
				'condition' => [ 'general_source' => ['posts', 'woocommerce', 'custom-posts'] ],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'background-repeat: {{VALUE}}',
				],
			]
		);

		/** Size */
		$this->add_control(
			'image_style_size',
			[
				'label' => esc_html__( 'Size', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'cover',
				'options' => [
					'auto'  => esc_html__( 'Auto', 'selection' ),
					'cover'  => esc_html__( 'Cover', 'selection' ),
					'contain'  => esc_html__( 'Contain', 'selection' ),
					'custom'  => esc_html__( 'Custom', 'selection' ),
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'background-size: {{value}};',
				],
				'condition' => [ 'general_source' => ['posts', 'woocommerce', 'custom-posts'] ]
			]
		);

		/** Type image size. */
		$this->add_control(
			'image_style_type_size',
			[
				'label' => esc_html__( 'Type image size', 'selection' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'large',
				'options' => [
					'thumbnail'  => esc_html__( 'Thumbnail', 'selection' ),
					'medium'  => esc_html__( 'Medium', 'selection' ),
					'large'  => esc_html__( 'Large', 'selection' ),
					'full'  => esc_html__( 'Full', 'selection' ),
				],
				'condition' => [ 'general_source' => ['posts', 'woocommerce', 'custom-posts'] ]
			]
		);

		/** Width. */
		$this->add_responsive_control(
			'image_style_size_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'background-size: {{size}}{{unit}};',
				],
				'condition' => [ 'image_style_size' => 'custom' ],
			]
		);

		$this->add_control(
			'image_header_item_style',
			[
				'label' => esc_html__( 'Image settings', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [ 'general_source' => ['posts', 'woocommerce', 'custom-posts'] ]
			]
		);

		/** Width. */
		$this->add_responsive_control(
			'image_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'width: {{size}}{{unit}};',
				],
			]
		);

		/** Height. */
		$this->add_responsive_control(
			'image_style_height',
			[
				'label'           => esc_html__( 'Height', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%', 'vh' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
                        'max' => 3000
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'height: {{size}}{{unit}};',
				],
			]
		);

		/** Alignment */
		$this->add_control(
			'image_alignment_style',
			[
				'label' => esc_html__( 'Alignment', 'selection' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'selection' ),
						'icon' => 'fa fa-align-left',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'selection' ),
						'icon' => 'fa fa-align-right',
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .mdp-sliper-image' => 'float: {{value}};',
				],
				'condition' => [ 'general_type' => 'card' ],
			]
		);

		$this->get_settings_style_group(
			'image_style',
			'.mdp-sliper-image',
			[],
			[
				'margin' => true,
				'padding' => true,
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
				'css-filter' => true
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Button styles TAB.
	 */
	public function get_button_section_styles() {

		/** Start section */
		$this->start_controls_section( 'button_style',
			[
				'label' => esc_html__( 'Button', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
			] );

		/** Width. */
		$this->add_control(
			'button_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mpd-sliper-button-link, .mpd-sliper-button' => 'width: {{size}}{{unit}};',
				],
			]
		);

		$this->get_settings_style_group(
			'button_style',
			'.mpd-sliper-button-link span, {{WRAPPER}} .mpd-sliper-button',
			[],
			[
				'margin' => true,
				'padding' => true,
				'typography' => true,
				'text-shadow' => true,
			],
			'none'
		);

		$this->get_settings_style_group(
			'button_style_align',
			'.mpd-sliper-button-link, {{WRAPPER}} .mpd-sliper-button',
			[],
			[
				'alignment' => true,
			],
			'none'
		);

		$this->get_settings_style_group(
			'button_style_border',
			'.mpd-sliper-button-link span, {{WRAPPER}} .mpd-sliper-button',
			[],
			[
				'border-radius' => true,
			],
			'none'
		);

		$this->start_controls_tabs( 'tabs_slides_button' );

		/** Content tab. */
		$this->start_controls_tab(
			'tab_normal',
			[
				'label' => esc_html__('NORMAL', 'selection'),
			]
		);

		$this->get_settings_style_group(
			'button_style_normal',
			'.mpd-sliper-button-link span, {{WRAPPER}} .mpd-sliper-button',
			[],
			[
				'color' => true,
				'background' => true,
				'border' => true,
				'box-shadow' => true,
			]
		);

		$this->end_controls_tab();

		/** Content tab. */
		$this->start_controls_tab(
			'tab_hover',
			[
				'label' => esc_html__('HOVER', 'selection')
			]
		);

		$this->get_settings_style_group(
			'button_style_hover',
			'.mpd-sliper-button-link span:hover, {{WRAPPER}} .mpd-sliper-button:hover',
			[],
			[
				'color' => true,
				'background' => true,
				'border' => true,
				'box-shadow' => true,
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Navigation styles TAB.
	 */
	public function get_navigation_section_styles() {

		/** Start section */
		$this->start_controls_section( 'navigation_style',
			[
				'label' => esc_html__( 'Navigation', 'selection' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'conditions' => [
					'relation' => 'or',
					'terms' => [
						[
							'name' => 'navigation_navigation_arrows',
							'operator' => '===',
							'value' => 'yes'
						],
						[
							'name' => 'navigation_navigation_dots',
							'operator' => '===',
							'value' => 'yes'
						]
					]
				]
			] );

		/** Arrows. */
		$this->add_control(
			'navigation_style_arrows_heading',
			[
				'label' => esc_html__( 'Arrows', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		/** Button position. */
		$this->add_responsive_control(
			'navigation_style_position_arrows',
			[
				'label'           => esc_html__( 'Button position', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => -100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-prev-next-button.previous' => 'left: {{size}}{{unit}}',
					'{{WRAPPER}} .flickity-prev-next-button.next' => 'right: {{size}}{{unit}}',
				],
				'condition' => [ 'general_rtl!' => 'yes', 'navigation_navigation_arrows' => 'yes' ]
			]
		);

		/** Button size */
		$this->add_responsive_control(
			'navigation_style_arrows_size',
			[
				'label'           => esc_html__( 'Button size', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 30,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-prev-next-button' => 'width: {{size}}{{unit}}!important; height: {{size}}{{unit}}!important;',
				],
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		/** Position arrows. */
		$this->add_responsive_control(
			'navigation_style_position_arrows_svg',
			[
				'label'           => esc_html__( 'Arrows position', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 20,
				],
				'separator' => 'before',
				'selectors'  => [
					'{{WRAPPER}} .flickity-prev-next-button .flickity-button-icon' => 'left: {{size}}{{unit}}; top: {{size}}{{unit}}',
				],
				'condition' => [ 'general_rtl!' => 'yes', 'navigation_navigation_arrows' => 'yes' ]
			]
		);

		/** Size Arrows */
		$this->add_responsive_control(
			'navigation_style_arrows_size_svg',
			[
				'label'           => esc_html__( 'Arrows size', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 60,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-prev-next-button .flickity-button-icon' => 'width: {{size}}{{unit}}!important; height: {{size}}{{unit}}!important;',
				],
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		$this->get_settings_style_group(
			'style_arrows',
			'.flickity-button',
			[ 'navigation_navigation_arrows' => 'yes' ],
			[
				'margin' => true,
				'padding' => true,
			]
		);

		/** Border */
		$this->add_control(
			'navigation_style_arrows_border_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'selection' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em' ],
				'selectors'  => [
					'{{WRAPPER}} .flickity-button' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
				],
				'default' => [
					'top' => '50',
					'right' => '50',
					'bottom' => '50',
					'left' => '50',
					'unit' => 'px',
				],
				'toggle'     => true,
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		$this->start_controls_tabs( 'tabs_slides_navigation');

		/** Content tab. */
		$this->start_controls_tab(
			'tab_normal_navigation',
			[
				'label' => esc_html__('NORMAL', 'selection'),
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		/** Color. */
		$this->add_control(
			'navigation_style_arrows_color',
			[
				'label' => esc_html__( 'Color', 'selection' ),
				'type' => Controls_Manager::COLOR,
				'scheme' => [
					'type' => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-button-icon' => 'fill: {{VALUE}}',
				],
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		/** Background */
		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'navigation_style_arrows_background',
				'label' => esc_html__( 'Background', 'selection' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .flickity-button',
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],

			]
		);

		$this->end_controls_tab();

		/** Content tab. */
		$this->start_controls_tab(
			'tab_hover_navigation',
			[
				'label' => esc_html__('HOVER', 'selection'),
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		/** Color. */
		$this->add_control(
			'navigation_style_arrows_hover_color',
			[
				'label' => esc_html__( 'Color', 'selection' ),
				'type' => Controls_Manager::COLOR,
				'scheme' => [
					'type' => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-button:hover .flickity-button-icon' => 'fill: {{VALUE}}',
				],
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		/** Background */
		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'navigation_style_arrows_background_hover',
				'label' => esc_html__( 'Background', 'selection' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .flickity-button:hover',
				'condition' => [ 'navigation_navigation_arrows' => 'yes' ],
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		/** Dots. */
		$this->add_control(
			'navigation_style_dots_heading',
			[
				'label' => esc_html__( 'Dots', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'condition' => [ 'navigation_navigation_dots' => 'yes' ],
			]
		);

		/** Position dots. */
		$this->add_responsive_control(
			'navigation_style_position_dots',
			[
				'label'           => esc_html__( 'Position', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%', 'vh' ],
				'range'           => [
					'px' => [
						'min'  => -1000,
						'max'  => 1000,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 25,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-page-dots' => 'bottom: {{size}}{{unit}}',
				],
				'condition' => [ 'navigation_navigation_dots' => 'yes' ],
			]
		);

		/** Position dots. */
		$this->add_responsive_control(
			'navigation_style_rotation_dots',
			[
				'label'           => esc_html__( 'Rotation', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 0,
						'max'  => 360,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-page-dots' => 'transform: rotate({{size}}deg)',
				],
				'condition' => [ 'navigation_navigation_dots' => 'yes' ],
			]
		);

		/** Size */
		$this->add_responsive_control(
			'navigation_style_dots_size',
			[
				'label'           => esc_html__( 'Size', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 10,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-page-dots .dot' => 'width: {{size}}{{unit}}; height: {{size}}{{unit}}',
				],
				'condition' => [ 'navigation_navigation_dots' => 'yes' ],
			]
		);

		/** Opacity */
		$this->add_responsive_control(
			'navigation_style_dots_opacity',
			[
				'label'           => esc_html__( 'Opacity', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 0.1,
						'step' => 0.1,
						'max' => 1
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0.25,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-page-dots .dot' => 'opacity: {{size}}',
					'{{WRAPPER}} .flickity-page-dots .dot.is-selected ' => 'opacity: 1',
				],
				'condition' => [ 'navigation_navigation_dots' => 'yes' ],
			]
		);

		/** Color. */
		$this->add_control(
			'navigation_style_dots_color',
			[
				'label' => esc_html__( 'Color', 'selection' ),
				'type' => Controls_Manager::COLOR,
				'scheme' => [
					'type' => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'selectors'  => [
					'{{WRAPPER}} .flickity-page-dots .dot' => 'background: {{VALUE}}',
				],
				'condition' => [ 'navigation_navigation_dots' => 'yes' ],
			]
		);

		/** End section */
		$this->end_controls_section();

	}

	/**
	 * Add the widget controls.
	 *
	 * @access protected
	 * @return void with category names
	 **/
	protected function register_controls() {

		/**
		 *  TAB: Content.
		 */

		/** General section. */
		$this->get_general_section_content();

		/** Navigation section. */
		$this->get_navigation_section_content();

		/** Layout section. */
		$this->get_layout_section_content();

		/** Slides section. */
		$this->get_slides_section_content();

		/**
		 *  TAB: Style.
		 */

		/** Slider styles section. */
		$this->get_slider_section_styles();

		/** Slide styles section. */
		$this->get_slide_section_styles();

		/** Overlay styles section. */
		$this->get_overlay_section_styles();

		/** Title styles section. */
		$this->get_title_section_styles();

		/** Description styles section. */
		$this->get_description_section_styles();

		/** Category styles section. */
		$this->get_category_section_styles();

		/** Price styles section. */
		$this->get_price_section_styles();

		/** Image styles section. */
		$this->get_image_section_styles();

		/** Button styles section. */
		$this->get_button_section_styles();

		/** Navigation styles section. */
		$this->get_navigation_section_styles();

	}

	/**
	 * We return an array with the names of the layouts.
	 *
	 * @param $settings - we get an array with all the data.
	 *
	 * @return array
	 */
	private function get_LayoutSortable( $settings ) {

		/** We get all the data on the lists of layouts. */
		$layout = $settings['layout_priority'];

		/** An array in which the names of the layouts and their width will be stored. */
		$res = array();

		/** We write the names of the layouts and their width into an array. */
		foreach ($layout as $key => $val){

			$res[] = [ 'name' => $val['layout_item'] ];
		}

		return $res;

	}

	/**
	 * Displays image for custom slide.
	 */
	public function get_custom_image() {
		?>
        <div class="mdp-sliper-image"></div>
		<?php
	}

	/**
	 * Displays title for custom slide.
	 *
	 * @param $settings - slide settings.
	 * @param $options - global settings.
	 */
	public function get_custom_title( $settings,  $options ) {
		?>

		<?php if( 'title' ===  $options['general_link_type'] ) :
			if ( ! empty( $settings['slides_content_link']['url'] ) ) {
				$this->add_link_attributes( 'slide_link', $settings['slides_content_link'] );
				$this->add_render_attribute( 'slide_link', 'class', 'mdp-sliper-title-link' );
			} ?>
            <a <?php echo $this->get_render_attribute_string( 'slide_link' ); ?>>
		<?php  endif;?>

        <<?php echo  esc_attr($options['title_style_tag']); ?> class="mdp-sliper-title">
		<?php echo esc_html( $settings['slides_content_title'] ); ?>
        </<?php echo  esc_attr($options['title_style_tag']); ?>>

		<?php if( 'title' ===  $options['general_link_type']  ) : ?>
            </a>
		<?php  endif; ?>

		<?php
	}

	/**
	 *  Displays description for custom slide.
	 *
	 * @param $settings - slide settings.
	 */
	public function get_custom_description( $settings ) {
		$allowed_html = array(
			'a' => array(
				'href'  => true,
				'title' => true,
                'style' => true,
                'class' => true,
				'role' => true,
                'rel'   => true
			),
			'button' => array(
				'title' => true,
				'style' => true,
				'class' => true,
                'role' => true
			),
			'br'     => array(),
			'em'     => array(
				'class' => true,
				'style' => true
            ),
			'strong' => array(
				'class' => true,
				'style' => true
            ),
			'b' => array(
				'class' => true,
				'style' => true
			),
			'span' => array(
				'class' => true,
				'style' => true
            ),
            'div' => array(
	            'class' => true,
                'style' => true
            ),
            'h1' => array(
	            'class' => true,
	            'style' => true
            ),
            'h2' => array(
	            'class' => true,
	            'style' => true
            ),
            'h3' => array(
	            'class' => true,
	            'style' => true
            ),
            'h4' => array(
	            'class' => true,
	            'style' => true
            ),
            'h5' => array(
	            'class' => true,
	            'style' => true
            ),
            'h6' => array(
	            'class' => true,
	            'style' => true
            ),
            'ul' => array(
	            'class' => true,
	            'style' => true
            ),
            'li' => array(
	            'class' => true,
	            'style' => true
            ),
            'ol' => array(
	            'class' => true,
	            'style' => true
            ),
            'iframe' => array(
                'src' => true,
                'width' => true,
                'height' => true,
                'frameborder' => true,
                'allowfullscreen' => true,
                'class' => true,
                'style' => true
            ),
			'img' => array(
				'src' => true,
				'width' => true,
				'height' => true,
				'alt' => true,
				'class' => true,
				'style' => true
			),
			'figure' => array(
				'class' => true,
				'style' => true
			),
			'figcaption' => array(
				'class' => true,
				'style' => true
			),
		);
		?>
        <div class="mdp-sliper-box-description">
            <p><?php echo wp_kses( $settings['slides_content_description'], $allowed_html ); ?></p>
        </div>
		<?php
	}

	/**
	 * Displays button for custom slide.
	 *
	 * @param $settings - slide settings.
	 * @param $options - global settings.
	 */
	public function get_custom_button( $settings, $options ) {
		?>
		<?php if( 'button' ===  $options['general_link_type'] ) :
            if ( ! empty( $settings['slides_content_link']['url'] ) ) {
                $this->add_link_attributes( 'slide_link', $settings['slides_content_link'] );
                $this->add_render_attribute( 'slide_link', 'class', 'mpd-sliper-button-link' );
            } ?>
            <a <?php echo $this->get_render_attribute_string( 'slide_link' ); ?>>
                <span>
		<?php  endif;?>

		<?php if( 'button' !==  $options['general_link_type'] ) : ?>
		    <span class="mpd-sliper-button">
	    <?php  endif;?>
            <?php echo esc_html( $settings['slides_content_button_text'] ); ?>
		<?php if( 'button' !==  $options['general_link_type'] ) : ?>
            </span>
        <?php  endif;?>

		<?php if( 'button' ===  $options['general_link_type']  ) : ?>
                </span>
            </a>
		<?php  endif; ?>

		<?php
	}

	/**
	 *  Display custom slider.
	 *
	 * @param $settings       - Slider settings.
	 * @param $LayoutSortable - An array of slides layouts.
	 * @param $options        - Global settings.
	 * @param $count          - Cycle counter.
	 */
	public function get_custom_slides_box( $settings, $LayoutSortable,  $options, $count ) {

		$class_inactive = 'mdp-' . $options["general_transition"] .'-inactive';
		if( $count <= ($options['general_display_slides']['size'] - 1 ) ){
			$class_inactive = 'mdp-' . $options["general_transition"] .'-active';
		}

        $slide_id = $settings['slides_content_slide_id'] !== '' ? $settings['slides_content_slide_id'] : 'sliper-slide-' . ( $count + 1 );
		?>

        <div id="<?php esc_attr_e( $slide_id ); ?>"
             class="carousel-cell
                    mdp-carousel-cell
                    mdp-<?php echo esc_attr( $options["general_transition"] ); ?>-active-sliper
                    elementor-repeater-item-<?php echo  esc_attr( $settings[ '_id'] ); ?>

                    <?php echo esc_attr( $class_inactive ); ?>" >

			<?php
			/** Makes an entire block link. */
			if( 'box' ===  $options['general_link_type'] ) :
                if ( ! empty( $settings['slides_content_link']['url'] ) ) {
                    $this->add_link_attributes( 'slide_link', $settings['slides_content_link'] );
                } ?>
                <a <?php echo $this->get_render_attribute_string( 'slide_link' ); ?>>
				<?php  endif; ?>

				<?php
				/** Overlay style. */
				if( 'overlay' === $options['general_type'] ):
				    foreach ($LayoutSortable as $val) {
					    if ( 'image' === $val[ 'name' ] ) {
						    $this->get_custom_image();
					    }
				    }
				?>

                <div class="mdp-sliper-box">

                    <div class="mdp-sliper-box-content">
                        <div class="mdp-sliper-box-overlay"></div>
                        <?php
                        endif;

                        /** We list the layouts in the given order. */
                        foreach ( $LayoutSortable as $val ) {

                            switch ( $val[ 'name' ] ) {
                                case 'image':
                                    if( 'overlay' !== $options['general_type'] ) {
                                        $this->get_custom_image();
                                    }
                                    break;
                                case 'title':
                                    $this->get_custom_title( $settings, $options );
                                    break;
                                case 'description':
                                    $this->get_custom_description( $settings );
                                    break;
                                case 'button':
                                    $this->get_custom_button( $settings, $options );
                                    break;
                                default:
                                    echo '';
                            }

                        }
                        ?>

                        <?php
                        /** End overlay style. */
                        if( 'overlay' === $options['general_type'] ): ?>
                    </div>
                </div>
			<?php  endif; ?>

            <?php
				/** End box link. */
				if( 'box' ===  $options['general_link_type'] ) : ?>
                </a>
		    <?php  endif; ?>

        </div>

		<?php
	}

	/**
	 * Custom: We transfer data to create a slider.
	 *
	 * @param $settings - Global settings.
	 * @param $LayoutSortable - An array of slides layouts.
	 */
	public function get_custom_info( $settings, $LayoutSortable ) {

		/** Count the number. */
		$quantity_custom = count( $settings['slides_items'] ) - 1;

		/** We display image-box item in a cycle. */
		for ( $i = 0; $i <= $quantity_custom; $i++) {
			$this->get_custom_slides_box( $settings['slides_items'][$i], $LayoutSortable, $settings, $i );
		}

	}

	/**
	 * Displays image for post slide.
	 *
	 * @param $data - Post data.
	 * @param $options - Global settings.
	 */
	public function get_posts_image( $data, $options ) {

		$image_url = get_the_post_thumbnail_url( $data, $options["image_style_type_size"] );

		if( empty($image_url) ){ $image_url =  Utils::get_placeholder_image_src(); }  ?>

        <div style="background-image: url(<?php echo esc_url($image_url); ?>);" class="mdp-sliper-image"></div>
		<?php
	}

	/**
	 * Displays title for post slide.
	 *
	 * @param $data - post data.
	 * @param $options - Global settings.
	 */
	public function get_posts_title( $data, $options ){

		/** Makes the title a link. */
		if( 'title' === $options['general_link_type'] ) :
			?>
            <a href="<?php echo esc_url( $data->guid ); ?>" class="mdp-sliper-title-link" target="<?php echo esc_attr( $options[ 'general_link_target' ] ); ?>">
		<?php  endif; ?>

        <<?php echo  esc_attr($options['title_style_tag']); ?> class="mdp-sliper-title">
		<?php  echo wp_kses_post( get_the_title( $data ) ); ?>
        </<?php echo  esc_attr($options['title_style_tag']); ?>>

		<?php if( 'title' === $options['general_link_type'] ) : ?>
            </a>
		<?php  endif; ?>

		<?php
	}

	/**
	 * Displays description for post slide.
	 *
	 * @param $data - post data.
	 */
	public function get_posts_description( $data ) {
		$excerpt = get_the_excerpt($data);
		if( !empty($excerpt ) ):
            ?>
            <div class="mdp-sliper-box-description">
                <p><?php echo esc_html( $excerpt ); ?></p>
            </div>
            <?php
        endif;
	}

	/**
	 * Displays button for post slide.
	 *
	 * @param $data - post data.
	 * @param $options - global settings.
	 */
	public function get_posts_button( $data, $options ) {
		if( 'button' === $options['general_link_type'] ) :
			?>
            <a href="<?php echo esc_url( get_post_permalink( $data ) ); ?>" class="mpd-sliper-button-link" target="<?php echo esc_attr( $options[ 'general_link_target' ] ); ?>">
                <span><?php echo esc_html( $options['general_button_text'] ); ?></span>
            </a>
		<?php
		endif;
	}

	/**
	 * Displays category for post slide.
	 *
	 * @param $data - post data.
	 */
	public function get_posts_category( $data ) {
		$category = get_the_category( $data->ID );
		$wc_cat = get_the_terms( $data->ID, 'product_cat' );

		if( isset($category[0]->name) || isset($wc_cat[0]->name) ):
			?>

            <p class="mdp-sliper-box-category">
				<?php
				if( isset($category[0]->name) ){
					echo esc_html($category[0]->name);
				}elseif( isset($wc_cat[0]->name) ){
					echo esc_html( $wc_cat[0]->name );
				}
				?>
            </p>

		<?php
		endif;
	}

	/**
	 * Displays price of the product.
	 *
	 * @param $settings - Post data.
	 * @param $options - Settings for displaying product prices.
	 */
	public function get_posts_price( $settings, $options ){

		$product = wc_get_product($settings->ID );
		if( 'woocommerce' === $options['general_source'] ):
			?>

            <p class="mdp-sliper-box-price">
				<?php echo wp_kses_post( $product->get_price_html(), 'post' ); ?>
            </p>

		<?php
		endif;
	}

	/**
	 * Display post slider.
	 *
	 * @param $settings       - Post data.
	 * @param $LayoutSortable - An array of slides layouts.
	 * @param $options        - Global settings.
	 * @param $count          - Cycle counter.
	 */
	public function get_posts_sliper_box(  $settings, $LayoutSortable, $options, $count ) {

		$class_inactive = 'mdp-' . $options["general_transition"] .'-inactive';
		if( $count <= ($options['general_display_slides']['size'] - 1 ) ){
			$class_inactive = 'mdp-' . $options["general_transition"] .'-active';
		}
		?>

        <div id="<?php esc_attr_e( 'sliper-slide-' . ( $count + 1 ) ); ?>"
             class="carousel-cell
                    mdp-carousel-cell
                    mdp-<?php echo esc_attr( $options["general_transition"] ); ?>-active-sliper
                    mdp-carousel-cell-<?php echo esc_attr( $this->get_id() ); ?>
                    <?php echo esc_attr( $class_inactive ); ?>">

			<?php if( 'box' === $options['general_link_type'] ) : ?>
            <a href="<?php echo esc_url( get_post_permalink( $settings ) ); ?>" target="<?php echo esc_attr( $options[ 'general_link_target' ] ); ?>">
				<?php  endif; ?>

				<?php
				/** Overlay style. */
				if( 'overlay' === $options['general_type'] ):
				    foreach ($LayoutSortable as $val) {
				        if( 'image' === $val['name'] ){
					        $this->get_posts_image( $settings, $options );
                        }
				    }
				?>

                <div class="mdp-sliper-box">
                    <div class="mdp-sliper-box-content">
                        <div class="mdp-sliper-box-overlay"></div>
						<?php
						endif;

						/** We list the layouts in the given order. */
						foreach ($LayoutSortable as $val) {

							switch ( $val['name'] ) {
								case 'image':
									if( 'overlay' !== $options['general_type'] ) {
										$this->get_posts_image( $settings, $options );
									}
									break;
								case 'title':
									$this->get_posts_title( $settings, $options );
									break;
								case 'description':
									$this->get_posts_description( $settings );
									break;
								case 'category':
									$this->get_posts_category( $settings );
									break;
								case 'price':
									$this->get_posts_price( $settings, $options );
									break;
								case 'button':
									$this->get_posts_button( $settings, $options );
									break;
								default:
									echo '';
							}

						}
						?>

						<?php
						/** End overlay style. */
						if( 'overlay' === $options['general_type'] ): ?>
                    </div>
                </div>
			<?php  endif; ?>

				<?php if( 'box' === $options['general_link_type'] ) : ?>
            </a>
		<?php  endif; ?>

        </div>

		<?php
	}

	/**
	 * Posts: We transfer data to create a slider.
	 *
	 * @param $settings - Global settings.
	 * @param $LayoutSortable - An array of slides layouts.
	 */
	public function get_posts_info( $settings, $LayoutSortable ) {

		/** We get an array with post data. */
		$posts = get_posts( [
            'category_name'     => $settings['general_posts_category'],
            'post_status'       => 'publish',
            'numberposts'       => -1
        ] );
		if ( $settings['general_source'] === 'custom-posts' ) { $posts = $settings[ 'general_item_custom_posts' ]; }

		/** We count the number of image-box item in the category. */
        if( !empty($posts) ){

            $posts_to_show = array();
            foreach ( $posts as &$post_id ) {

	            if ( get_post_status( $post_id ) === 'publish' ) {

		            $posts_to_show[] = $post_id;

                }

            }

	        $quantity_posts = count( $posts_to_show );

	        if ( $quantity_posts > 0 ) {

		        /** We display image-box item in a cycle. */
		        for ( $i = 0; $i <= $quantity_posts - 1; $i ++ ) {
			        $this->get_posts_sliper_box( $posts_to_show[ $i ], $LayoutSortable, $settings, $i );
		        }

	        } else {
		        echo esc_html__( 'There are no posts in this category.', 'selection' );
	        }
        }else {
	        echo esc_html__( 'There are no posts in this category.', 'selection' );
        }

	}

	/**
	 * Woocommerce: We transfer data to create a slider.
	 *
	 * @param $settings - Global settings.
	 * @param $LayoutSortable - An array of slides layouts.
	 */
	public function get_woocommerce_info( $settings, $LayoutSortable ) {
		/** We get an array with post data. */
		$product = get_posts( [ 'product_cat' => $settings['general_products_category'], 'post_type'   => 'product', 'numberposts' => -1 ] );

		/** We count the number of image-box item in the category. */
		$quantity = count( $product );

		if( $quantity > 0 ){
			/** We display image-box item in a cycle. */
			for ( $i = 0; $i <= $quantity - 1; $i++) {
				$this->get_posts_sliper_box( $product[$i], $LayoutSortable, $settings, $i );
			}
		}else{
			echo esc_html__('There are no products in this category.', 'selection');
		}

	}

	/**
	 *  Returns the type of device used.
	 */
	public function get_device() {

		$detect = new MobileDetect;

		/** Detect current user device. */
		$device = 'desktop';

		if ( $detect->isTablet() ) {
			$device = 'tablet';
		}

		if ( $detect->isMobile() && ! $detect->isTablet() ) {
			$device = 'mobile';
		}

		return $device;

	}

	/**
	 * Render Frontend Output. Generate the final HTML on the frontend.
	 *
	 * @access protected
	 **/
	protected function render() {

		/** Get all the values from the admin panel. */
		$settings = $this->get_settings_for_display();

		/** We return an array with the names of the layouts and their width. */
		$LayoutSortable = $this->get_LayoutSortable( $settings );

		$watchCss = 'false';

        /** Show message is plugin was configured for work with woo products, but for now WooCommerce is disabled in WordPress */
        if ( $settings['general_source'] === 'woocommerce' && ! class_exists( 'woocommerce' ) ) {
            if ( is_admin() ) {
	            echo esc_html__( 'This slider is use the WooCommerce products, but is WooCommerce plugin is disabled.', 'selection' );
            }
            return;
        }

		/** Show message is plugin was configured for work with custom post type, but for post type is not exists */
        if ( $settings['general_source'] === 'custom-posts' ) {

            /** Remove custom posts that is not public */
	        $posts_to_show = array();
	        foreach ( $settings[ 'general_item_custom_posts' ] as &$post_id ) {

		        if ( get_post_status( $post_id ) === 'publish' && post_type_exists( get_post_type( $post_id ) ) ) {

			        $posts_to_show[] = $post_id;

		        }

	        }
	        $settings[ 'general_item_custom_posts' ] = $posts_to_show;

	        if ( empty( $posts_to_show ) ) {
                if ( is_admin() ){
		            echo esc_html__( 'This slider is use the custom post type that is not exists.', 'selection' );
                }
		        return;
	        }

        }

		?>

        <div class="mdp-sliper-container">

            <div id="mdp-sliper-<?php echo esc_attr( $this->get_id() ); ?>"
                 class="mdp-sliper mdp-sliper-<?php echo esc_attr( $this->get_id() ); ?>"
                 data-autoplay="<?php esc_attr_e( $settings['navigation_display_duration'] !== null ? $settings['navigation_display_duration']['size'] : 'false' ); ?>"
                 data-keyboard="<?php echo ( 'yes' === $settings['navigation_keyboard_navigation'] ) ? 'true' : 'false'; ?>"
                 data-hash="<?php echo ( 'yes' === $settings['navigation_hash'] ) ? 'true' : 'false'; ?>"
                 data-dots="<?php echo ( 'yes' === $settings['navigation_navigation_dots'] ) ? 'true' : 'false'; ?>"
                 data-prevNext="<?php echo ( 'yes' === $settings['navigation_navigation_arrows'] ) ? 'true' : 'false'; ?>"
                 data-infiniteScrolling="<?php echo ( 'yes' === $settings['navigation_infinite_scrolling'] ) ? 'true' : 'false'; ?>"
                 data-pause="<?php echo ( 'yes' === $settings['navigation_pause_hover'] ) ? 'true' : 'false'; ?>"
                 data-scroll="<?php echo ( 'yes' === $settings['navigation_freely_scrolled'] ) ? 'true' : 'false'; ?>"
                 data-friction="<?php echo esc_attr( $settings['navigation_friction']['size'] ); ?>"
                 data-attraction="<?php echo esc_attr( $settings['navigation_attraction']['size'] ); ?>"
                 data-draggable="<?php echo ( 'yes' === $settings['navigation_draggable'] ) ? 'true' : 'false'; ?>"
                 data-dragThreshold="<?php esc_attr_e( $settings['navigation_drag_threshold'] !== null ? $settings['navigation_drag_threshold']['size'] : 'false' ); ?>"
                 data-rightToLeft="<?php echo ( 'yes' === $settings['general_rtl'] ) ? 'true' : 'false'; ?>"
                 data-cellAlign="<?php echo esc_attr($settings['general_slides_align']); ?>"
                 data-groupCells="<?php echo esc_attr($settings['general_display_slides']['size']); ?>"
                 data-initialIndex="<?php echo esc_attr( $settings['slides_initial_slide']['size'] ); ?>"
                 data-watchCss="<?php echo esc_attr($watchCss); ?>"
                 data-transition="<?php echo esc_attr( $settings['general_transition'] ); ?>"
                 data-slideWidth="<?php echo esc_attr( $settings['slide_style_width']['size'] ); ?>"
                style="opacity: 0; height: <?php esc_attr_e( $settings['slider_style_height']['size'] ); esc_attr_e( $settings['slider_style_height']['unit'] ) ?>">

                <?php

                switch ($settings['general_source']){

                    case 'custom':
                        $this->get_custom_info( $settings, $LayoutSortable );
                        break;

	                case 'custom-posts':
                    case 'posts':
                        $this->get_posts_info( $settings, $LayoutSortable );
                        break;

                    case 'woocommerce':
                        $this->get_woocommerce_info($settings, $LayoutSortable);
                        break;

                    default:
                        echo esc_html__( 'The source of information was not found', 'selection' );

                }
                ?>

            </div>

        </div>

        <!--suppress JSUnresolvedFunction -->
        <script>
        <?php if( \Elementor\Plugin::$instance->editor->is_edit_mode() ) : ?>
            jQuery( document ).ready( function () {
        <?php else: ?>
            document.addEventListener('DOMContentLoaded', function(){
        <?php endif; ?>

				<?php if ( is_admin() ) { ?>

                const sliperInitInterval = setInterval( () => {

                    window.Sliper();

                    if ( document.querySelectorAll( `#mdp-sliper-<?php esc_attr_e( $this->get_id() ); ?> .flickity-viewport .carousel-cell` ).length > 0 ) {
                        clearInterval( sliperInitInterval );
                    }

                }, 1 );

				<?php } else { ?>

                window.Sliper();

				<?php } ?>
            } );
        </script>

		<?php

	}

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/category/sliper-elementor';

    }
}
