<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Testimoner for Elementor
 * Customizable testimonial for Elementor editor
 * Exclusively on https://1.envato.market/testimoner-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Core\Schemes\Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Icons_Manager;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;
use Merkulove\Selection\Unity\ElementorControls;

/** @noinspection PhpUnused */
/**
 * Testimoner - Custom Elementor Widget.
 **/
class testimoner_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
             'mdp-selection-elementor-admin',
                UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );
        wp_register_style(
                'mdp-testimoner-elementor',
                   UnityPlugin::get_url() . 'css/testimoner-elementor' . UnityPlugin::get_suffix() . '.css',
                       [],
                       UnityPlugin::get_version()
        );
	    wp_register_script(
                'mdp-testimoner-elementor',
                   UnityPlugin::get_url() . 'js/testimoner-elementor' . UnityPlugin::get_suffix() . '.js',
                       [ 'jquery' ],
                       UnityPlugin::get_version(),
              true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-testimoner-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Testimonials', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-testimoner-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Testimonials' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-testimoner-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-testimoner-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Section Layout Settings. */
        $this->section_layout_settings();

        /** Content -> Section Description Settings. */
        $this->section_description_settings();

        /** Content -> Section Author Image Settings. */
        $this->section_author_image_settings();

        /** Content -> Section Author Settings. */
        $this->section_author_settings();

        /** Content -> Section Company Settings. */
        $this->section_company_settings();

        /** Content -> Section Rating Settings. */
        $this->section_rating_settings();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Description. */
        $this->section_style_description();

        /** Style -> Section Style Author. */
        $this->section_style_author();

        /** Style -> Section Style Company. */
        $this->section_style_company();

        /** Style -> Section Style Author Image. */
        $this->section_style_author_image();

        /** Style -> Section Style Rating. */
        $this->section_style_rating();

    }

    /**
     * Add widget controls: Content -> Layout Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_layout_settings() {

        $this->start_controls_section( 'section_layout_settings', [
            'label' => esc_html__( 'Layout', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $repeater = new Repeater();

        $repeater->add_control(
            'layout',
            [
                'label'     => esc_html__( 'Layout', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'author'    => esc_html__( 'Author', 'selection' ),
                    'company'    => esc_html__( 'Company', 'selection' ),
                    'description'    => esc_html__( 'Description', 'selection' ),
                    'rating'    => esc_html__( 'Rating', 'selection' ),
                    'author_image'    => esc_html__( 'Author image', 'selection' ),
                ],
                'default'   => 'author',
                'separator' => 'none',
            ]
        );

        $repeater->add_control(
            'author_tag',
            [
                'label'     => esc_html__( 'Tag', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'    => esc_html__( 'h1', 'selection' ),
                    'h2'    => esc_html__( 'h2', 'selection' ),
                    'h3'    => esc_html__( 'h3', 'selection' ),
                    'h4'    => esc_html__( 'h4', 'selection' ),
                    'h5'    => esc_html__( 'h5', 'selection' ),
                    'h6'    => esc_html__( 'h6', 'selection' ),
                    'div'    => esc_html__( 'div', 'selection' ),
                    'span'    => esc_html__( 'span', 'selection' ),
                    'p'    => esc_html__( 'p', 'selection' ),
                ],
                'condition' => [
                        'layout' => 'author'
                ],
                'default'   => 'div',
                'separator' => 'none',
            ]
        );

        $repeater->add_control(
            'company_tag',
            [
                'label'     => esc_html__( 'Tag', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'    => esc_html__( 'h1', 'selection' ),
                    'h2'    => esc_html__( 'h2', 'selection' ),
                    'h3'    => esc_html__( 'h3', 'selection' ),
                    'h4'    => esc_html__( 'h4', 'selection' ),
                    'h5'    => esc_html__( 'h5', 'selection' ),
                    'h6'    => esc_html__( 'h6', 'selection' ),
                    'div'    => esc_html__( 'div', 'selection' ),
                    'span'    => esc_html__( 'span', 'selection' ),
                    'p'    => esc_html__( 'p', 'selection' ),
                ],
                'condition' => [
                        'layout' => 'company'
                ],
                'default'   => 'div',
                'separator' => 'none',
            ]
        );


        $repeater->add_control(
            'author_separator',
            [
                'label' => esc_html__( 'Separator', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your separator here', 'selection' ),
                'condition' => [
                        'layout' => 'author'
                ]
            ]
        );

        $this->add_control(
            'layouts_list',
            [
                'label' => esc_html__( 'Repeater List', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'layout' => esc_html__( 'author_image' )
                    ],
                    [
                        'layout' => esc_html__( 'description' )
                    ],
                    [
                        'layout' => esc_html__( 'author' )
                    ],
                    [
                        'layout' => esc_html__( 'company' )
                    ],
                    [
                        'layout' => esc_html__( 'rating' )
                    ],
                ],
                'title_field' => 'Layout - {{{ layout }}}',
            ]
        );
        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Description Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_description_settings()
    {

        $this->start_controls_section('section_description_settings', [
            'label' => esc_html__('Description', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'testimonial_description',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'type' => Controls_Manager::WYSIWYG,
                'default' => esc_html__( 'Default description', 'selection' ),
                'placeholder' => esc_html__( 'Type your description here', 'selection' ),
            ]
        );

        $this->add_control(
            'description_text_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                        '{{WRAPPER}} .mdp-testimoner-elementor-description' => 'text-align: {{VALUE}}'
                ]
            ]
        );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Author Image Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_author_image_settings()
    {
        $this->start_controls_section('section_author_image_settings', [
            'label' => esc_html__('Author image', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'author_image',
            [
                'label' => esc_html__( 'Choose Image', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        $this->add_control(
            'custom_author_image_size',
            [
                'label' => esc_html__( 'Custom size', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'author_image_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 150,
                ],
                'condition' => [
                        'custom_author_image_size' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-testimoner-elementor-author-image img' => 'width: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_control(
            'author_image_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 150,
                ],
                'condition' => [
                    'custom_author_image_size' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-testimoner-elementor-author-image img' => 'height: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Image_Size::get_type(),
            [
                'name' => 'author_image',
                'exclude' => [ 'custom' ],
                'include' => [],
                'default' => 'medium',
            ]
        );

        $this->add_control(
            'author_image_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                        '{{WRAPPER}} .mdp-testimoner-elementor-author-image' => 'justify-content: {{VALUE}}'
                ]
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Author Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_author_settings()
    {
        $this->start_controls_section('section_author_settings', [
            'label' => esc_html__('Author', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'author_name',
            [
                'label' => esc_html__( 'Author', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Author', 'selection' ),
                'placeholder' => esc_html__( 'Type author name here', 'selection' ),
            ]
        );

        $this->add_control(
            'author_text_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                        '{{WRAPPER}} .mdp-testimoner-elementor-author' => 'text-align: {{VALUE}}'
                ],
            ]
        );

        $this->add_control(
            'author_name_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );

        $this->end_controls_section();
    }


    /**
     * Add widget controls: Content -> Company Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_company_settings()
    {
        $this->start_controls_section( 'section_company_settings', [
            'label' => esc_html__( 'Company', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'company_name',
            [
                'label' => esc_html__( 'Company', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Company', 'selection' ),
                'placeholder' => esc_html__( 'Type author name here', 'selection' ),
            ]
        );

        $this->add_control(
            'company_text_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                        '{{WRAPPER}} .mdp-testimoner-elementor-company' => 'text-align: {{VALUE}}'
                ]
            ]
        );

        $this->add_control(
            'company_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Rating Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_rating_settings()
    {
        $this->start_controls_section('section_rating_settings', [
            'label' => esc_html__('Rating', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'rating_count',
            [
                'label' => esc_html__( 'Rating count', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 5,
                'step' => 1,
            ]
        );

        $this->add_control(
            'rating_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                        '{{WRAPPER}} .mdp-testimoner-elementor-rating--stars' => 'justify-content: {{VALUE}}'
                ]
            ]
        );

        $this->add_control(
            'rating_icons',
            [
                'label' => esc_html__( 'Rating icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
            ]
        );


        $this->add_control(
            'enable_rich_snippet',
            [
                'label' => esc_html__( 'Enable Rich Snippet', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'json_schema_markup',
            [
                'label' => esc_html__( 'JSON+LD Markup', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'condition' => [
                        'enable_rich_snippet' => 'yes'
                ],
                'default' => wp_kses_post( '{ "@context": "https://schema.org/",
                  "@type": "Review",
                  "itemReviewed": {
                     "@type": "CreativeWorkSeries",
                     "name": "[title]"
                  },
                  "reviewRating": {
                     "@type": "Rating",
                     "ratingValue": "[value]",
                     "bestRating": "[best]",
                     "ratingCount": "[count]"
                  },
                  "author": {
                     "@type": "Person",
                     "name": "[author]"
                  },
                  "reviewBody": "[description]"
                }' ),
                'description' => wp_kses_post( '
                    <p>The following variables are available:</p>
                    <p><strong>[title]</strong> - Post Title.</p>
                    <p><strong>[value]</strong> - A numerical quality rating for the item.</p>
                    <p><strong>[author]</strong> - Name of testimonial author.</p>
                    <p><strong>[description]</strong> - Testimonial description.</p>
                    <p><strong>[best]</strong> - The highest value allowed in this rating system.</p>
                    <p><strong>[count]</strong> - The total number of ratings for the post on your site.</p>
                ' )
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            $section_id.'_margin_padding_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );
    }


    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $html_class
     * @param bool $include_color
     * @param bool $include_typography
     * @param bool $additional_color
     * @param string $additional_color_class
     * @param string $additional_color_name
     * @param bool $include_bg
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $html_class, $include_color = true, $include_typography = true, $additional_color = false, $additional_color_class = '', $additional_color_name = '', $include_bg = true ) {

        if ( $include_typography ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $additional_color ) {

            $this->add_control(
                $section_id.'_'.$additional_color_name. '_normal',
                [
                    'label' => esc_html__("$additional_color_name color", 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$additional_color_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_color ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__('Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );

            $this->add_control(
                $section_id . '_separate_normal',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );
        }




        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $additional_color ) {

            $this->add_control(
                $section_id.'_'.$additional_color_name.'_hover_color',
                [
                    'label' => esc_html__("$additional_color_name color", 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class:hover .$additional_color_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_color ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__('Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class:hover" => 'color: {{VALUE}}',
                    ],
                ]
            );
        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .$html_class:hover",
                ]
            );

            $this->add_control(
                $section_id.'_separate_hover',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );
        }


        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();
    }


    /**
     * Add widget controls: Style -> Section Style Description.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_description() {

        $this->start_controls_section( 'section_style_description', [
            'label' => esc_html__( 'Description', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_description', 'mdp-testimoner-elementor-description' );

        $this->generate_typography_tabs_controls( 'section_style_description', 'mdp-testimoner-elementor-description' );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Author.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_author()
    {

        $this->start_controls_section('section_style_author', [
            'label' => esc_html__('Author', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_author', 'mdp-testimoner-elementor-author' );

        $this->add_control(
            'author_container_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-testimoner-elementor-author' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_author', 'mdp-testimoner-elementor-author', true, true, true, 'mdp-testimoner-elementor-author-separator', 'Separator' );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Company.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_company()
    {

        $this->start_controls_section('section_style_company', [
            'label' => esc_html__('Company', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_company', 'mdp-testimoner-elementor-company' );

        $this->add_control(
            'company_container_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-testimoner-elementor-company' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_company', 'mdp-testimoner-elementor-company' );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Author Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_author_image()
    {

        $this->start_controls_section('section_style_author_image', [
            'label' => esc_html__('Author image', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_author_image', 'mdp-testimoner-elementor-author-image' );

        $this->generate_typography_tabs_controls( 'section_style_author_image', 'mdp-testimoner-elementor-author-image img', false, false, false, '', '', false );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Rating.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_rating()
    {

        $this->start_controls_section('section_style_rating', [
            'label' => esc_html__('Rating', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_rating', 'mdp-testimoner-elementor-rating' );

        $this->add_control(
            'rating_icons_size',
            [
                'label' => esc_html__( 'Rating icons size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-testimoner-elementor-rating--star i' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'space_between_icons',
            [
                'label' => esc_html__( 'Space between icons', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-testimoner-elementor-rating--star' => 'margin-right: calc({{SIZE}}{{UNIT}} / 2); margin-left: calc({{SIZE}}{{UNIT}} / 2);',
                ],
            ]
        );


        $this->start_controls_tabs( 'section_style_rating_style_tabs' );

        $this->start_controls_tab( 'section_style_rating_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );


            $this->add_control(
                'section_style_rating_active_star_color_normal',
                [
                    'label' => esc_html__("Active star color", 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'default' => 'rgba(252, 204, 0, 1)',
                    'selectors' => [
                        "{{WRAPPER}} .mdp-testimoner-elementor-rating--star.mdp-testimoner-elementor-rating--star-active" => 'color: {{VALUE}} !important',
                    ],
                ]
            );


            $this->add_control(
                 'section_style_rating_normal_text_color',
                [
                    'label' => esc_html__('Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'default' => 'rgba(214, 213, 207, 1)',
                    'selectors' => [
                        "{{WRAPPER}} .mdp-testimoner-elementor-rating--star" => 'color: {{VALUE}}',
                    ],
                ]
            );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' =>  'section_style_rating_normal_background',
                'label' => esc_html__('Background type', 'selection'),
                'types' => ['classic', 'gradient', 'video'],
                'selector' => "{{WRAPPER}} .mdp-testimoner-elementor-rating",
            ]
        );

        $this->add_control(
              'section_style_rating_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );




        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_rating_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-testimoner-elementor-rating",
            ]
        );

        $this->add_responsive_control(
            'section_style_rating_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-testimoner-elementor-rating" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_rating_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-testimoner-elementor-rating",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( 'section_style_rating_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );


            $this->add_control(
                'section_style_rating_active_star_hover_color',
                [
                    'label' => esc_html__("Active star color", 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'default' => 'rgba(252, 204, 0, 1)',
                    'selectors' => [
                        "{{WRAPPER}} .mdp-testimoner-elementor-rating--stars:hover .mdp-testimoner-elementor-rating--star.mdp-testimoner-elementor-rating--star-active" => 'color: {{VALUE}} !important',
                    ],
                ]
            );


            $this->add_control(
                'section_style_rating_hover_color',
                [
                    'label' => esc_html__('Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'default' => 'rgba(214, 213, 207, 1)',
                    'selectors' => [
                        "{{WRAPPER}} .mdp-testimoner-elementor-rating--stars:hover .mdp-testimoner-elementor-rating--star" => 'color: {{VALUE}}',
                    ],
                ]
            );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' =>  'section_style_rating_background_hover',
                'label' => esc_html__('Background type', 'selection'),
                'types' => ['classic', 'gradient', 'video'],
                'selector' => "{{WRAPPER}} .mdp-testimoner-elementor-rating:hover",
            ]
        );


        $this->add_control(
            'section_style_rating_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_rating_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-testimoner-elementor-rating:hover",
            ]
        );

        $this->add_responsive_control(
            'section_style_rating_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-testimoner-elementor-rating:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_rating_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-testimoner-elementor-rating:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        set_transient( 'plugin_id', $this->get_id() );
        ?>
        <!-- Start Testimoner for Elementor WordPress Plugin -->
        <div class="mdp-testimoner-elementor-box" data-rating="<?php esc_attr_e( $settings['rating_count'] ); ?>" >
            <?php foreach ( $settings[ 'layouts_list' ] as $item ): ?>
                <?php switch ( $item['layout'] ):
                case 'description': ?>
                    <div class="mdp-testimoner-elementor-description">
                        <?php  echo wp_kses_post( $settings['testimonial_description'] ) ?>
                    </div>
                <?php break; ?>
                <?php case 'author_image': ?>
                    <div class="mdp-testimoner-elementor-author-image">
                        <?php if( $settings['custom_author_image_size'] === 'yes' ): ?>
                            <img src="<?php echo esc_url( $settings['author_image']['url'] )?>" alt="">
                        <?php else: ?>
                        <?php echo wp_kses_post( Group_Control_Image_Size::get_attachment_image_html( $settings, 'author_image' ) ); ?>
                        <?php endif; ?>
                    </div>
                <?php break; ?>
                <?php case 'author': ?>
                    <?php if ( !empty( $settings['author_name_link']['url'] ) ): ?>
                        <a href="<?php echo esc_url( $settings['author_name_link']['url'] ) ?>" >
                            <?php echo wp_kses_post( '<'.$item['author_tag'].' class="mdp-testimoner-elementor-author">'
                                .$settings['author_name'].' <span class="mdp-testimoner-elementor-author-separator">'.$item['author_separator'].
                            '</span> </'.$item['author_tag'].'>' ) ?>
                        </a>
                    <?php else: ?>
                            <?php echo wp_kses_post( '<'.$item['author_tag'].' class="mdp-testimoner-elementor-author">'
                                .$settings['author_name'].' <span class="mdp-testimoner-elementor-author-separator">'.$item['author_separator'].
                                '</span> </'.$item['author_tag'].'>' ) ?>
                    <?php endif; ?>
                <?php break; ?>
                <?php case 'company': ?>
                    <?php if ( !empty( $settings['company_link']['url'] ) ): ?>
                        <a href="<?php echo esc_url( $settings['company_link']['url'] ) ?>">
                            <?php echo wp_kses_post( '<'.$item['company_tag'].' class="mdp-testimoner-elementor-company">'
                                .$settings['company_name'].
                                '</'.$item['company_tag'].'>' ) ?>
                        </a>
                    <?php else: ?>
                            <?php echo wp_kses_post( '<'.$item['company_tag'].' class="mdp-testimoner-elementor-company">'
                                .$settings['company_name'].
                                '</'.$item['company_tag'].'>' ) ?>
                    <?php endif; ?>
                    <?php break; ?>
                    <?php case 'rating': ?>
                        <div class="mdp-testimoner-elementor-rating">
                            <ul class="mdp-testimoner-elementor-rating--stars">
                                <li class="mdp-testimoner-elementor-rating--star"><?php Icons_Manager::render_icon( $settings['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                <li class="mdp-testimoner-elementor-rating--star"><?php Icons_Manager::render_icon( $settings['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                <li class="mdp-testimoner-elementor-rating--star"><?php Icons_Manager::render_icon( $settings['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                <li class="mdp-testimoner-elementor-rating--star"><?php Icons_Manager::render_icon( $settings['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                <li class="mdp-testimoner-elementor-rating--star"><?php Icons_Manager::render_icon( $settings['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                            </ul>
                        </div>
                    <?php break; ?>
                <?php endswitch; ?>
            <?php endforeach; ?>

        </div>
        <!-- End Testimoner for Elementor WordPress Plugin -->
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>

            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    testimonerReady<?php esc_attr_e( $widget_hash ); ?>( mdpTestimoner.addTestimoner.bind( mdpTestimoner) );
                } catch ( msg ) {
                    const testimonerReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    testimonerReady<?php esc_attr_e( $widget_hash ); ?>( mdpTestimoner.addTestimoner.bind( mdpTestimoner ) );
                }
            </script>
            <?php
        }

    }


    /**
     * JS Render.
     *
     * @access protected
     *
     * @return void
     **/
    protected function content_template() {
        ?>
        <#
        var image = {
            id: settings.author_image.id,
            url: settings.author_image.url,
            size: settings.author_image_size,
            dimension: settings.author_image_custom_dimension,
            model: view.getEditModel()
        };

        var image_url = elementor.imagesManager.getImageUrl( image );
        #>
        <div class="mdp-testimoner-elementor-box" data-rating="{{{ settings.rating_count }}}" >
            <# _.each( settings.layouts_list, function( item, index ) { #>
                <# if ( item.layout === 'description' ) { #>
                    <div class="mdp-testimoner-elementor-description">
                        {{{ settings.testimonial_description }}}
                    </div>
                <# } #>
                <# if ( item.layout === 'author_image' ) { #>
                    <div class="mdp-testimoner-elementor-author-image">
                        <div class="mdp-testimoner-elementor-author-image">
                            <# if ( settings.custom_author_image_size === 'yes' ) { #>
                            <img src="{{{ settings.author_image.url }}}" alt="">
                            <# } else { #>
                            <img src="{{{ image_url }}}" />
                            <# } #>
                        </div>
                    </div>
                <# } #>
                <# if ( item.layout === 'author' ) { #>
                    <# if ( settings.author_name_link !== '' ) { #>
                        <a href="{{ settings.author_name_link.url }}">
                            <{{{ item.author_tag }}} class="mdp-testimoner-elementor-author">
                                {{{ settings.author_name }}} <span class="mdp-testimoner-elementor-author-separator">{{{ item.author_separator }}}</span>
                            </{{{ item.author_tag }}}>
                        </a>
                    <# } else if( settings.author_name_link === '' ) { #>
                        <{{{ item.author_tag }}} class="mdp-testimoner-elementor-author">
                            {{{ settings.author_name }}} <span class="mdp-testimoner-elementor-author-separator">{{{ item.author_separator }}}</span>
                        <{{{ '/' }}}{{{ item.author_tag }}}>
                    <# } #>
                <# } #>
                <# if ( item.layout === 'company' ) { #>
                    <# if ( settings.company_link !== '' ) { #>
                        <a href="{{ settings.company_link.url }}">
                            <{{{ item.company_tag }}} class="mdp-testimoner-elementor-company">
                                {{{ settings.company_name }}}
                            <{{'/'}}{{{ item.company_tag }}}>
                        </a>
                    <# } else if( settings.company_link === '' ) { #>
                        <{{{ item.company_tag }}} class="mdp-testimoner-elementor-company">
                            {{{ settings.company_name }}}
                        <{{{ '/' }}}{{{ item.company_tag }}}>
                    <# } #>
                <# } #>
                <# if ( item.layout === 'rating' ) { #>
                    <div class="mdp-testimoner-elementor-rating">
                        <ul class="mdp-testimoner-elementor-rating--stars">
                            <li class="mdp-testimoner-elementor-rating--star"><i class="{{{ settings.rating_icons.value }}}"></i></li>
                            <li class="mdp-testimoner-elementor-rating--star"><i class="{{{ settings.rating_icons.value }}}"></i></li>
                            <li class="mdp-testimoner-elementor-rating--star"><i class="{{{ settings.rating_icons.value }}}"></i></li>
                            <li class="mdp-testimoner-elementor-rating--star"><i class="{{{ settings.rating_icons.value }}}"></i></li>
                            <li class="mdp-testimoner-elementor-rating--star"><i class="{{{ settings.rating_icons.value }}}"></i></li>
                        </ul>
                    </div>
                <# } #>
            <# } )  #>
        </div>
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>

            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    testimonerReady<?php esc_attr_e( $widget_hash ); ?>( mdpTestimoner.addTestimoner.bind( mdpTestimoner) );
                } catch ( msg ) {
                    const testimonerReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    testimonerReady<?php esc_attr_e( $widget_hash ); ?>( mdpTestimoner.addTestimoner.bind( mdpTestimoner ) );
                }
            </script>
            <?php
        }
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/testimoner';

    }

}
