<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Walker for Elementor
 * Google Street View for Elementor Builder
 * Exclusively on https://1.envato.market/walker-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Background;
use Elementor\Core\Schemes\Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Text_Shadow;
use Merkulove\Selection\Unity\Settings;
use Elementor\Plugin;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Walker - Custom Elementor Widget.
 **/
class walker_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {
        $options = Settings::get_instance()->options;

        parent::__construct( $data, $args );

        wp_register_style(
          'mdp-selection-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );
        wp_register_style(
         'mdp-walker-elementor',
            UnityPlugin::get_url() . 'css/walker-elementor' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );
	    wp_register_script(
        'mdp-walker-elementor',
           UnityPlugin::get_url() . 'js/walker-elementor' . UnityPlugin::get_suffix() . '.js',
               [ 'jquery' ],
               UnityPlugin::get_version(),
      true
        );
        wp_register_script(
       'google-maps-api',
          'https://maps.googleapis.com/maps/api/js?key='.$options[ 'google_maps_api_key' ]
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-walker-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Google Street View', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-walker-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Google Street View', 'map', 'Google', 'street', 'view', 'panorama', 'embed', 'photo', 'image' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-walker-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-walker-elementor', 'google-maps-api' ];

    }

    /**
     * Street View Sources Tabs
     *
     * @since 1.0.0
     * @access public
     */
    public function content_streetview_sources() {

        # Panorama Source
        $this->add_control(
            'source',
            [
                'label' => __( 'Street View Source', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'source_latlng' => [
                        'title' => __( 'Coordinates', 'selection' ),
                        'icon' => 'fa fa-map-pin',
                    ],
                    'source_id' => [
                        'title' => __( 'Panorama ID', 'selection' ),
                        'icon' => 'fa fa-street-view',
                    ],
                ],
                'default' => 'source_latlng',
                'toggle' => true,
            ]
        );

        # Latitude
        $this->add_control(
            'lat',
            [
                'label' => esc_html__( 'Latitude', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( '-77.5542751', 'selection' ),
                'default' => '-77.5542751',
                'condition' => [ 'source' => 'source_latlng' ],
                'dynamic' => [
                    'active' => true,
                ],
            ]
        );

        # Longitude
        $this->add_control(
            'lng',
            [
                'label' => esc_html__( 'Longitude', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( '166.1616476', 'selection' ),
                'default' => '166.1616476',
                'condition' => [ 'source' => 'source_latlng' ],
                'dynamic' => [
                    'active' => true,
                ],
            ]
        );

        # Panorama ID
        $this->add_control(
            'panorama_id',
            [
                'label' => esc_html__( 'Panorama ID', 'selection' ),
                'description' => esc_html__( 'Use ', 'selection' ) .
                    '<a href="https://istreetview.com/" target="_blank" rel="noopener">' . esc_html__( 'istreetview.com', 'selection' ) . '</a>' .
                    esc_html__( ' to get a panorama ID.', 'selection' ),
                'label_block' => true,
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'CAoSLEFGMVFpcFBUS2NLUGZZbTZKLTUtQkNYNUFzRkRwUkpqaUtIay1oMzM1SWtD', 'selection' ),
                'default' => 'CAoSLEFGMVFpcFBUS2NLUGZZbTZKLTUtQkNYNUFzRkRwUkpqaUtIay1oMzM1SWtD',
                'condition' => [ 'source' => 'source_id' ]
            ]
        );

        # Panorama description
        $this->add_control(
            'description',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Description', 'selection' ),
                'condition' =>
                    [
                        'source' => 'source_id',
                        'address_control' => 'yes',
                    ]
            ]
        );

        # Copyrights
        $this->add_control(
            'copyrights',
            [
                'label' => esc_html__( 'Custom copyrights', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Copyrights', 'selection' ),
                'condition' =>
                    [
                        'source' => 'source_id',
                        'google_links' => 'yes',
                    ]
            ]
        );

    }

    /**
     * Content for Street View
     *
     * @since 1.0.0
     * @access public
     */
    private function content_streetview() {

        # Start section
        $this->start_controls_section(
            'content_streetview',
            [
                'label' => esc_html__( 'Street View', 'selection' ),
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        # Street View Sources
        $this->content_streetview_sources();

        # Heading
        $this->add_control(
            'heading',
            [
                'label' => esc_html__( 'Horizontal angle', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'deg' ],
                'range' => [
                    'deg' => [
                        'min' => 0,
                        'max' => 360,
                        'step' => .5
                    ],
                ],
                'default' => [
                    'unit' => 'deg',
                    'size' => 0,
                ],
                'separator' => 'before'
            ]
        );

        # Pitch
        $this->add_control(
            'pitch',
            [
                'label' => esc_html__( 'Vertical angle', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'deg' ],
                'range' => [
                    'deg' => [
                        'min' => -90,
                        'max' => 90,
                        'step' => .5
                    ],
                ],
                'default' => [
                    'unit' => 'deg',
                    'size' => 0,
                ]
            ]
        );

        # Zoom
        $this->add_control(
            'zoom',
            [
                'label' => esc_html__( 'Zoom', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'deg' ],
                'range' => [
                    'deg' => [
                        'min' => 0,
                        'max' => 4,
                        'step' => .5
                    ],
                ],
                'default' => [
                    'unit' => 'deg',
                    'size' => 0,
                ],
                'separator' => 'after'
            ]
        );

        # Compass
        $this->add_control(
            'pan_control',
            [
                'label' => esc_html__( 'Compass', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Zoom
        $this->add_control(
            'zoom_control',
            [
                'label' => esc_html__( 'Zoom', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Address
        $this->add_control(
            'address_control',
            [
                'label' => esc_html__( 'Address', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Fullscreen
        $this->add_control(
            'fullscreen_control',
            [
                'label' => esc_html__( 'Fullscreen', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Links
        $this->add_control(
            'links_control',
            [
                'label' => esc_html__( 'Links', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Motion tracking control
        $this->add_control(
            'motion_tracking_control',
            [
                'label' => esc_html__( 'Motion Tracking Control', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Motion tracking
        $this->add_control(
            'motion_tracking',
            [
                'label' => esc_html__( 'Motion Tracking', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'condition' => [ 'motion_tracking_control' => 'yes' ]
            ]
        );

        # Google Logo
        $this->add_control(
            'google_logo',
            [
                'label' => esc_html__( 'Google Logo', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-panorama .gm-style > div:nth-child(2)' => 'display: block;',
                ],
            ]
        );

        # Google Links
        $this->add_control(
            'google_links',
            [
                'label' => esc_html__( 'Google Links', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-panorama .gm-style > div:nth-child(4)' => 'display: block;',
                    '{{WRAPPER}} .mdp-walker .mdp-panorama .gm-style > div:nth-child(6)' => 'display: block;',
                    '{{WRAPPER}} .mdp-walker .mdp-panorama .gm-style > div:last-child' => 'display: block;',
                ],
            ]
        );

        # Overlay
        $this->add_control(
            'overlay',
            [
                'label' => esc_html__( 'Overlay', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Description
        $this->add_control(
            'caption',
            [
                'label' => esc_html__( 'Caption', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Panorama rotation
        $this->add_control(
            'rotation',
            [
                'label' => esc_html__( 'Rotation', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Stop after click
        $this->add_control(
            'click',
            [
                'label' => esc_html__( 'Stop after click', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'condition' => [ 'rotation' => 'yes' ]
            ]
        );

        # Rotation loop
        $this->add_control(
            'loop',
            [
                'label' => esc_html__( 'Loop Rotation', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'condition' => [ 'rotation' => 'yes' ]
            ]
        );

        # Rotation Count
        $this->add_control(
            'rotation_count',
            [
                'label' => esc_html__( 'Rotation Count', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 10,
                'step' => .5,
                'default' => 1,
                'condition' => [
                    'loop!' => 'yes',
                    'rotation' => 'yes'
                ]
            ]
        );

        # Rotation Direction
        $this->add_control(
            'rotation_direction',
            [
                'label' => esc_html__( 'Direction of Rotation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    '1'      => esc_html__( 'Сlockwise', 'selection' ),
                    '-1'     => esc_html__( 'Сounterclock-wise', 'selection' ),
                ],
                'default' => '1',
                'condition' => [ 'rotation' => 'yes' ]
            ]
        );

        # Rotation Speed
        $this->add_control(
            'rotation_speed',
            [
                'label' => esc_html__( 'Rotation Speed', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 50,
                        'step' => 1
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 25,
                ],
                'condition' => [ 'rotation' => 'yes' ]
            ]
        );

        # Street View Side-By-Side
        $this->add_control(
            'side_by_side',
            [
                'label' => esc_html__( 'Side-By-Side View', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'condition' => [ 'source' => 'source_latlng' ],
                'separator' => 'before'
            ]
        );

        # End section
        $this->end_controls_section();

    }

    /**
     * Content for Map
     *
     * @since 1.0.0
     * @access public
     */
    private function content_map() {

        # Start section
        $this->start_controls_section(
            'content_map',
            [
                'label' => esc_html__( 'Map', 'selection' ),
                'tab'   => Controls_Manager::TAB_CONTENT,
                'condition' =>
                    [
                        'side_by_side' => 'yes',
                        'source' => 'source_latlng'
                    ]
            ]
        );

        # Side-By-Side map position
        $this->add_responsive_control(
            'side_by_side_position',
            [
                'label' => esc_html__( 'Map Position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'left'      => esc_html__( 'Left', 'selection' ),
                    'right'     => esc_html__( 'Right', 'selection' ),
                    'top'       => esc_html__( 'Above', 'selection' ),
                    'bottom'    => esc_html__( 'Below', 'selection' ),
                ],
                'default' => 'left',
            ]
        );

        # Zoom
        $this->add_control(
            'map_zoom',
            [
                'label' => __( 'Initial zoom', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 25,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 15,
                ],
            ]
        );

        # Fullscreen
        $this->add_control(
            'google_map_fullscreen',
            [
                'label' => esc_html__( 'Fullscreen', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-fullscreen-control' => 'display: block;',
                ],
            ]
        );

        # Layers Switcher
        $this->add_control(
            'google_map_switcher',
            [
                'label' => esc_html__( 'Layers Switcher', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:nth-child(11)' => 'display: block;',
                ],
            ]
        );

        # Zoom controls
        $this->add_control(
            'google_map_zoom',
            [
                'label' => esc_html__( 'Zoom', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > .gm-bundled-control.gm-bundled-control-on-bottom' => 'display: block;',
                ],
            ]
        );

        # Google Logo
        $this->add_control(
            'google_map_logo',
            [
                'label' => esc_html__( 'Google Logo', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:nth-child(3)' => 'display: block;',
                ],
            ]
        );

        # Google Links
        $this->add_control(
            'google_map_links',
            [
                'label' => esc_html__( 'Google Links', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:nth-child(5)' => 'display: block;',
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:nth-child(6)' => 'display: block;',
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:nth-child(7)' => 'display: block;',
                    '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:nth-child(9)' => 'display: block;'
                ],
            ]
        );

        # End section
        $this->end_controls_section();

    }

    /**
     * Content for Caption
     *
     * @since 1.0.0
     * @access private
     */
    private function caption_content() {

        # Start section
        $this->start_controls_section(
            'caption_content',
            [
                'label' => esc_html__( 'Caption', 'selection' ),
                'tab' => Controls_Manager::TAB_CONTENT,
                'condition' => [ 'caption' => 'yes' ]
            ]
        );

        # Show header
        $this->add_control(
            'show_header',
            [
                'label' => esc_html__( 'Header', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        # Header Alignment
        $this->add_responsive_control(
            'header_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-caption > *:first-child' => 'align-items: {{header_align}};',
                ],
                'condition' => [ 'caption' => 'yes', 'show_header' => 'yes' ],
                'toggle' => true,
            ]
        );

        # Header
        $this->add_control(
            'header',
            [
                'label' => esc_html__( 'Header', 'selection' ),
                'show_label' => false,
                'type' => Controls_Manager::TEXTAREA,
                'dynamic' => ['active' => true],
                'rows' => 2,
                'default' => esc_html__( 'Header', 'selection' ),
                'placeholder' => esc_html__( 'Header', 'selection' ),
                'condition' => [ 'show_header' => 'yes' ]
            ]
        );

        # Header HTML Tag
        $this->add_control(
            'header_tag',
            [
                'label' => esc_html__( 'HTML Tag', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h1' => 'H1',
                    'h2' => 'H2',
                    'h3' => 'H3',
                    'h4' => 'H4',
                    'h5' => 'H5',
                    'h6' => 'H6',
                    'div' => 'div',
                    'span' => 'span',
                    'p' => 'p',
                ],
                'default' => 'h3',
                'separator' => 'after',
                'condition' => [ 'show_header' => 'yes' ]
            ]
        );

        # Show Subheader
        $this->add_control(
            'show_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'condition' => [ 'show_header' => 'yes' ],
            ]
        );

        # Subheader
        $this->add_control(
            'walker_sub_name',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'show_label' => false,
                'type' => Controls_Manager::TEXTAREA,
                'dynamic' => ['active' => true],
                'rows' => 2,
                'default' => esc_html__( 'Subheader', 'selection' ),
                'placeholder' => esc_html__( 'Subheader', 'selection' ),
                'condition'   => [ 'show_subheader' => 'yes', 'show_header' => 'yes' ],
            ]
        );

        # Subheader Position
        $this->add_control(
            'subheader_position',
            [
                'label' => esc_html__( 'Subheader Position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'top' => 'Above Header',
                    'bottom' => 'Under Header',
                ],
                'default' => 'bottom',
                'separator' => 'after',
                'condition'   => [ 'show_subheader' => 'yes', 'show_header' => 'yes' ]
            ]
        );

        # Show description
        $this->add_control(
            'show_description',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => '',
            ]
        );

        # Description Alignment
        $this->add_responsive_control(
            'description_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-description' => 'text-align: {{header_align}};',
                ],
                'condition' => [ 'caption' => 'yes', 'show_description' => 'yes' ],
                'toggle' => true,
            ]
        );

        # Description
        $this->add_control(
            'description_content',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'show_label' => false,
                'type' => Controls_Manager::TEXTAREA,
                'dynamic' => ['active' => true],
                'rows' => 5,
                'default' => esc_html__( 'Description', 'selection' ),
                'placeholder' => esc_html__( 'Description', 'selection' ),
                'condition' => [ 'show_description' => 'yes' ],
            ]
        );

        # Description HTML Tag
        $this->add_control(
            'description_tag',
            [
                'label' => esc_html__( 'HTML Tag', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'div' => 'div',
                    'span' => 'span',
                    'p' => 'p',
                ],
                'default' => 'p',
                'condition' => [ 'show_description' => 'yes' ],
            ]
        );

        # Justify Caption Content
        $this->add_responsive_control(
            'description_justify',
            [
                'label' => esc_html__( 'Justify Content', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'flex-start' => 'Start',
                    'center' => 'Center',
                    'flex-end' => 'End',
                    'space-around' => 'Space Around',
                    'space-between' => 'Space Between',
                    'space-evenly' => 'Space Evenly',
                ],
                'default' => 'flex-start',
                'selectors' => [
                    '{{WRAPPER}} .mdp-caption' => 'justify-content: {{value}}',
                ],
                'separator' => 'before'
            ]
        );

        /** End section */
        $this->end_controls_section();

    }

    /**
     * Style controls for Street View
     *
     * @since 1.0.0
     * @access private
     */
    private function style_streetview() {

        # Start widget style section
        $this->start_controls_section(
            'streetview_style',
            [
                'label' => esc_html__( 'Street View', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        # Margin
        $this->add_control(
            'streetview_margin',
            [
                'label' => __( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-panorama' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'separator' => 'after'
            ]
        );

        # Street View Height
        $this->add_responsive_control(
            'streetview_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type'  => Controls_Manager::SLIDER,
                'size_units' => [ 'px', 'vh' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 500,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-panorama' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        # Street View Width
        $this->add_responsive_control(
            'streetview_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type'  => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px', 'vh' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-panorama' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'separator' => 'after'
            ]
        );

        # Street View Align
        $this->add_control(
            'streetview_align',
            [
                'label' => __( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => __( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => __( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => __( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'space-between' => [
                        'title' => __( 'Space between', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker' => 'justify-content: {{value}};',
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        # CSS Filter
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name' => 'streetview_css_filters',
                'selector' => '{{WRAPPER}} .mdp-walker .mdp-panorama .gm-style > div:first-child',
            ]
        );

        # Street View Border
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'      => 'streetview_border',
                'selector'  => '{{WRAPPER}} .mdp-walker .mdp-panorama',
                'separator' => 'before',
            ]
        );

        # Street View Radius
        $this->add_control(
            'streetview_radius',
            [
                'label'         => esc_html__( 'Border Radius', 'selection' ),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .mdp-walker .mdp-panorama' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'separator' => 'after',
            ]
        );

        # Street View Shadow
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'streetview_shadow',
                'selector' => '{{WRAPPER}} .mdp-walker .mdp-panorama',
            ]
        );

        # End style
        $this->end_controls_section();

    }

    /**
     * Style controls for Map
     *
     * @since 1.0.0
     * @access private
     */
    private function style_map() {

        # Start widget style section
        $this->start_controls_section(
            'map_style',
            [
                'label' => esc_html__( 'Map', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [ 'side_by_side' => 'yes' ]
            ]
        );

        # Map Margin
        $this->add_control(
            'map_margin',
            [
                'label' => __( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'separator' => 'after'
            ]
        );

        # Map Height
        $this->add_responsive_control(
            'map_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type'  => Controls_Manager::SLIDER,
                'size_units' => [ 'px', 'vh' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 500,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        # Map Width
        $this->add_responsive_control(
            'map_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type'  => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px', 'vh' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker .mdp-map' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'separator' => 'after'
            ]
        );

        # CSS Filter
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name' => 'map_css_filters',
                'selector' => '{{WRAPPER}} .mdp-walker .mdp-map .gm-style > div:first-child',
            ]
        );

        # Map Border
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'      => 'map_border',
                'selector'  => '{{WRAPPER}} .mdp-walker .mdp-map',
                'separator' => 'before',
            ]
        );

        # Map Border Radius
        $this->add_control(
            'map_radius',
            [
                'label'         => esc_html__( 'Border Radius', 'selection' ),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .mdp-walker .mdp-map' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'separator' => 'after',
            ]
        );

        # Map Box Shadow
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'map_shadow',
                'selector' => '{{WRAPPER}} .mdp-walker .mdp-map',
            ]
        );

        # End style
        $this->end_controls_section();

    }

    /**
     * Style controls for Overlay
     *
     * @since 1.0.0
     * @access private
     */
    private function style_overlay() {

        # Start widget style section
        $this->start_controls_section(
            'overlay_style',
            [
                'label' => esc_html__( 'Overlay', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [ 'overlay' => 'yes' ]
            ]
        );

        # Overlay Background
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'overlay_background',
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-overlay',
            ]
        );

        # Overlay Blend Mode
        $this->add_control(
            'overlay_blend',
            [
                'label' => esc_html__( 'Blend Mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'none' =>esc_html__( 'None', 'selection' ),
                    'multiply' => esc_html__( 'Multiply', 'selection' ),
                    'screen' => esc_html__( 'Screen', 'selection' ),
                    'overlay' => esc_html__( 'Overlay', 'selection' ),
                    'darken' => esc_html__( 'Darken', 'selection' ),
                    'lighten' => esc_html__( 'Lighten', 'selection' ),
                    'color-dodge' => esc_html__( 'Color-dodge', 'selection' ),
                    'color-burn' => esc_html__( 'Color-burn', 'selection' ),
                    'hard-light' => esc_html__( 'Hard-light', 'selection' ),
                    'soft-light' => esc_html__( 'Soft-light', 'selection' ),
                    'difference' => esc_html__( 'Difference', 'selection' ),
                    'exclusion' => esc_html__( 'Exclusion', 'selection' ),
                    'hue' => esc_html__( 'Hue', 'selection' ),
                    'saturation' => esc_html__( 'Saturation', 'selection' ),
                    'color' => esc_html__( 'Color', 'selection' ),
                    'luminosity' => esc_html__( 'Luminosity', 'selection' ),
                ],
                'selectors'     => [
                    '{{WRAPPER}} .mdp-walker .mdp-overlay' => 'mix-blend-mode: {{value}};',
                ],
            ]
        );

        # Overlay Border Radius
        $this->add_control(
            'overlay_radius',
            [
                'label'         => esc_html__( 'Border Radius', 'selection' ),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .mdp-walker .mdp-overlay' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'separator' => 'after',
            ]
        );

        # End style
        $this->end_controls_section();

    }

    /**
     * Style controls for Header
     *
     * @since 1.0.0
     * @access private
     */
    private function style_header() {

        # Start section
        $this->start_controls_section( 'style_header',
            [
                'label' => esc_html__( 'Header', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [ 'caption' => 'yes', 'show_header' => 'yes' ],
            ]
        );

        # Header Margin
        $this->add_responsive_control(
            'margin_header',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-header' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Header Padding
        $this->add_responsive_control(
            'padding_header',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-header' => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Header Color
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_control(
            'color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-header' => 'color: {{VALUE}}',
                ],
            ]
        );

        # Header Typography
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'header_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' => Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .mdp-walker-header',
            ]
        );

        # Header Shadow
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'header_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-walker-header',
            ]
        );

        # Header Background
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'header_background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-walker-header',
            ]
        );

        # Subheader
        $this->add_control(
            'text_animation_header',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition'   => ['show_subheader' => 'yes']
            ]
        );

        # Subheader Margin
        $this->add_responsive_control(
            'margin_subheader',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-subheader' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
                'condition'   => ['show_subheader' => 'yes']
            ]
        );

        # Subheader Padding
        $this->add_responsive_control(
            'padding_subheader',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-subheader' => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
                'condition'   => ['show_subheader' => 'yes']
            ]
        );

        # Subheader Color
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_control(
            'color_subheader',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-subheader' => 'color: {{VALUE}}',
                ],
                'condition'   => ['show_subheader' => 'yes']
            ]
        );

        # Subheader Typography
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'subheader_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' => Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .mdp-walker-subheader',
                'condition'   => ['show_subheader' => 'yes']
            ]
        );

        # Subheader Shadow
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'subheader_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-walker-subheader',
                'condition'   => ['show_subheader' => 'yes']
            ]
        );

        # End header style
        $this->end_controls_section();

    }

    /**
     * Style for Description
     *
     * @since 1.0.0
     * @access private
     */
    private function style_description() {

        # Section Start
        $this->start_controls_section( 'style_description',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [ 'caption' => 'yes', 'show_description' => 'yes' ],
            ]
        );

        # Description Margin
        $this->add_responsive_control(
            'margin_description',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-description' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Description Padding
        $this->add_responsive_control(
            'padding_description',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-description' => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Description Color
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_control(
            'color_description',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-walker-description' => 'color: {{VALUE}}',
                ],
            ]
        );

        # Description Typography
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'description_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' => Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .mdp-walker-description',
            ]
        );

        # Description Shadow
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'description_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-walker-description',
            ]
        );

        # Description Background
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'description_background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-walker-description',
            ]
        );

        # Section end
        $this->end_controls_section();

    }

    /**
     * Add widget controls on the Content tab.
     *
     * @since 1.0.0
     * @access private
     **/
    private function tab_content() {

        /** Content -> Street View */
        $this->content_streetview();

        /** Content -> Map */
        $this->content_map();

        /** Content -> Caption */
        $this->caption_content();

    }

    /**
     * Add widget controls on the Style tab.
     *
     * @since 1.0.0
     * @access private
     **/
    private function tab_style() {

        /** Style -> Street View */
        $this->style_streetview();

        /** Style -> Map */
        $this->style_map();

        /** Style -> Overlay */
        $this->style_overlay();

        /** Style -> Header */
        $this->style_header();

        /** Style -> Description */
        $this->style_description();

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /** Controls of the Content Tab */
        $this->tab_content();

        /** Controls of the Style Tab */
        $this->tab_style();

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** Render API key notice if api key length is to short */
        if ( strlen( Settings::get_instance()->options[ 'google_maps_api_key' ] ) < 24 ) {
            ?>
            <p class="mdp-walker-notice">
                <?php esc_html_e( 'Please specify the Google Maps ', 'selection' )?>
                <a href="<?php echo get_site_url() . esc_attr( '/wp-admin/admin.php?page=mdp_walker_elementor_settings' ); ?>"
                   target="_blank"
                   rel="noopener">
                    <?php esc_html_e( 'API Key', 'selection' )?>
                </a>
            </p>
            <?php
            return;
        }

        /** @var array of the widget settings $settings */
        $settings = $this->get_settings_for_display();

        /** @var string widget unique id $walker_id */
        $walker_id = $this->get_id();

        /** Walker render attributes */
        $this->render_attributes();
        ?>

        <!-- Walker for Elementor: Begin -->
        <div id="mdp-walker-<?php esc_attr_e($walker_id); ?>" <?php echo $this->get_render_attribute_string( 'mdp-walker' ); ?> >

            <?php if ( $settings[ 'side_by_side' ] === 'yes' ) { ?>
                <div id="map-<?php esc_attr_e( $walker_id ); ?>" class="mdp-map"></div>
            <?php } ?>

            <div id="panorama-<?php esc_attr_e( $walker_id ); ?>" class="mdp-panorama"></div>

            <?php if ( $settings[ 'overlay' ] === 'yes' ) { ?>
                <div class="mdp-overlay"></div>
            <?php } ?>

            <?php if ( $settings[ 'caption' ] === 'yes' ) { ?>
                <div class="mdp-caption">
                    <?php
                    $this->render_header( $settings );
                    $this->render_description( $settings );
                    ?>
                </div>
            <?php } ?>

        </div>
        <!-- Walker for Elementor: End -->

        <?php
        /** Run scripts in the Editor on widget render and re-render */
        if ( Plugin::$instance->editor->is_edit_mode() || Plugin::$instance->preview->is_preview_mode() ) { ?>

            <script>
                mdpwalkerElementor( 'mdp-walker-<?php esc_attr_e($walker_id); ?>' );
            </script>

            <?php
        }
    }

    /**
     * Add render attributes for rendering widget section
     *
     * @since 1.0.0
     * @access public
     */
    private function render_attributes() {

        /** @var array of the widget settings $settings */
        $settings = $this->get_settings_for_display();

        /** Add render attributes for the Walker wrapper */
        $this->add_render_attribute( 'mdp-walker', 'class', 'mdp-walker' );

        /** Add render attributes by type of Street View source */
        if ( $settings[ 'source' ] === 'source_latlng' ) {
            $this->add_render_attribute( 'mdp-walker', 'data-lat', esc_attr( $settings[ 'lat' ] ) );
            $this->add_render_attribute( 'mdp-walker', 'data-lng', esc_attr( $settings[ 'lng' ] ) );
        } else {
            $this->add_render_attribute( 'mdp-walker', 'data-pano', esc_attr( $settings[ 'panorama_id' ] ) );
            $this->add_render_attribute( 'mdp-walker', 'data-description', esc_attr( $settings[ 'description' ] ) );
            $this->add_render_attribute( 'mdp-walker', 'data-copyrights', esc_attr( $settings[ 'copyrights' ] ) );
        }

        /** Add render attributes for Motion Tracking Controls */
        if ( $settings[ 'motion_tracking_control' ] === '' ) {
            $this->add_render_attribute( 'mdp-walker', 'data-motion-tracking', 'false' );
            $this->add_render_attribute( 'mdp-walker', 'data-motion-tracking-control', 'false' );
        } else {
            $this->add_render_attribute( 'mdp-walker', 'data-motion-tracking', 'true' );
            $this->add_render_attribute( 'mdp-walker', 'data-motion-tracking-control', ( $settings[ 'motion_tracking' ] === '' ) ? 'false' : 'true' );
        }

        /** Add render attributes for the panorama view angle and zoom */
        $this->add_render_attribute( 'mdp-walker', 'data-heading', esc_attr( $settings[ 'heading' ][ 'size' ] ) );
        $this->add_render_attribute( 'mdp-walker', 'data-pitch', esc_attr( $settings[ 'pitch' ][ 'size' ] ) );
        $this->add_render_attribute( 'mdp-walker', 'data-zoom', esc_attr( $settings[ 'zoom' ][ 'size' ] ) );

        /** Add render attributes for Street View UI */
        $this->add_render_attribute( 'mdp-walker', 'data-visible', 'true' );
        $this->add_render_attribute( 'mdp-walker', 'data-pan-control', esc_attr( ( $settings[ 'pan_control' ] === '' ) ? 'false' : 'true' ) );
        $this->add_render_attribute( 'mdp-walker', 'data-zoom-control', esc_attr( ( $settings[ 'zoom_control' ] === '' ) ? 'false' : 'true' ) );
        $this->add_render_attribute( 'mdp-walker', 'data-address-control', ( $settings[ 'address_control' ] === '' ) ? 'false' : 'true' );
        $this->add_render_attribute( 'mdp-walker', 'data-fullscreen-control', esc_attr( ( $settings[ 'fullscreen_control' ] === '' ) ? 'false' : 'true' ) );
        $this->add_render_attribute( 'mdp-walker', 'data-links-control', esc_attr( ( $settings[ 'links_control' ] === '' ) ? 'false' : 'true' ) );

        /** Add render rotation attributes */
        $this->add_render_attribute( 'mdp-walker', 'data-rotation', ( $settings[ 'rotation' ] === '' ) ? 'false' : 'true' );
        if ( $settings[ 'rotation' ] === 'yes' ) {
            $this->add_render_attribute( 'mdp-walker', 'data-direction', esc_attr( $settings[ 'rotation_direction' ] ) );
            $this->add_render_attribute( 'mdp-walker', 'data-speed', esc_attr( $settings[ 'rotation_speed' ][ 'size' ] ) );
            $this->add_render_attribute( 'mdp-walker', 'data-click', esc_attr( ( $settings[ 'click' ] === '' ) ? 'false' : 'true' ) );
            if ( $settings[ 'loop' ] === 'yes' ) {
                $this->add_render_attribute( 'mdp-walker', 'data-loop', 'true' );
            } else {
                $this->add_render_attribute( 'mdp-walker', 'data-count', esc_attr( $settings[ 'rotation_count' ] ) );
            }
        }

        /** Prepare render attributes for Side-by-Side layout */
        if ( $settings[ 'side_by_side' ] === 'yes' ) {
            $this->add_render_attribute( 'mdp-walker', 'class', 'mdp-walker-sbs-' . $settings[ 'side_by_side_position' ] );
            $this->add_render_attribute( 'mdp-walker', 'data-map', esc_attr( $settings[ 'map_zoom' ][ 'size' ] ) );
        }

        /** Prepare render attributes for Caption */
        $this->add_render_attribute( 'walker_sub_name', 'class', 'mdp-walker-subheader' );
        $this->add_render_attribute( 'header', 'class', 'mdp-walker-header' );
        $this->add_render_attribute( 'description', 'class', 'mdp-walker-description' );

    }

    /**
     * Render Header
     *
     * @param $settings
     *
     * @since 1.0.0
     * @access private
     */
    private function render_header( $settings ) {

        if ( $settings[ 'show_header' ] === 'yes' || $settings[ 'show_subheader' ] === 'yes' ) {

            /** Header open tag */
            ?><<?php esc_attr_e( $settings[ 'header_tag' ] ) ?>><?php

            /** Display the subheader above the title. */
            if( $settings[ 'show_subheader' ] === 'yes' and $settings[ 'subheader_position' ] === 'top' ) {
                $this->add_inline_editing_attributes( 'walker_sub_name', 'basic' );
                ?><span <?php echo $this->get_render_attribute_string( 'walker_sub_name' ); ?>><?php echo wp_kses_post( $settings[ 'walker_sub_name' ] ); ?></span><?php
            }

            /** Display the header. */
            $this->add_inline_editing_attributes( 'header', 'basic' );
            ?><span <?php echo $this->get_render_attribute_string( 'header' ); ?>><?php echo wp_kses_post( $settings[ 'header' ] ); ?></span><?php


            /** Display the subheader below the title. */
            if($settings[ 'show_subheader' ] === 'yes' and $settings[ 'subheader_position' ] === 'bottom' ) {
                $this->add_inline_editing_attributes( 'walker_sub_name', 'basic' );
                ?><span <?php echo $this->get_render_attribute_string( 'walker_sub_name' ); ?>><?php echo wp_kses_post( $settings[ 'walker_sub_name' ] ); ?></span><?php
            }

            /** Header close tag */
            ?></<?php echo esc_attr( $settings['header_tag'] ) ?>><?php

        }

    }

    /**
     * Render Description
     *
     * @param $settings
     *
     * @since 1.0.0
     * @access private
     */
    private function render_description( $settings ) {

        if ( $settings[ 'show_description' ] !== 'yes' ) {
            return;
        }

        $this->add_inline_editing_attributes( 'description', 'basic' );
        echo '<' . esc_attr( $settings[ 'description_tag' ] ) . ' ' . $this->get_render_attribute_string( 'description' ) . ' >' . wp_kses_post( $settings[ 'description_content' ] ) . '</' . esc_attr( $settings[ 'description_tag' ] ) . '>';

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/tag/walker/';
    }

}
