<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Watcher for Elementor
 * Customizable video player for Elementor editor
 * Exclusively on https://1.envato.market/watcher-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmytro Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Repeater;
use Elementor\Utils;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Watcher - Custom Elementor Widget.
 **/
class watcher_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
         'mdp-selection-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );

        wp_register_style(
            'mdp-watcher-elementor',
                UnityPlugin::get_url() . 'css/watcher-elementor' . UnityPlugin::get_suffix() . '.css',
                    ['mediaelement', 'wp-mediaelement'],
                    UnityPlugin::get_version()
        );

        wp_register_style(
            'mdp-watcher-elementor-jumpforward',
                UnityPlugin::get_url() . 'css/lib/jump-forward' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );

        wp_register_style(
            'mdp-watcher-elementor-playlist',
                UnityPlugin::get_url() . 'css/lib/playlist' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );

        wp_register_style(
            'mdp-watcher-elementor-skip-back',
                UnityPlugin::get_url() . 'css/lib/skip-back' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );

        wp_register_style(
            'mdp-watcher-elementor-speed',
                UnityPlugin::get_url() . 'css/lib/speed' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );

        wp_register_script(
            'mdp-watcher-elementor-playlist',
            UnityPlugin::get_url() . 'js/lib/playlist' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

        wp_register_script(
            'mdp-watcher-elementor-skip-back',
            UnityPlugin::get_url() . 'js/lib/skip-back' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

        wp_register_script(
            'mdp-watcher-elementor-jumpforward',
            UnityPlugin::get_url() . 'js/lib/jump-forward' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

        wp_register_script(
            'mdp-watcher-elementor-speed',
            UnityPlugin::get_url() . 'js/lib/speed' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

	    wp_register_script(
                'mdp-watcher-elementor',
                   UnityPlugin::get_url() . 'js/watcher-elementor' . UnityPlugin::get_suffix() . '.js',
                       [ 'jquery', 'mediaelement' ],
                       UnityPlugin::get_version(),
              true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-watcher-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Video player', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-watcher-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Video player', 'video', 'player', 'audio', 'media', 'youtube', 'vimeo' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [
            'mdp-watcher-elementor',
            'mdp-selection-elementor-admin',
            'mdp-watcher-elementor-jumpforward',
            'mdp-watcher-elementor-playlist',
            'mdp-watcher-elementor-skip-back',
            'mdp-watcher-elementor-speed',
        ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [
            'mdp-watcher-elementor',
            'mdp-watcher-elementor-jumpforward',
            'mdp-watcher-elementor-playlist',
            'mdp-watcher-elementor-skip-back',
            'mdp-watcher-elementor-speed'
        ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_general();

        /** Content -> Playlist Content Section. */
        $this->section_content_playlist();

        /** Content -> Controls Content Section. */
        $this->section_content_controls();

        /** Content -> Video Poster Content Section. */
        $this->section_content_video_poster();

        /** Content -> Video Content Section. */
        $this->section_content_video();


    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Controls Box. */
        $this->section_style_controls_box();

        /** Style -> Section Style Video Container. */
        $this->section_style_video_container();

        /** Style -> Section Style Pause Button. */
        $this->create_button_styles( 'section_style_pause_button', 'Pause button', 'mejs-pause' );

        /** Style -> Section Style Play Button. */
        $this->create_button_styles( 'section_style_play_button', 'Play button', 'mejs-play' );

        /** Style -> Section Style Mute Button. */
        $this->create_button_styles( 'section_style_mute_button', 'Mute button', 'mejs-mute' );

        /** Style -> Section Style Unmute Button. */
        $this->create_button_styles(
            'section_style_unmute_button',
                'Unmute button',
              'mejs-unmute'
        );


        /** Style -> Section Style Previous Button. */
        $this->create_button_styles(
            'section_style_previous_button',
                'Previous button',
              'mejs-prev-button'
        );

        /** Style -> Section Style Next Button. */
        $this->create_button_styles(
            'section_style_next_button',
                'Next button',
              'mejs-next-button'
        );

        /** Style -> Section Style Playlist Button. */
        $this->create_button_styles(
            'section_style_playlist_button',
                'Playlist button',
              'mejs-playlist-button',
          false,
                      [ 'video_source' => 'playlist' ]
        );

        /** Style -> Section Style Skip Back Button. */
        $this->create_button_styles(
            'section_style_skip_back_button',
            'Skip back button',
            'mejs-skip-back-button',
            true
        );

        /** Style -> Section Style Jump Forward Button. */
        $this->create_button_styles(
            'section_style_jump_forward_button',
            'Jump forward button',
            'mejs-jump-forward-button',
            true
        );

        /** Style -> Section Style Speed Button. */
        $this->create_button_styles(
            'section_style_speed_button',
            'Speed button',
            'mejs-speed-button',
            true
        );

        /** Style -> Section Style Fullscreen Button. */
        $this->create_button_styles(
            'section_style_fullscreen_button',
            'Fullscreen button',
            'mejs-fullscreen-button'
        );

        /** Style -> Section Style Loop Button. */
        $this->create_button_styles(
            'section_style_loop_button',
            'Loop button',
            'mejs-loop-button'
        );

        /** Style -> Section Style Video Speed Selector. */
        $this->section_style_video_speed_selector();

        /** Style -> Section Style Video Speed Selector Item. */
        $this->section_style_video_speed_selector_item();

        /** Style -> Section Style Progressbar. */
        $this->section_style_progressbar();

        /** Style -> Section Style Horizontal Volume Bar. */
        $this->section_style_horizontal_volume_bar();

        /** Style -> Section Style Vertical Volume Bar. */
        $this->section_style_vertical_volume_bar();

        /** Style -> Section Style Live Broadcast Text. */
        $this->section_style_live_text();

        /** Style -> Section Style Playlist Box. */
        $this->section_style_playlist_box();

        /** Style -> Section Style Playlist Image. */
        $this->section_style_playlist_video_image();

        /** Style -> Section Style Playlist Video. */
        $this->section_style_playlist_video();

        /** Style -> Section Style Playlist Video Title. */
        $this->create_playlist_text_styles(
         'section_style_playlist_video_title',
             'Playlist video title',
           'mejs-playlist-selector-label'
        );

        /** Style -> Section Style Playlist Video Description. */
        $this->create_playlist_text_styles(
         'section_style_playlist_video_description',
            'Playlist video description',
          'mejs-playlist-item-description'
        );

        /** Style -> Section Style Chapters Container */
        $this->section_style_chapters_container();

        /** Style -> Section Style Chapters Item Container */
        $this->section_style_chapters_item_container();

        /** Style -> Section Style Chapters Title Container */
        $this->section_style_chapter_title();

        /** Style -> Section Style Chapters Time Container */
        $this->section_style_chapter_time();

        /** Style -> Section Style Chapters Separator */
        $this->section_style_chapter_separator();

    }

    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'video_player_alignment',
            [
                'label' => esc_html__( 'Player alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-watcher-elementor-video-wrapper' => 'justify-content: {{VALUE}}',
                ],
                'default' => 'center',
                'toggle' => true,
                'condition' => [
                    'video_source!' => 'playlist'
                ]
            ]
        );

        $this->add_control(
            'video_source',
            [
                'label'     => esc_html__( 'Source', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'self-hosted'    => esc_html__( 'Self hosted', 'selection' ),
                    'external'    => esc_html__( 'External', 'selection' ),
                    'playlist'    => esc_html__( 'Playlist', 'selection' ),
                ],
                'default'   => 'self-hosted',
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'external_video_source_url',
            [
                'label' => esc_html__( 'Source', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'options' => [ 'url' ],
                'default' => [ 'url' => '' ],
                'dynamic' => [
                    'active' => true,
                ],
                'label_block' => true,
                'condition' => [
                    'video_source' => 'external'
                ]
            ]
        );

        $this->add_control(
            'self_hosted_video_url',
            [
                'label' => esc_html__( 'Video', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'media_types' => ['video'],
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'video_source' => 'self-hosted'
                ],
            ]
        );

        $this->add_control(
            'disable_iframe_overlay',
            [
                'label' => esc_html__( 'Disable overlay', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => '',
                'selectors' => [
                    '{{WRAPPER}} .mejs-iframe-overlay' => 'display: none;'
                ],
                'description' => esc_html__( 'Disables overlay on videos with iframe.', 'selection' )
            ]
        );

        $this->add_control(
            'chapters',
            [
                'label' => esc_html__( 'Enable chapters', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => '',
                'condition' => [
                    'video_source!' => 'playlist'
                ]
            ]
        );

        $this->add_control(
            'chapters_content',
            [
                'label' => esc_html__( 'Chapters values', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 5,
                'description' => esc_html__(
                'Type your video chapters in format: title:time(in seconds) and separate chapters with coma',
             'selection'
                ),
                'condition' => [
                    'chapters' => 'yes',
                    'video_source!' => 'playlist'
                ]
            ]
        );

        $this->add_control(
            'chapter_items',
            [
                'label' => esc_html__( 'Chapter items', 'selection' ),
                'type' => Controls_Manager::SELECT2,
                'label_block' => true,
                'multiple' => true,
                'options' => [
                    'time'  => esc_html__( 'Time', 'selection' ),
                    'title' => esc_html__( 'Title', 'selection' ),
                ],
                'condition' => [
                    'chapters' => 'yes',
                    'video_source!' => 'playlist'
                ],
                'default' => [ 'time', 'title' ],
            ]
        );

        $this->add_control(
            'chapters_separator',
            [
                'label' => esc_html__( 'Chapters separator', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your separator here', 'selection' ),
                'condition' => [
                    'chapters' => 'yes',
                    'video_source!' => 'playlist'
                ],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Playlist Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_playlist() {

        $this->start_controls_section('section_content_playlist', [
            'label' => esc_html__('Playlist', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'video_source' => 'playlist'
            ]
        ] );

        $this->add_control(
            'enable_show_playlist',
            [
                'label' => esc_html__( 'Show playlist videos', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
                'description' => esc_html__( 'Display or hide playlist button and box with all playlist videos.', 'selection' )
            ]
        );

        $this->add_control(
            'show_playlist_by_default',
            [
                'label' => esc_html__( 'Show playlist box by default', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => '',
                'condition' => [
                    'enable_show_playlist' => 'yes'
                ]
            ]
        );

        $this->add_responsive_control(
            'video_player_playlist_track_content_alignment',
            [
                'label' => esc_html__( 'Playlist track content alignment', 'audier-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-content label' => 'text-align: {{VALUE}};',
                    '{{WRAPPER}} .mejs-playlist-item-description' => 'text-align: {{VALUE}};'
                ],
                'default' => 'left',
                'toggle' => true
            ]
        );

        $this->add_responsive_control(
            'video_player_playlist_track_image_horizontal_alignment',
            [
                'label' => esc_html__( 'Thumbnail horizontal alignment', 'audier-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail' => 'text-align: {{VALUE}};',
                ],
                'default' => 'left',
                'toggle' => true
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'playlist_video_source',
            [
                'label'     => esc_html__( 'Source', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'self-hosted'    => esc_html__( 'Self hosted', 'selection' ),
                    'external'    => esc_html__( 'External', 'selection' ),
                ],
                'default'   => 'self-hosted',
                'separator' => 'none',
            ]
        );

        $repeater->add_control(
            'playlist_self_hosted_video_url',
            [
                'label' => esc_html__( 'Video', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'media_types' => ['video'],
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'playlist_video_source' => 'self-hosted'
                ],
            ]
        );

        $repeater->add_control(
            'playlist_video_source_url',
            [
                'label' => esc_html__( 'Source', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'options' => [ 'url' ],
                'default' => [ 'url' => '' ],
                'dynamic' => [
                    'active' => true,
                ],
                'label_block' => true,
                'condition' => [
                    'playlist_video_source' => 'external'
                ]
            ]
        );

        $repeater->add_control(
            'playlist_video_title',
            [
                'label' => esc_html__( 'Video title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'playlist_video_description',
            [
                'label' => esc_html__( 'Video description', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'dynamic' => [
                    'active' => true,
                ],
                'rows' => 10,
                'placeholder' => esc_html__( 'Type your description here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'playlist_video_thumbnail',
            [
                'label' => esc_html__( 'Video thumbnail', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        $repeater->add_control(
            'playlist_chapters',
            [
                'label' => esc_html__( 'Enable chapters', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => ''
            ]
        );

        $repeater->add_control(
            'playlist_chapters_content',
            [
                'label' => esc_html__( 'Chapters values', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 5,
                'description' => esc_html__(
                    'Type your video chapters in format: title:time(in seconds) and separate chapters with coma',
                    'selection'
                ),
                'condition' => [
                    'playlist_chapters' => 'yes',
                ]
            ]
        );

        $repeater->add_control(
            'playlist_chapter_items',
            [
                'label' => esc_html__( 'Chapter items', 'selection' ),
                'type' => Controls_Manager::SELECT2,
                'label_block' => true,
                'multiple' => true,
                'options' => [
                    'time'  => esc_html__( 'Time', 'selection' ),
                    'title' => esc_html__( 'Title', 'selection' ),
                ],
                'condition' => [
                    'playlist_chapters' => 'yes'
                ],
                'default' => [ 'time', 'title' ],
            ]
        );


        $repeater->add_control(
            'playlist_chapters_separator',
            [
                'label' => esc_html__( 'Chapters separator', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your separator here', 'selection' ),
                'condition' => [
                    'playlist_chapters' => 'yes'
                ],
            ]
        );


        $this->add_control(
            'playlist_videos',
            [
                'label' => esc_html__( 'Playlist', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'title_field' => '{{{ playlist_video_title }}}',
            ]
        );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Creates player control setting.
     *
     * @param $id
     * @param $repeater
     * @param $title
     * @param $condition
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function create_control_text_setting( $id, $repeater, $title, $condition ) {
         $repeater->add_control(
            'settings_' . $id,
            [
                'label' => esc_html__( $title, 'selection' ),
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
                'condition' => [
                    'video_player_control' => $condition
                ]
            ]
        );
    }

    /**
     * Add widget controls: Content -> Creates switcher player control.
     *
     * @param $id
     * @param $title
     * @param $default
     * @param array $condition
     * @param string $description
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function create_enable_setting( $id, $title, $default, $condition = [], $description = '' ) {
        $control_settings = [
            'label' => esc_html__( $title, 'selection' ),
            'type' => Controls_Manager::SWITCHER,
            'return_value' => 'yes',
            'default' => $default,
        ];

        if ( !empty( $description ) ) {
            $control_settings['description'] = esc_html__( $description, 'selection' );
        }

        if ( !empty( $condition ) ) { $control_settings['condition'] = $condition; }

        $this->add_control( 'enable_' . $id, $control_settings );
    }


    /**
     * Add widget controls: Content -> Controls Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_controls() {

        $this->start_controls_section( 'section_content_controls', [
            'label' => esc_html__('Controls', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $repeater = new Repeater();

        $repeater->add_control(
            'video_player_control',
            [
                'label'     => esc_html__( 'Player control', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'playpause'    => esc_html__( 'Play button', 'selection' ),
                    'progress'    => esc_html__( 'Progress bar', 'selection' ),
                    'duration'    => esc_html__( 'Duration', 'selection' ),
                    'current'     => esc_html__( 'Current time', 'selection' ),
                    'volume'     => esc_html__( 'Volume', 'selection' ),
                    'nexttrack'  => esc_html__( 'Next track', 'selection' ),
                    'prevtrack'  => esc_html__( 'Previous track', 'selection' ),
                    'skipback'  => esc_html__( 'Skip back', 'selection' ),
                    'jumpforward' => esc_html__( 'Jump forward', 'selection' ),
                    'speed'       => esc_html__( 'Speed', 'selection' ),
                    'fullscreen'  => esc_html__( 'Fullscreen', 'selection' ),
                    'loop'       => esc_html__( 'Loop', 'selection' )
                ],
                'default'   => 'playpause',
                'separator' => 'none',
            ]
        );

        $this->create_control_text_setting( 'play_text', $repeater, 'Play button text', 'playpause' );

        $this->create_control_text_setting( 'pause_text', $repeater, 'Pause button text', 'playpause' );

        $this->create_control_text_setting( 'mute_text', $repeater, 'Mute button text', 'volume' );

        $this->create_control_text_setting( 'unmute_text', $repeater, 'Unmute button text', 'volume' );

        $this->create_control_text_setting( 'prev_text', $repeater, 'Previous button text', 'prevtrack' );

        $this->create_control_text_setting( 'next_text', $repeater, 'Next button text', 'nexttrack' );

        $this->create_control_text_setting( 'speed_text', $repeater, 'Speed text', 'speed' );

        $this->create_control_text_setting( 'speed_char', $repeater, 'Speed char', 'speed' );

        $this->create_control_text_setting( 'loop_text', $repeater, 'Loop text', 'loop' );

        $this->create_control_text_setting(
            'fullscreen_text',
                $repeater,
          'Fullscreen text',
      'fullscreen'
        );


        $repeater->add_control(
            'settings_video_volume',
            [
                'label'     => esc_html__( 'Volume bar layout', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'vertical'    => esc_html__( 'Vertical', 'selection' ),
                    'horizontal'    => esc_html__( 'Horizontal', 'selection' )
                ],
                'default'   => 'horizontal',
                'separator' => 'none',
                'condition' => [
                    'video_player_control' => 'volume',
                ]
            ]
        );

        $repeater->add_control(
            'settings_skip_back_interval',
            [
                'label' => esc_html__( 'Skip back interval', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 30
                ],
                'condition' => [
                    'video_player_control' => 'skipback'
                ]
            ]
        );

        $repeater->add_control(
            'settings_jump_forward_interval',
            [
                'label' => esc_html__( 'Jump forward interval', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 30
                ],
                'condition' => [
                    'video_player_control' => 'jumpforward'
                ]
            ]
        );

        $repeater->add_control(
            'settings_default_speed',
            [
                'label' => esc_html__( 'Default speed', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0.1,
                        'max' => 2.0,
                        'step' => 0.1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1.0
                ],
                'condition' => [
                    'video_player_control' => 'speed'
                ]
            ]
        );

        $repeater->add_control(
            'settings_speeds',
            [
                'label' => esc_html__( 'Speeds', 'selection' ),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => [
                    '0.75'  => esc_html__( '0.75', 'selection' ),
                    '1.25' => esc_html__( '1.25', 'selection' ),
                    '1.50' => esc_html__( '1.50', 'selection' ),
                ],
                'default' => [ '0.75', '1.00', '1.25', '1.50' ],
                'condition' => [
                    'video_player_control' => 'speed'
                ]
            ]
        );

        $this->add_control(
            'video_player_controls',
            [
                'label' => esc_html__( 'Player controls', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'video_player_control' => esc_html__( 'playpause' )
                    ],
                    [
                        'video_player_control' => esc_html__( 'current' )
                    ],
                    [
                        'video_player_control' => esc_html__( 'progress' )
                    ],
                    [
                        'video_player_control' => esc_html__( 'duration' )
                    ],
                    [
                        'video_player_control' => esc_html__( 'volume' )
                    ],
                ],
                'title_field' => 'Control - {{{ video_player_control }}}',
            ]
        );

        $this->create_enable_setting( 'enable_keyboard', 'Keyboard support', 'yes' );

        $this->create_enable_setting(
                'hide_volume_on_touch_devices',
               'Hide volume on touch devices',
             'yes'
        );

        $this->create_enable_setting( 'click_to_play_pause', 'Click to play/pause', 'yes' );

        $this->create_enable_setting( 'always_show_controls', 'Always show controls', 'no' );

        $this->create_enable_setting(
            'hide_video_controls_on_load',
           'Hide controls on load',
         'no',
                [ 'enable_always_show_controls!' => 'yes' ]
        );

        $this->create_enable_setting(
            'hide_video_controls_on_pause',
           'Hide controls on pause',
         'no',
                [ 'enable_always_show_controls!' => 'yes' ]
        );

        $this->add_control(
            'settings_controls_timeout_default',
            [
                'label' => esc_html__( 'Hide controls after(ms)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 10000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1500
                ],
                'condition' => [
                    'enable_always_show_controls!' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'settings_controls_timeout_mouse_enter',
            [
                'label' => esc_html__( 'Hide controls mouse enter after(ms)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 10000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 2500
                ],
                'condition' => [
                    'enable_always_show_controls!' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'settings_controls_timeout_mouse_leave',
            [
                'label' => esc_html__( 'Hide controls mouse leave after(ms)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 10000,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1000
                ],
                'condition' => [
                    'enable_always_show_controls!' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'settings_start_volume',
            [
                'label' => esc_html__( 'Initial volume', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1,
                        'step' => 0.1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0.8
                ]
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Video Poster Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_video_poster() {
        $this->start_controls_section( 'section_content_video_poster', [
            'label' => esc_html__('Video poster', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'settings_poster',
            [
                'label' => esc_html__( 'Video poster', 'selection' ),
                'type' => Controls_Manager::MEDIA
            ]
        );

        $this->create_enable_setting( 'show_poster_when_ended', 'Show poster when ended', 'no' );

        $this->create_enable_setting( 'show_poster_when_paused', 'Show poster when paused', 'no' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Video Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_video() {

        $this->start_controls_section( 'section_content_audio', [
            'label' => esc_html__('Video', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'settings_video_height',
            [
                'label' => esc_html__( 'Video height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'description' => esc_html__( 'Height of the video in pixels', 'selection' )
            ]
        );

        $this->add_control(
            'settings_video_width',
            [
                'label' => esc_html__( 'Video width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'description' => esc_html__( 'Width of the video in pixels', 'selection' )
            ]
        );

        $this->add_control(
            'player_muted',
            [
                'label' => esc_html__( 'Muted', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'enable_loop',
            [
                'label' => esc_html__( 'Loop', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'video_autoplay',
            [
                'label' => esc_html__( 'Autoplay', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
                'description' => esc_html__(
                    'This setting will be working only if video is muted',
                  'selection'
                )
            ]
        );

        $this->create_enable_setting( 'always_show_hours', 'Always show hours', 'no' );

        $this->create_enable_setting( 'pause_other_players', 'Pause other players', 'yes' );

        $this->create_enable_setting(
            'force_live',
           'Force live',
         'no',
                [],
      'If enabled, the Live Broadcast message will be displayed and progress bar will be hidden.'
        );

        $this->create_enable_setting(
            'enable_autosize',
           'Autosize',
         'no',
                [],
      'Resize to media dimensions.'
        );

        $this->create_enable_setting(
            'use_smooth_hover',
           'Smooth hover',
         'yes',
                [],
      'Enable smooth behavior when hovering progress bar'
        );

        $this->end_controls_section();
    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_additional_colors' => array_key_exists( 'include_additional_colors', $opts ) ?
                $opts['include_additional_colors'] : false,
            'include_hover_additional_colors' => array_key_exists( 'include_hover_additional_colors', $opts ) ?
                $opts['include_hover_additional_colors'] : false,
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_border' => array_key_exists( 'include_border', $opts ) ?
                $opts['include_border'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_bg'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_background' => array_key_exists( 'additional_background', $opts ) ?
                $opts['additional_background'] : false,
            'typography_name' => array_key_exists( 'typography_name', $opts ) ?
                $opts['typography_name'] : 'Typography',
            'additional_background_name' => array_key_exists( 'additional_background_name', $opts ) ?
                $opts['additional_background_name'] : '',
            'additional_background_class' => array_key_exists( 'additional_background_class', $opts ) ?
                $opts['additional_background_class'] : '',
            'additional_border_radius_class' => array_key_exists( 'additional_border_radius_class', $opts ) ?
                $opts['additional_border_radius_class'] : '',
            'additional_border_radius_hover_class' => array_key_exists( 'additional_border_radius_hover_class', $opts ) ?
                $opts['additional_border_radius_hover_class'] : '',
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'FOCUS',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ?
                $opts['additional_color_hover_class'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3,
            'additional_colors' => array_key_exists( 'additional_colors', $opts ) ?
                $opts['additional_colors'] : '',
            'hover_additional_colors' => array_key_exists( 'hover_additional_colors', $opts ) ?
                $opts['hover_additional_colors'] : [],
            'additional_bg_selector' => array_key_exists( 'additional_bg_selector', $opts ) ?
                $opts['additional_bg_selector'] : '',
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__( $style_opts['typography_name'], 'selection' ),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_additional_colors'] && is_array( $style_opts['additional_colors'] ) ) {
            foreach ( $style_opts['additional_colors'] as $additional_color ) {
                $this->add_control(
                    $section_id . '_' . $additional_color['name'] . '_normal_text_color',
                    [
                        'label' => esc_html__( $additional_color['name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'selectors' => [
                            "{{WRAPPER}} .".$additional_color['class'] => $additional_color['value'],
                        ],
                    ]
                );
            }
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'] . $style_opts['additional_bg_selector'],
                ]
            );

        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'] => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_normal',
                    'label' => esc_html__('Border Type', 'selection'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'],
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['include_hover_additional_colors'] && is_array( $style_opts['hover_additional_colors'] ) ) {
            foreach ( $style_opts['hover_additional_colors'] as $additional_color ) {
                $this->add_control(
                    $section_id . '_' . $additional_color['name'] . '_hover_text_color',
                    [
                        'label' => esc_html__( $additional_color['name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'selectors' => [
                            "{{WRAPPER}} .".$additional_color['class'] => $additional_color['value'],
                        ],
                    ]
                );
            }
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'] . ":hover" . $style_opts['additional_bg_selector'] . "hover",
                ]
            );
        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'].':hover' => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }


        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_hover',
                    'label' => esc_html__('Border Type', 'selection'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'] . ":hover",
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'selection' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'watcher-elementor '),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'selection' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            if ( $style_opts['include_border'] ) {

                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => $section_id . '_border_active',
                        'label' => esc_html__('Border Type', 'selection'),
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );

            }

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Create button style settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_button_styles( $section_id, $name, $selector, $include_color_typography = false, $condition = [] ) {
        $section_settings =  [
            'label' => esc_html__( $name, 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ];

        if ( !empty( $condition ) ) {
            $section_settings['condition'] = $condition;
        }

        $this->start_controls_section( $section_id, $section_settings );


        $this->add_responsive_control(
            $section_id . '_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} ." . $selector => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $typography_tabs_controls_options = [
            'html_class' => $selector,
            'include_typography' => false,
            'include_color' => false
        ];

        if ( $include_color_typography ) {
            $typography_tabs_controls_options['include_typography'] = true;
            $typography_tabs_controls_options['typography_class'] = $selector . ' button';
            $typography_tabs_controls_options['include_color'] = true;
            $typography_tabs_controls_options['color_class'] = $selector . ' button';
            $typography_tabs_controls_options['color_hover_class'] = $selector . ':hover button';
        }

        $this->generate_typography_tabs_controls( $section_id, $typography_tabs_controls_options );

        $this->end_controls_section();
    }


    /**
     * Add widget controls: Style -> Section Style Live Text.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_live_text() {

        $this->start_controls_section( 'section_style_live_text', [
            'label' => esc_html__( 'Live broadcast text', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'enable_force_live' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_live_text',
            'mejs-broadcast'
        );


        $this->generate_typography_tabs_controls( 'section_style_live_text', [
            'html_class' => 'mejs-volume-slider',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mejs-broadcast',
            'color_class' => 'mejs-broadcast',
            'color_hover_class' => 'mejs-broadcast:hover'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Controls Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_controls_box() {

        $this->start_controls_section( 'section_style_controls_box', [
            'label' => esc_html__( 'Controls box', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
        ] );

        $this->add_responsive_control(
            'section_style_controls_box_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .mejs-controls" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_controls_box', [
            'html_class' => 'mejs-controls',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Controls Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_video_container() {

        $this->start_controls_section( 'section_style_video_container', [
            'label' => esc_html__( 'Video container', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_typography_tabs_controls( 'section_style_video_container', [
            'html_class' => 'mejs-container',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Progressbar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_progressbar() {

        $this->start_controls_section( 'section_style_progressbar', [
            'label' => esc_html__( 'Progress bar', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );


        $this->add_responsive_control(
            'section_style_progressbar_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mejs-time-slider" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_progressbar', [
            'html_class' => 'mejs-time-slider',
            'include_typography' => false,
            'include_color' => false,
            'include_additional_colors' => true,
            'include_hover_additional_colors' => true,
            'include_bg' => false,
            'additional_transition_selector' => '{{WRAPPER}} .mejs-time-current, {{WRAPPER}} .mejs-time-total',
            'additional_colors' => [
                [
                    'name' => 'Time loaded bar',
                    'class' => 'mejs-time-loaded',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time total bar',
                    'class' => 'mejs-time-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time current bar',
                    'class' => 'mejs-time-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ],
            'hover_additional_colors' => [
                [
                    'name' => 'Time loaded bar',
                    'class' => 'mejs-time-rail:hover .mejs-time-loaded',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time total bar',
                    'class' => 'mejs-time-rail:hover .mejs-time-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time current bar',
                    'class' => 'mejs-time-rail:hover .mejs-time-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ]
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Horizontal Volume Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_horizontal_volume_bar() {

        $this->start_controls_section( 'section_style_horizontal_volume_bar', [
            'label' => esc_html__( 'Horizontal volume bar', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_horizontal_volume_bar_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mejs-horizontal-volume-total" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_horizontal_volume_bar', [
            'html_class' => 'mejs-horizontal-volume-total',
            'include_typography' => false,
            'include_color' => false,
            'include_additional_colors' => true,
            'include_hover_additional_colors' => true,
            'include_bg' => false,
            'additional_transition_selector' => '{{WRAPPER}} .mejs-horizontal-volume-current, {{WRAPPER}} .mejs-horizontal-volume-total',
            'additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-horizontal-volume-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-horizontal-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ],
            'hover_additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-horizontal-volume-total:hover',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-horizontal-volume-total:hover .mejs-horizontal-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ]
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Vertical Volume Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_vertical_volume_bar() {

        $this->start_controls_section( 'section_style_vertical_volume_bar', [
            'label' => esc_html__( 'Vertical volume bar', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_vertical_volume_bar_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mejs-volume-slider" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_vertical_volume_bar', [
            'html_class' => 'mejs-volume-slider',
            'include_typography' => false,
            'include_color' => false,
            'include_additional_colors' => true,
            'include_hover_additional_colors' => true,
            'additional_transition_selector' => '{{WRAPPER}} .mejs-volume-current, {{WRAPPER}} .mejs-volume-total',
            'additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-volume-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ],
            'hover_additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-volume-total:hover',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-volume-total:hover .mejs-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ]
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Playlist Video.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_video() {

        $this->start_controls_section( 'section_style_playlist_video', [
            'label' => esc_html__( 'Playlist video', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'video_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_video',
            'mejs-playlist-item-inner'
        );

        $this->generate_typography_tabs_controls( 'section_style_playlist_video', [
            'html_class' => 'mejs-playlist-selector-list-item',
            'include_typography' => false,
            'include_color' => false,
            'additional_bg_selector' => ', {{WRAPPER}} .mejs-playlist-selected'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Video Speed Selector.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_video_speed_selector() {

        $this->start_controls_section( 'section_style_video_speed_selector', [
            'label' => esc_html__( 'Video speed selector', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_video_speed_selector',
            'mejs-speed-selector'
        );

        $this->generate_typography_tabs_controls( 'section_style_video_speed_selector', [
            'html_class' => 'mejs-speed-selector',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Video Speed Selector Item.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_video_speed_selector_item() {

        $this->start_controls_section( 'section_style_video_speed_selector_item', [
            'label' => esc_html__( 'Video speed selector item', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_video_speed_selector_item',
            'mejs-speed-selector-list-item'
        );

        $this->generate_typography_tabs_controls( 'section_style_video_speed_selector_item', [
            'html_class' => 'mejs-speed-selector-list-item',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mejs-speed-selector-list-item label',
            'color_class' => 'mejs-speed-selector-list-item label',
            'color_hover_class' => 'mejs-speed-selector-list-item:hover label',
        ] );

        $this->end_controls_section();

    }


    /**
     * Creates playlist text styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_playlist_text_styles( $section_id, $name, $selector ) {

        $this->start_controls_section( $section_id, [
            'label' => esc_html__( $name, 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'video_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            $section_id,
            $selector
        );

        $this->generate_typography_tabs_controls( $section_id, [
            'html_class' => $selector,
            'typography_class' => $selector,
            'color_class' => $selector,
            'color_hover_class' => 'mejs-playlist-item-inner:hover .' . $selector,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Video Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_video_image() {

        $this->start_controls_section( 'section_style_playlist_video_image', [
            'label' => esc_html__( 'Playlist video image', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'video_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_video_image',
            'mejs-playlist-item-thumbnail img'
        );

        $this->add_control(
            'video_playlist_track_image_object_fit',
            [
                'label'     => esc_html__( 'Object fit', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'fill'    => esc_html__( 'Fill', 'audier-elementor' ),
                    'contain'    => esc_html__( 'Contain', 'audier-elementor' ),
                    'cover'    => esc_html__( 'Cover', 'audier-elementor' ),
                    'scale-down'    => esc_html__( 'Scale down', 'audier-elementor' ),
                    'none'    => esc_html__( 'None', 'audier-elementor' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail img' => 'object-fit: {{VALUE}};'
                ],
                'default'   => 'fill',
                'separator' => 'none',
            ]
        );

        $this->add_responsive_control(
            'section_style_playlist_video_image_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail img' => 'width: {{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'section_style_playlist_video_image_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail img' => 'height: {{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_playlist_video_image', [
            'html_class' => 'mejs-playlist-item-thumbnail img',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_box() {

        $this->start_controls_section( 'section_style_playlist_box', [
            'label' => esc_html__( 'Playlist box', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'video_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_box',
            'mejs-playlist-selector'
        );

        $this->add_responsive_control(
            'section_style_playlist_box_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-layer' => 'width: {{SIZE}}{{UNIT}} !important;'
                ]
            ]
        );


        $this->add_responsive_control(
            'section_style_playlist_box_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-layer' => 'height: {{SIZE}}{{UNIT}} !important;'
                ]
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_playlist_box', [
            'html_class' => 'mejs-playlist-selector',
            'include_typography' => false,
            'include_color' => false
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Chapters Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_chapters_container() {

        $this->start_controls_section( 'section_style_chapters_container', [
            'label' => esc_html__( 'Chapters container', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'chapters_alignment',
            [
                'label' => esc_html__( 'Chapters alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-vertical' => 'align-items: {{VALUE}}',
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-horizontal' => 'justify-content: {{VALUE}}',
                ],
                'default' => 'flex-start',
                'toggle' => true
            ]
        );

        $this->add_control(
            'chapters_style_layout',
            [
                'label'     => esc_html__( 'Chapters layout', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'vertical'    => esc_html__( 'Vertical', 'selection' ),
                    'horizontal'    => esc_html__( 'Horizontal', 'selection' ),
                ],
                'default'   => 'horizontal',
                'separator' => 'after',
            ]
        );

        $this->generate_margin_padding_controls(
            'section_style_chapters_container',
            'chapters_space_between_items'
        );

        $this->add_responsive_control(
            'chapters_space_between_items',
            [
                'label' => esc_html__( 'Space between items', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'size' => 10,
                    'unit' => 'px'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-horizontal .mdp-watcher-elementor-chapter-block' => 'margin-left: {{SIZE}}{{UNIT}}; margin-right: {{SIZE}}{{UNIT}}',
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-horizontal .mdp-watcher-elementor-chapter-block:first-of-type' => 'margin-left: 0;',
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-horizontal .mdp-watcher-elementor-chapter-block:last-of-type' => 'margin-right: 0;',
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-vertical .mdp-watcher-elementor-chapter-block' => 'margin-bottom: {{SIZE}}{{UNIT}}; margin-top: {{SIZE}}{{UNIT}}',
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-vertical .mdp-watcher-elementor-chapter-block:first-of-type' => 'margin-top: 0;',
                    '{{WRAPPER}} .mdp-watcher-elementor-chapters-layout-vertical .mdp-watcher-elementor-chapter-block:last-of-type' => 'margin-bottom: 0;'

                ]
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_chapters_container', [
            'html_class' => 'mdp-watcher-elementor-chapters-wrapper',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Chapters Item Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_chapters_item_container() {

        $this->start_controls_section( 'section_style_chapters_item_container', [
            'label' => esc_html__( 'Chapters item container', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_chapters_item_container',
            'mdp-watcher-elementor-chapter-block'
        );

        $this->generate_typography_tabs_controls( 'section_style_chapters_item_container', [
            'html_class' => 'mdp-watcher-elementor-chapter-block',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Chapter Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_chapter_title() {

        $this->start_controls_section( 'section_style_chapter_title', [
            'label' => esc_html__( 'Chapter title', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_chapter_title',
            'mdp-watcher-elementor-chapter-title'
        );


        $this->generate_typography_tabs_controls( 'section_style_chapter_title', [
            'html_class' => 'mdp-watcher-elementor-chapter-title',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-watcher-elementor-chapter-title',
            'color_class' => 'mdp-watcher-elementor-chapter-title',
            'color_hover_class' => 'mdp-watcher-elementor-chapter-block:hover'
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Chapter Time.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_chapter_time() {

        $this->start_controls_section( 'section_style_chapter_time', [
            'label' => esc_html__( 'Chapter time', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_chapter_time',
            'mdp-watcher-elementor-chapter-time'
        );

        $this->generate_typography_tabs_controls( 'section_style_chapter_time', [
            'html_class' => 'mdp-watcher-elementor-chapter-time',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-watcher-elementor-chapter-time',
            'color_class' => 'mdp-watcher-elementor-chapter-time',
            'color_hover_class' => 'mdp-watcher-elementor-chapter-block:hover'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Chapter Separator.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_chapter_separator() {

        $this->start_controls_section( 'section_style_chapter_separator', [
            'label' => esc_html__( 'Chapter separator', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_chapter_separator',
            'mdp-watcher-elementor-chapter-separator'
        );

        $this->generate_typography_tabs_controls( 'section_style_chapter_separator', [
            'html_class' => 'mdp-watcher-elementor-chapter-separator',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-watcher-elementor-chapter-separator',
            'color_class' => 'mdp-watcher-elementor-chapter-separator',
            'color_hover_class' => 'mdp-watcher-elementor-chapter-separator:hover'
        ] );

        $this->end_controls_section();

    }


    /**
     * Create video player.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_video_player( $settings ) {
        $video_url = $settings['video_source'] === 'self-hosted' ?
            esc_url( $settings['self_hosted_video_url']['url'] ) :
            esc_url( $settings['external_video_source_url']['url'] );
        $is_editor = Plugin::instance()->editor->is_edit_mode();
        $admin_error_message = $is_editor ?
            esc_html__( 'You need to add source of your video to display player!', 'selection' ) :
            '';

        return !empty( $video_url ) ? sprintf(
            '<div class="mdp-watcher-elementor-video-wrapper">
                        <video %s id="mdp-watcher-elementor-player-%s" width="100%%" %s>
                            <source src="%s">
                        </video>
                    </div>',
     $settings['video_autoplay'] === 'yes' ? 'autoplay' : '',
            $this->get_id(),
            $settings['player_muted'] === 'yes' ? 'muted' : '',
            $video_url

        ) : $admin_error_message;
    }

    /**
     * Create video player controls settings.
     *
     * @return array
     * @since 1.0.0
     * @access private
     *
     */
    private function get_video_player_controls( $settings ) {
        $settings_controls = $settings['video_player_controls'];
        $controls = [];

        foreach ( $settings_controls as $settings_control ) {
            $controls[] = esc_attr( $settings_control['video_player_control'] );
        }


        return array_values( array_unique( $controls ) );

    }

    /**
     * Converts string to camel case.
     *
     * @param $string
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function to_camel_case( $string ) {

        $str = str_replace( ' ', '', ucwords( str_replace( '_', ' ', $string ) ) );

        $str[0] = strtolower( $str[0] );

        return $str;
    }

    /**
     * Get boolean settings for video player.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_video_player_boolean_settings( $settings ) {
        $player_settings = [];

        foreach ( $settings as $key => $setting ) {
            /** If control id has prefix "enbale_" add to player settings array  */
            if ( strpos( $key, 'enable' ) !== false ) {
                $key_parts = explode( '_', $key, 2 );
                $player_settings[$this->to_camel_case( $key_parts[1] )] = $setting;
            }
        }

        return $player_settings;

    }

    private function get_array_item_value( $setting ) {

        if ( isset( $setting['size'] ) ) {
            $array_value = $setting['size'];
        } elseif ( isset( $setting['url'] ) ) {
            $array_value = $setting['url'];
        } else {
            $array_value = $setting;
        }

        return $array_value;
    }

    private function get_video_player_setting( $setting ) {

        if ( is_array( $setting ) ) {
            $player_setting = $this->get_array_item_value( $setting );
        } else {
            $player_setting = $setting;
        }

        return $player_setting;
    }

    /**
     * Get settings for video player.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_video_player_settings( $settings ) {
        $player_settings = [];
        $controls_settings = $settings['video_player_controls'];

        /** Set controls settings. */
        foreach ( $controls_settings as $controls_setting ) {
            foreach ( $controls_setting as $key => $controls_setting_item ) {
                /** If control id has prefix "settings_" add to player settings array  */
                if ( strpos( $key, 'settings' ) !== false ) {
                    $key_parts = explode( '_', $key, 2 );
                    if ( empty( $controls_setting_item ) ) { continue; }
                    $player_settings[$this->to_camel_case( $key_parts[1] )] =
                        $this->get_video_player_setting( $controls_setting_item );
                }
            }
        }

        /** Set other settings. */
        foreach ( $settings as $key => $setting ) {
            if ( strpos( $key, 'settings' ) !== false ) {
                $key_parts = explode( '_', $key, 2 );
                if ( empty( $setting ) ) { continue; }
                $player_settings[$this->to_camel_case( $key_parts[1] )] = $this->get_video_player_setting( $setting );
            }
        }

        return $player_settings;

    }


    /**
     * Create playlist videos.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_playlist_videos( $playlist ) {
        $playlist_videos = [];

        foreach ( $playlist as $playlist_item ) {
            $video_src = $playlist_item['playlist_video_source'] === 'self-hosted' ?
                esc_url( $playlist_item['playlist_self_hosted_video_url']['url'] ) :
                esc_url( $playlist_item['playlist_video_source_url']['url'] );
            $video_thumbnail = !empty( $playlist_item['playlist_video_thumbnail']['url'] ) ?
                sprintf(
                    'data-playlist-thumbnail="%s"',
                    esc_url( $playlist_item['playlist_video_thumbnail']['url'] )
                ) : '';
            $video_title = !empty( $playlist_item['playlist_video_title'] ) ?
                sprintf( 'title="%s"', esc_attr( $playlist_item['playlist_video_title'] ) ) : '';
            $video_description = !empty( $playlist_item['playlist_video_description'] ) ?
                sprintf(
                    'data-playlist-description="%s"',
                    esc_attr( $playlist_item['playlist_video_description'] )
                ) : '';


            if ( !empty( $video_src ) && !empty( $video_title ) ) {
                $playlist_videos[] = sprintf(
                '<source src="%s" %s %s %s >',
                        $video_src,
                        $video_title,
                        $video_thumbnail,
                        $video_description
                );
            }
        }

        return count( $playlist_videos ) >= 2 ? implode( ' ', $playlist_videos ) : '';

    }

    /**
     * Create playlist.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_playlist( $settings ) {
        $playlist_videos = $this->create_playlist_videos( $settings['playlist_videos'] );
        $is_editor = Plugin::instance()->editor->is_edit_mode();
        $admin_error_message = $is_editor ?
            esc_html__(
                'You need to add at least two videos with source and title!',
                'selection'
            ) : '';

        return !empty( $playlist_videos ) ? sprintf(
    '<video 
               %s 
               id="mdp-watcher-elementor-player-%s" 
               class="mdp-watcher-elementor-player" 
               width="100%%"
               %s >
               %s
           </video>',
     $settings['video_autoplay'] === 'yes' ? 'autoplay' : '',
            $this->get_id(),
            $settings['player_muted'] === 'yes' ? 'muted' : '',
            $playlist_videos
        ) : $admin_error_message;
    }

    /**
     * Creates chapters for playlist videos.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_playlist_chapters( $playlist, $layout ) {
        $playlist_chapters = '';

        foreach ( $playlist as $index => $playlist_item ) {
            $playlist_chapters .= $this->create_chapters(
                $playlist_item['playlist_chapters'] === 'yes',
                               $playlist_item['playlist_chapters_content'],
                               $playlist_item['playlist_chapter_items'],
                               $playlist_item['playlist_chapters_separator'],
                               $layout,
                               $index
            );
        }

        return $playlist_chapters;
    }

    /**
     * Creates chapters time block.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chapter_time( $time ) {
        return sprintf(
     '<span class="mdp-watcher-elementor-chapter-time">%s</span>',
            date( 'H:i:s', (float)esc_attr( $time ) )
        );
    }

    /**
     * Creates chapters title block.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chapter_title( $title ) {
        return sprintf( '<span class="mdp-watcher-elementor-chapter-title">%s</span>', esc_html( $title ) );
    }

    /**
     * Creates chapters for video.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chapters( $chapters_enabled, $chapters, $items, $separator, $layout, $index = '' ) {

        if ( !$chapters_enabled ) { return ''; }

        $chapters_items = explode( ',', $chapters );
        $chapters_html = '';
        $chapter_separator = sprintf(
     '<span class="mdp-watcher-elementor-chapter-separator">%s</span>',
            $separator
        );

        foreach ( $chapters_items as $chapters_item ) {
            $chapters_item = explode( ':', $chapters_item );
            $chapters_html .= sprintf(
                '<div class="mdp-watcher-elementor-chapter-block" data-chapter-time="%s"> %s %s </div> %s',
                        esc_attr( $chapters_item[1] ),
                 in_array( 'title', $items ) ? $this->create_chapter_title( $chapters_item[0] ) : '',
                        in_array( 'time', $items ) ? $this->create_chapter_time( $chapters_item[1] ) : '',
                        !empty( $chapter_separator ) ? $chapter_separator : ''
            );
        }

        return sprintf(
            '<div class="mdp-watcher-elementor-chapters-wrapper mdp-watcher-elementor-chapters-layout-%s" %s>
                        %s
                    </div>',
                    esc_attr( $layout ),
             $index !== '' ? 'data-chapter-index="' . $index . '"' : '',
                    $chapters_html
        );

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        $player_controls = json_encode( $this->get_video_player_controls( $settings ) );
        $player_bool_settings = json_encode( $this->get_video_player_boolean_settings( $settings ) );
        $video_source = esc_attr( $settings['video_source'] );
        $player_settings = json_encode( $this->get_video_player_settings( $settings ) );
        $video_chapters = $video_source === 'playlist' ?
            $this->create_playlist_chapters( $settings['playlist_videos'], $settings['chapters_style_layout'] ) :
            $this->create_chapters(
 $settings['chapters'] === 'yes',
                $settings['chapters_content'],
                $settings['chapter_items'],
                $settings['chapters_separator'],
                $settings['chapters_style_layout']
            );

        $not_playlist_class = $video_source !== 'playlist' ? 'mdp-watcher-elementor-not-playlist' : '';

        echo sprintf(
        '<!-- Start Watcher for Elementor WordPress Plugin -->
                <div 
                    class="mdp-watcher-elementor-box %s"
                    data-controls="%s"
                    data-player-bool-settings="%s"
                    data-player-settings="%s"
                    data-source="%s"
                    data-show-playlist-by-default="%s"
                >%s %s</div>
               <!-- End Watcher for Elementor WordPress Plugin -->',
                $not_playlist_class,
                esc_attr( $player_controls ),
                esc_attr( $player_bool_settings ),
                esc_attr( $player_settings ),
                $video_source,
                $video_source === 'playlist' ? esc_attr( $settings['show_playlist_by_default'] ) : '',
                $video_source !== 'playlist' ?
                    $this->create_video_player( $settings ) :
                    $this->create_playlist( $settings ),
                $video_chapters
        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/watcher';

    }

}
