<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

use Merkulove\Selection\Unity\UI;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/**
 * SINGLETON: The class used to implement meta-boxes UI.
 * @since 1.0.0
 **/
final class MetaBoxUI {

    /**
     * The one true MetaBoxUI.
     *
     * @var MetaBoxUI
     * @since 1.0.0
     **/
    private static $instance;

    /**
     * @param        $message
     * @param        $key
     * @param        $options
     * @param        $default
     * @param        $label
     * @param        $helper_text
     * @param string $method
     **/
    public function render_select( $message, $key, $options, $default, $label, $helper_text, $method = 'render_select' ) {

        /** Get value from post meta. */
        $value = get_post_meta( $message->ID, $key, true );
        $value = ( ! empty( $value ) ) ? $value : $default;

        /** Render select from UI. */
        UI::get_instance()->$method(
            $options,
            $value,
            $label,
            $helper_text,
            [
                'id' => $key,
                'name' => $key,
            ]
        );

    }

    /**
     * Main MetaBoxUI Instance.
     *
     * Insures that only one instance of MetaBoxUI exists in memory at any one time.
     *
     * @static
     * @since 1.0.0
     *
     * @return MetaBoxUI
     **/
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

            self::$instance = new self;

        }

        return self::$instance;

    }

} // End Class MetaBoxUI.
